
public class ControlObject extends GameObject implements iact_Appear
{
    private Vector m_vVolPosition = new Vector();
    private Vector m_vVolSpeed = new Vector();
    private Vector m_vVolLocalSpeed = new Vector();
    private Vector m_vVolForward = new Vector();
    private Vector m_vVolRight = new Vector();
    private Vector m_vVolUp = new Vector();
    
    /** [native] Ustawienie flag obiektu
        @see EFlags.
     */
    public native void SetFlags(int iFlags);
        
    /** [native] Ustawienie na 1 podanej flagi obiektu
        @see EFlags.
     */
    public native void SetFlag(int iFlag);
    
    /** [native] Ustawienie na 0 podanej flagi obiektu
        @see EFlags.
     */
    public native void ClearFlag(int iFlag);
    /** [native] Pobranie wartosci wszystkich flag
        @see EFlags.
     */
    public native int GetFlags();
    
    /** [native] Sprawdza czy podana flaga jest ustawiona tzn. =1
        @see EFlags.
     */
    public native boolean IsSetFlag(int iFlag);
    
    /** [native] Sprawdza czy podana flaga jest wylaczona tzn. =0
        @see EFlags.
     */
    public native boolean IsClearFlag(int iFlag);
    
    /** [native] Sprawdza czy podany obiekt jest obiektem niezaleznym od grida
     */
    public native boolean IsGridIndependent();
    
    /**[native] Ustawia typ fizyki.
	 * @param nPhysics typ fizyki
     * @see EPhysics.
    */
	public native void SetPhysicsType( int nPhysics );
    
    /**[native] Ustawia typ fizyki.
	 * @return typ fizyki
     * @see EPhysics.
    */
	public native int GetPhysicsType();
    
    /**[native] Ustawia czy fizyka jest aktywna i ma byc automatycznie
     * updateowana w CLevel::Frame().
	 * @param bActive zmienna ustawiajca czy fizyka ma byc aktywna czy nie
	 * uwaga:
	 * jesli obiekt nie ma ustawionego zadnego trybu fizyki to
	 * zawolanie metody nie odniesie zadnego skutku
	 * */
	public native void SetPhysicsActive( boolean bActive );
    
    /**[native] Pobiera stan fizyki.
	 * @return zmienna logiczna mowiaca czy fizyka jest aktywna czy nie (<code>boolean</code>)*/
    public native boolean GetPhysicsActive();
    
    /**[native] Ustawia fizyke na aktywna do automatycznego
     * updateowania w CLevel::Frame().
	 * @param fTime czas z ktorym zostanie zainicjowana fizyka dla obiektu
	 * uwaga:
	 * jesli obiekt nie ma ustawionego zadnego trybu fizyki to
	 * zawolanie metody nie odniesie zadnego skutku
	 * */
	public native void SetPhysicsActive( float fTime );
    
    /**
     * Ustawia fizyke w tryb pracy uproszczonej
     * uzywane w sieci do wylaczania stalego ziarna
     */
    public native void EnablePhysicsSimplifySimulate();
    
    /**[native] Ustawia maksymalna predkosc obrotowa.
	 * @param fSpeed maksymalna predkosc obrotowa w stopniach/s (<code>float</code>)*/
    public native void SetMaxAngSpeed( float fSpeed );
    
    /**[native] Ustawia maksymalny kat wspinania przy chodzeniu.
     * Poza tym wplywa na zeslizgiwanie sie obiektu po pochylosci w GRAVITYWALK, jesli
     * nie moge podejsc, to sie zeslizguje.
	 * @param fDeg kat w stopniach (0 oznacza, ze mozna sie wspinac pionowo)*/
    public native void SetMaxClimbAngle( float fDeg );
    
	/**[native] Ustawia pozycje obiektu. Jezeli obiekt jest przylinkowany do innego,
	 * to ustawia pozycje w ukladzie parenta.
	 * @param v nowa pozycja obiektu (<code>Vector</code>)*/
    public native void SetPosition(Vector v);
	
	/**[native] Ustawia pozycje obiektu w swiecie.
	 * @param v nowa pozycja obiektu (<code>Vector</code>)*/
    public native void SetPositionWorld(Vector v);
	
	/**[native] Ustawia scale obiektu
	 * @param scale nowa scala obiektu (<code>Vector</code>)
	 */
	public native void SetScale( Vector scale );

    public void SetScale(float fX, float fY, float fZ)
    {
        SetScale(new Vector(fX, fY, fZ));
    }
    
    /**[native] Pobiera skale obiektu
     * @return skala obiektu
     */
    public native Vector GetScale();
    
	/**[native] Odwraca obiekt w lokalnej osi Y o 180 stopni poprzez obrot obiektu
	 * wokol osi X lub Z
	 * @param bFlipByZ obrot wokol X gdy false, wokol Z gdy true
	 */
	public native void FlipYLocal( boolean bFlipByZ );
	
	/**[native] Obraca obiekt wzgledem danej osi lokalnej o podany kat
	 * @param vAxis os
	 * @param vAngle kat (w stopniach)
	 */
	public native void RotateLocal( Vector vAxis, float fAngle );
	
	/**[native] Ustawia obiekt wg normalnej terenu pod nim
	 */
	public native void AlignToGround();
	
	/**[native] Przesuwa obiekt.
	 * @param v wartosc przesuniecia (<code>Vector</code>)*/
    public native void Move(Vector v);
		
	/**[native] Ustawia wektor predkosci obiektu w swiecie.
	 * @param v wektor predkosci obiektu wzgledem swiata 
	 * @param bAccel czy ma dzialac przyspieszenie/spowolnienie w gravitywalk (patrz {@link #SetWalkAccel( float fA)})
	 */
	public native void SetSpeed( Vector v, boolean bAccel);

    public void SetSpeed(Vector v)
    {
        //ustalamy predkosc bez ewentualnego przyspieszenia/spowolnienia
        SetSpeed(v, false);
    }
        
    /**[native] Ustawia kierunek wektora predkosci obiektu w swiecie (nie zmienia wartosci predkosci).
	 * @param v wektor predkosci obiektu wzgledem swiata 
	 */
	public native void SetSpeedDir( Vector v );
    
    /**[native] Ustawia wartosc predkosci obiektu w swiecie.
	 * @param v dlugosc wektora predkosci obiektu wzgledem swiata 
	 */
	public native void SetSpeedValue( float v, boolean bAccel);
    
    public void SetSpeedValue(float v)
    {
        //ustalamy predkosc bez ewentualnego przyspieszenia/spowolnienia
        SetSpeedValue(v, false);
    }

    /**[native] Aby postac chodzaca na gravitywalk plynnie sie poruszala musi to 
     * robic z przyspieszeniem, ta metoda ustala wartosc tego przyspieszenia
     * DZIALA TYLKO DLA FIZYKI GRAVITYWALK !!!!!
     * @param fA przyspieszenie i zwolnienie przy chodzeniu (ruszanie, zatrzymywanie sie)
     */
    public native void SetWalkAccel( float fA);

    public native void SetWalkDestSpeed(Vector v);
                                        
    public native Vector GetWalkDestSpeed();

    public native void GetWalkDestSpeed(Vector v);
    
    public native float GetWalkAccel();
    
	/**[native] Ustawia wektor predkosci obiektu lokalnie.
	 * @param v wektor predkosci obiektu wzgledem wlasnego ukladu, 
	 * z zawsze oznacza "w przod"
	 * @param bAccel czy ma dzialac przyspieszenie/spowolnienie w gravitywalk (patrz {@link #SetWalkAccel( float fA)})
	 */
	public native void SetLocalSpeed( Vector v, boolean bAccel);

    public void SetLocalSpeed(Vector v)
    {
        //ustalamy predkosc bez ewentualnego przyspieszenia/spowolnienia
        SetLocalSpeed(v, false);
    }
    
	/**[native] Ustawia docelowy wektor predkosci obiektu w swiecie.
	 * @param v wektor predkosci obiektu wzgledem swiata 
	 */
	public native void SetDestSpeed( Vector v );

    /**[native] Ustawia docelowy wektor kierunku predkosci obiektu w swiecie (nie zmienia
     * wartosci predkosci).
	 * @param v wektor predkosci obiektu wzgledem swiata 
	 */
	public native void SetDestSpeedDir( Vector v );
    
	/**[native] Ustawia docelowa wartosc predkosci obiektu w swiecie.
	 * @param v dlugosc wektora predkosci obiektu wzgledem swiata 
	 */
	public native void SetDestSpeedValue( float v );

    
	/**[native] Ustawia maxymalna wartosc przyspieszenia.
	 * @param v przyspieszenie w cm/s^2 
	 */
	public native void SetMaxAcceleration( float v );

    /**[native] Ustawia predkosc obrotowa obiektu.
	 * @param fAngSpeed predkosc obrotowa obiektu w stopniach/s (<code>float</code>)*/
	public native void SetAngSpeed( float fAngSpeed );
    
	/**[native] Pobiera pozycje obiektu w swiecie.
	 * @return pozycja obiektu (<code>Vector</code>)*/
	public native Vector GetPosition();	
	
	/**[native] Pobiera pozycje obiektu w swiecie.
	 * @param pozycja obiektu (<code>Vector</code>)*/
	public native void GetPosition(Vector v);
    
    
    /**[native] Pobiera pozycje lokalna obiektu.
	 * @return pozycja obiektu (<code>Vector</code>)*/
	public native Vector GetLocalPosition();
	
	/**[native] Pobiera pozycje lokalna obiektu.
	 * @param pozycja obiektu (<code>Vector</code>)*/
	public native void GetLocalPosition(Vector v);
	
	/**Uaktualnia zmienna m_vPosition z pozycja obiektu w swiecie*/
	public void UpdatePositionVector()
    {
        GetPosition(m_vVolPosition);
    }
    
    public Vector GetPositionVectorVolatile()
    {
        GetPosition(m_vVolPosition);
        return m_vVolPosition;
    }
    
	/**[native] Pobiera wektor "w przod" bazy obiektu
	 * @return wektor "w przod" bazy obiektu (<code>Vector</code>)*/
	public native Vector GetForwardVector();
	
	/**[native] Pobiera wektor "w przod" bazy obiektu
	 * @param wektor "w przod" bazy obiektu (<code>Vector</code>)*/
	public native void GetForwardVector(Vector v);
	
	/**Uaktualnia zmienna m_vForward z kierunkiem "w przod"*/
	public void UpdateForwardVector()
    {
        GetForwardVector(m_vVolForward);
    }
    
    public Vector GetForwardVectorVolatile()
    {
        GetForwardVector(m_vVolForward);
        return m_vVolForward;
    }
    
	/**[native] Pobiera wektor "w prawo" bazy obiektu
	 * @return wektor "w prawo" bazy obiektu (<code>Vector</code>)*/
	public native Vector GetRightVector();
	
	/**[native] Pobiera wektor "w prawo" bazy obiektu
	 * @param wektor "w prawo" bazy obiektu (<code>Vector</code>)*/
	public native void GetRightVector(Vector v);
	
	/**Uaktualnia zmienna m_vRight z kierunkiem "w prawo"*/
	public void UpdateRightVector()
    {
        GetRightVector(m_vVolRight);
    }
    
    public Vector GetRightVectorVolatile()
    {
        GetRightVector(m_vVolRight);
        return m_vVolRight;
    }
    
	/**[native] Pobiera wektor "w gore" bazy obiektu
	 * @return wektor "w gore" bazy obiektu (<code>Vector</code>)*/
    public native Vector GetUpVector();
    
	/**[native] Pobiera wektor "w gore" bazy obiektu
	 * @param wektor "w gore" bazy obiektu (<code>Vector</code>)*/
    public native void GetUpVector(Vector v);
    
	/**Uaktualnia zmienna m_vup z kierunkiem "w gore"*/
	public void UpdateUpVector()
    {
        GetUpVector(m_vVolUp);
    }
    
    public Vector GetUpVectorVolatile()
    {
        GetUpVector(m_vVolUp);
        return m_vVolUp;
    }
    
	/**[native] Ustawia orientacje obiektu w swiecie
	 * @param v nowa orientacja obiektu w swiecie	 */
	public native void SetDir(Vector v);
    
    /**[native] Ustawia orientacje obiektu w ukladzie lokalnym
     * Oznacza to, ze gdy obiekt jest podlaczony do innego to bedzie ustalany kierunek wzgledem rodzica.
	 * @param v nowa orientacja obiektu w swiecie	 */
	public native void SetDirLocal(Vector v);
    
    /**[native] Ustawia docelowa orientacje obiektu w swiecie
	 * @param v nowa orientacja obiektu w swiecie	 */
	public native void SetDestDir(Vector v);

    /**[native] Dziala jak SetDestDir, ale nie pozwala na zmiane predkosci po update fizyki
	 * @param v nowa orientacja obiektu w swiecie	 */
	public native void SetFacingDestDir(Vector v);

    /**[native] Wlacza flage powodujaca nie updejtowanie kierunku predkosci do
     * ustawienia mesha metoda SetDir, SetDestDir w fizyce GravityWalk i Walk.
     */
    public native void SetFacingFlag(boolean bState);
    
    /**[native] Ustawia polozenie i orientacje obiektu w swiecie - glownym kryterium
     * jest wektor up - wygodne do orientowania obiektu np. do normalnej do terenu
	 * @param vForward nowa orientacja obiektu w swiecie	
	 * @param vUp wektor "w gore" obiektu
	 * @param vPos nowa pozycja obiektu w swiecie */
	public native void FromUpForwardPos(Vector vUp, Vector vForward, Vector vPos);
	
    /**[native] Pobiera wektor predkosci obiektu w swiecie.
	 * @return wektor predkosci obiektu wzgledem swiata (<code>Vector</code>)*/
	public native Vector GetSpeed();
	
    /**[native] Pobiera wektor predkosci obiektu w swiecie.
	 * @param wektor predkosci obiektu wzgledem swiata (<code>Vector</code>)*/
	public native void GetSpeed(Vector v);
    
    /**[native] Wylicza wektor predkosci obiektu w swiecie, ktory bedzie
     * wyliczony przez fizyke w najblizszej ramce
	 * @return wektor predkosci obiektu wzgledem swiata (<code>Vector</code>)*/
	public native Vector GetSpeedNextFrame();
	
    /**[native] Wylicza wektor predkosci obiektu w swiecie, ktory bedzie
     * wyliczony przez fizyke w najblizszej ramce
	 * @param wektor predkosci obiektu wzgledem swiata (<code>Vector</code>)*/
	public native void GetSpeedNextFrame(Vector v);
	
	/**Uaktualnia zmienna m_vSpeed z wektorem predkosci w swiecie*/
	public void UpdateSpeedVector()
    {
        GetSpeed(m_vVolSpeed);
    }
    
    public Vector GetSpeedVectorVolatile()
    {
        GetSpeed(m_vVolSpeed);
        return m_vVolSpeed;
    }
    
	/**[native] Pobiera wektor predkosci lokalnej obiektu.
	 * @return wektor predkosci lokalnej obiektu (<code>Vector</code>)*/
	public native Vector GetLocalSpeed();
    
	/**[native] Pobiera wektor predkosci lokalnej obiektu.
	 * @param wektor predkosci lokalnej obiektu (<code>Vector</code>)*/
	public native void GetLocalSpeed(Vector v);
    
	/**Uaktualnia zmienna m_vLocalSpeed z wektorem predkosci w ukl. lokalnym obiektu*/
	public void UpdateLocalSpeedVector()
    {
        GetLocalSpeed(m_vVolLocalSpeed);
    }
    
    public Vector GetLocalSpeedVectorVolatile()
    {
        GetLocalSpeed(m_vVolLocalSpeed);
        return m_vVolLocalSpeed;
    }
    
    /**[native] Pobiera wektor predkosci obiektu w swiecie z uwzglednieniem predkosci
     * rodzicow w hierarchii tego obiektu
	 * @return wektor predkosci obiektu wzgledem swiata (<code>Vector</code>)*/
	public native Vector GetAbsoluteSpeed();
	
    /**[native] Pobiera predkosc obrotowa obiektu.
	 * @return wartosc predkosci obrotowej obiektu w stopniach/s (<code>float</code>)*/
	public native float GetAngSpeed();

    /**[native] Pobiera efektywna predkosc obrotowa obiektu.
	 * @return wartosc predkosci obrotowej obiektu w stopniach/s (<code>float</code>)*/
	public native float GetCurAngSpeed();

    /**[native] Pobiera dlugosc wektora predkosci.
	 * @return predkosc obiektu (<code>float</code>)*/
	public native float GetSpeedValue();
    
    /**[native] Ustawia mase obiektu.
	 * @param fMass masa obiektu w kg (<code>float</code>)*/
	public native void SetMass( float fMass );
    
    /**[native] Pobiera mase obiektu.
	 * @return masa obiektu w kg (<code>float</code>)*/
	public native float GetMass();
    
    /**[native] Pobiera czas ostatniego updateu fizyki.
	 * @return vzas gry w sekundach (<code>float</code>)*/
	public native float GetPhysicsLastUpdateTime();
    
    
    /**[native] Ustawia czas ostatniego updateu fizyki.
	 * @param fTime czas w sekundach	 */
	public native void SetPhysicsLastUpdateTime(float fTime);
    
    /**[native] Pobiera z fizyki environment poruszania sie obiektu.
	 * @return identyfikator srodowiska w ktorym porusza sie obiekt (<code>int</code>)*/
	public native void SetPhysicsEnv( int nEnv );
    
    /**[native] Ustawia environment poruszania sie obiektu.
	 * @param nEnv - identyfikator srodowiska w ktorym porusza sie obiekt (<code>int</code>)*/
	public native int GetPhysicsEnv();
    
    /**[native] Pobranie predkosci spadania obiektu z fizyka GravityWalk
     * @return wektor predkosci spadania, gdy fizyka inna niz GravityWalk to
     * zwraca null
     */
    public native Vector GetFallSpeed();
    
    /**[native] Ustala maksymalna predkosc spadania obiektu z fizyka GravityWalk
    */
    public native void SetMaxFallSpeed(Vector vSpeed);
    
    /**[native] Ustala predkosc spadania obiektu z fizyka GravityWalk, ma sens wtedy, gdy
     * grawitacja - masa ({@link #SetMass(float fMass)}) jest ustawiona na zero, inaczej
     * bedzie caly czas zmieniana przy updejcie fizyki.
    */
    public native void SetFallSpeed(Vector vSpeed);
    
    
    /**[native] Popycha symulacje o podany czas.
	 * @param fTime czas o jaki symulujemy obiekt (<code>float</code>)*/
	public native void UpdatePhysics( float fTime );

    /**[native] Popycha symulacje o podany czas - symuluje tylko fizyke ODEGravityWalk bez update'u ODE
	 * @param fTime czas o jaki symulujemy obiekt (<code>float</code>)*/
    public native void UpdatePhysicsOnlyGW(float fTime);

    /**[native] Popycha symulacje o podany czas - symuluje tylko ODE dla siebie samego - nie dla calego swiata
	 * @param fTime czas o jaki symulujemy obiekt (<code>float</code>)*/
    public native void UpdatePhysicsOnlyODE(float fTime);
    
    /**[native] Ustawia docelowa pozycje obiektu w swiecie
	 * @param vPos wymagana pozycja obiektu w swiecie
	 * @param fRadius dopuszczalny promien (odleglosc od punktu) */
	public native void SetDestination(Vector vPos,float fRadius);
    
    /**[native] Ustawia docelowa pozycje obiektu w swiecie
	 * @param vPos wymagana pozycja obiektu w swiecie	 */
	public native void SetDestinationPos(Vector vPos);
    
    /**[native] Ustawia dopuszczalny promien wokol punktu docelowego
     * dla ktorego bedziemy uwazali ze obiekt juz jest na miejscu
	 * @param fRadius dopuszczalny promien	 */
	public native void SetDestinationRadius(float fRadius);
    
    /**[native] Kasuje wolanie handlera po dotarciu do punktu
     */
    public native void ClearDestinationPos();
    
    /**[native] Kasuje wolanie handlera po obroceniu sie
     */
    public native void ClearDestinationDir();

    /**[native] Ustawia katy zmiany stanu nachylenia obiektu (katy w stopniach)
     * @param fUphillStart - kat wejscia w stan podchodzenia (np. 45)
     * @param fUphillEnd - kat wyjscia ze stanu podchodzenia (np. 30)
     * @param fDownhillStart - kat wejscia w stan schodzenia (np. 45)
     * @param fDownhillEnd - kat wyjscia ze stanu schodzenia (np. 30)
     */
    public native void SetUpDownhillAngles(float fUphillStart, float fUphillEnd, float fDownhillStart, float fDownhillEnd);
    
    /**[native] Powoduje dodanie wektora predkosci do wektora predkosci spadania
     * Uwaga: Wywolanie odniesie skutek tylko, jesli obiekt uzywa fizyki 
     *  EPhysics._GRAVITYWALK
     */
    public native void Jump(Vector vJumpSpeed);
    
    /**[native] Metoda przelicza podany wektor(w ukladzie ControlObject) do ukladu globalnego.
     * @param vVector wektor w ukladzie ControlObjecta
     * @return wektor w ukladzie swiata - globalnym tzn. vVector przemnozony przez macierz ControlObject
     */
    public native Vector VectorLocalToWorld(Vector vVector);
    
    /**[native] Metoda przelicza podany wektor(w ukladzie swiata) do ukladu ControlObject.
     * @param vVector wektor w ukladzie swiata
     * @return wektor w ukladzie ControlObjecta tzn. vVector przemnozony przez macierz odwrtona ControlObject
     */
    public native Vector VectorWorldToLocal(Vector vVector);
    
    /**[native] Metoda przelicza podana pozycje(w ukladzie ControlObject) do ukladu globalnego.
     * @param vVector wektor w ukladzie ControlObjecta
     * @return wektor w ukladzie swiata - globalnym tzn. vVector przemnozony przez macierz ControlObject
     */
    public native Vector PointLocalToWorld(Vector vVector);
    
    /**[native] Metoda przelicza podana pozycje(w ukladzie globalnym) do ukladu ControlObject.
     * @param vVector wektor w ukladzie swiata - globalnym 
     * @return wektor w ukladzie ControlObjecta tzn. vVector przemnozony przez macierz odwrotna ControlObject
     */
    public native Vector PointWorldToLocal(Vector vVector);
    
    /**[native] Metoda wylicza odleglosc do podanego ControlObjecta.
     * @param cObj obiekt do ktorego odleglosc nas interesuje
     * @return odleglosc
     */
    public native float GetDistanceTo(ControlObject cObj);
    
    /**[native] Metoda wylicza odleglosc kwadrat do podanego ControlObjecta.
     * @param cObj obiekt do ktorego odleglosc nas interesuje
     * @return odleglosc kwadrat
     */
    public native float GetDistance2To(ControlObject cObj);

    /**[native] Metoda wylicza odleglosc do podanego punktu.
     * @param vPoint punkt do ktorego odleglosc nas interesuje
     * @return odleglosc
     */
    public native float GetDistanceTo(Vector vPoint);
    
    /**[native] Metoda wylicza odleglosc kwadrat do podanego punktu.
     * @param vPoint punkt do ktorego odleglosc nas interesuje
     * @return odleglosc kwadrat
     */
    public native float GetDistance2To(Vector vPoint);

    
	////////////////////////////////
	// metody zwiazane z pathmapa //
	////////////////////////////////

	/**[native] Czy obiekt ma modyfikowac pathmape
	 * @param bUpdate czy update'owac pathmape	(<code>boolean</code>) */
	final native void UpdatePathMap(boolean bUpdate);

	/**[native] Czy obiekt ma niezamazywac pathmapy pod soba - powinno byc
	 * ustawiane tylko dla niektorych obiektow, ktore moga ignorowac pathmape
	 * @param bPreserve czy zachowywac pathmape (<code>boolean</code>) */
	final native void PreservePathMap(boolean bPreserve);
    
	/**[native] Czy obiekt ma modyfikowac pathmape
	 * @param bUpdate czy update'owac (<code>boolean</code>)
	 * @param wielkosc stampa obiektu, w przypadku metody jednoparametrowej
	 * domyslnie jest 1 (nie moze byc <= 0) (<code>boolean</code>)
	 */
	final native void UpdatePathMap(boolean bUpdate, int iStampSize);

	/**[native] Czy obiekt ma modyfikowac pathmape
	 * @param bUpdate czy update'owac (<code>boolean</code>)
	 * @param wielkosc stampa obiektu, w przypadku metody jednoparametrowej
	 * @param czy obiekt ma nie zamazywac mapy pod soba
	 * domyslnie jest 1 (nie moze byc <= 0) (<code>boolean</code>)
	 */
	final native void UpdatePathMap(boolean bUpdate, int iStampSize, boolean bPreserve);
    
	/**[native] Czy obiekt ma modyfikowac dodatkowa mape (do budowania mapy przechodnosci)
	 * @param bUpdate czy update'owac (<code>boolean</code>)
	 * @param rozdzielczosc pliku wynikowego (<code>boolean</code>)
	 */
	final native void UpdateAdditionalMap(boolean bUpdate, int iResolution);
    
	/**[native] Zapis dodatkowej mapy (do budowania mapy przechodnosci)
	 * @param sFileName nazwa pliku raw(<code>boolean</code>)
	 */
	final native void SaveAdditionalMap(String sFileName);
    
	final native boolean IsUpdatingPathMap();
	final native int GetStampSize();	
	
    ////////////////////////////////
	// metody obslugujace kolizje //
	////////////////////////////////
	
	/**[native] Stwierdza czy z aktualnego ControlObjecta 'widac' podany.<br>
	 * Punkt poczatku i konca promienia kolizji jest przesuwany o podane translacje.
	 * Mozna podac zamiast wektorow null i wtedy nie bedzie translacji.
	 * @param Target obiekt z ktorym kolizje badamy (<code>ControlObject</code>)
	 * @param vStartTranslation wektor przesuniecia poczatku promienia kolizji, 
	 * jesli <code>null</code> to brak translacji (<code>Vector</code>)
	 * @param vEndTranslation wektor przesuniecia konca promienia kolizji, 
	 * jesli <code>null</code> to brak translacji (<code>Vector</code>)
	 * @return czy widac Target
	 */
	final native boolean CheckTrace(ControlObject Target, Vector vStartTranslation, Vector vEndTranslation);

    /**[native] Odpowiednik @link CheckTrace z pozycjami podawanymi we wspolrzednych swiata
     */
    public final native boolean CheckTraceAbsolute(ControlObject Target, Vector vStartPosition, Vector vEndPosition);
    
    
    ////////////////////////////////////////////////////////////////////////////
	// metody pobierania ControlObjects z levelu, do ktorego jest przylaczony //
	// obiekt na rzecz, ktorego metody zostana zawolane                       //
	////////////////////////////////////////////////////////////////////////////

    /**[native] Pobranie obiektow w promieniu od obiektu na rzecz ktorego jest zawolana, dodatkowo pozwala 
     * okreslic klase obiektow, gdy klasa == NULL to nie bierze tego parametru pod uwage. Nie zwraca obiektu
     * na rzecz ktorego zostala zawolana.
     * Zwracane sa obiekty dokladnie podanej klasy.
     * @param fRadius wielkosc promienia (<code>float</code>)
     * @param sClassName nazwa klasy (<code>String</code>)
     * @return tablica znalezionych obiektow (moze miec 0 elementow) (<code>ControlObject []</code>)
     */
    final native ControlObject [] GetObjectsInRadiusOfClass(float fRadius, String sClassName);
    
    /**[native] Pobranie obiektow w promieniu od obiektu na rzecz ktorego jest zawolana, dodatkowo pozwala 
     * okreslic klase obiektow, gdy klasa == NULL to nie bierze tego parametru pod uwage. Nie zwraca obiektu
     * na rzecz ktorego zostala zawolana.
     * Zwracane sa obiekty potomne od podanej klasy.
     * @param fRadius wielkosc promienia (<code>float</code>)
     * @param sClassName nazwa klasy (<code>String</code>)
     * @return tablica znalezionych obiektow (moze miec 0 elementow) (<code>ControlObject []</code>)
     */
    public final native ControlObject [] GetObjectsInRadiusFromClass(float fRadius, String sClassName);
    
    /**[native] Pobranie obiektow w promieniu od obiektu na rzecz ktorego jest zawolana, dodatkowo pozwala 
     * okreslic klase obiektow, gdy klasa == NULL to nie bierze tego parametru pod uwage. Nie zwraca obiektu
     * na rzecz ktorego zostala zawolana.
     * Zwracane sa obiekty potomne od podanej klasy.
     * @param fRadius wielkosc promienia (<code>float</code>)
     * @param nClassID identyfikator klasy (<code>String</code>)
     * @return tablica znalezionych obiektow (moze miec 0 elementow) (<code>ControlObject []</code>)
     */
    public final native ControlObject [] GetObjectsInRadiusFromClass(float fRadius, int nClassID);
    
    /**[native] Pobranie obiektow w promieniu od obiektu na rzecz ktorego jest zawolana, dodatkowo pozwala 
     * okreslic flagi i klase obiektow, gdy klasa == NULL to nie bierze tego parametru pod uwage. Nie zwraca obiektu
     * na rzecz ktorego zostala zawolana.
     * Zwracane sa obiekty potomne od podanej klasy.
     * @param fRadius wielkosc promienia (<code>float</code>)
     * @param iFlags flagi z EFlags (<code>int</code>)
     * @param sClassName nazwa klasy (<code>String</code>)
     * @return tablica znalezionych obiektow (moze miec 0 elementow) (<code>ControlObject []</code>)
     */
    public final native ControlObject [] GetObjectsInRadiusOfFlagsFromClass(float fRadius, int iFlags, String sClassName);
    
    /**[native] Pobranie obiektow w extentsach podanych wzgledem pozycji obiektu dla ktorego jest
     * zawolane. Dodatkowo pozwala okreslic klase obiektow, gdy klasa == NULL to nie bierze tego 
     * parametru pod uwage. Nie obiektu na rzecz ktorego zostalo zawolane.
     * @param vCorner1 wspolrzedna min boxa (<code>Vector</code>)
     * @param vCorner2 wspolrzedna max boxa (<code>Vector</code>)
     * @param sClassName nazwa klasy (<code>String</code>)
     * @return tablica znalezionych obiektow (<code>ControlObject []</code>)
     */
    // TODO final native ControlObject [] GetObjectsInExtents(Vector vCorner1, Vector vCorner2, String sClassName);

    /////////////////////////////////////////////
	// metody do ustawiania obiektow na levelu //
	/////////////////////////////////////////////
    
    //juz nieuzywane
    /**[native] Powoduje  uruchomnienie efektu on nazwie zadanej parametrem.
     * Pozycja i orientacja sa pobierane z macierzy ControlObjectu.
     * @param sFXName nazwa zdefiniowanej sekwencji (ta podawana jako parametr wywolania metody Sequence("nazwa sekwencji")) (<code>String</code>)
     * Patrz komentarz do metod ExecuteFX w klasie GameObject
     */    
    
    //public native int ExecuteFX(ControlObject ctrlObj,String sFXName);    
    
    /**[native] Powoduje  uruchomnienie efektu on nazwie zadanej parametrem.     
     * @param sFXName nazwa zdefiniowanej sekwencji (ta podawana jako parametr wywolania metody Sequence("nazwa sekwencji")) (<code>String</code>)
     * @param pos pozycja FX'a
     * @param rot - obrot wzgledem rodzica. Wektor zawiera katy o jakie zostanie obrocona
     * macierz efektu odpowiednio wzgledem osi X, Y i Z. Umozliwia alternatywne zadanie orientacji FX'a w stosunku do zadawania przez
     * normalna. Obroty beda wykonywane ukladzie swiata, badz ukladzie obiektu pod ktory dany efekt jest przypiety (<code>Vector</code>)
     * 
     * Patrz komentarz do metod ExecuteFX w klasie GameObject
     */
    
    //public native int ExecuteFX(String sFXName,Vector pos,Vector rot);
    //public native int ExecuteFX(ControlObject ctrlObj,String sFXName,Vector pos,Vector rot);
    
    /** Metoda kasuje wszystkie fxy podpiete do obiektu
     */
    public native void DeleteAttachedFXes();
    
    /////////////////////////////////
    // obsluga hierarchii obiektw //
    /////////////////////////////////

    /**[native] Podlacza obiekt obj do danego obiektu.
     * @param obj - podlaczany obiekt
     */
    public final native void AttachChild(ControlObject obj);

    /**[native] Podlacza obiekt obj do danego obiektu. Zachowuje pozycje obiektu obj w swiecie.
     * @param obj - podlaczany obiekt
     */
    public final native void AttachChildAndKeepWorldPos(ControlObject obj);

    /**[native] Odlacza obiekt obj od danego obiektu
     * @param obj - odlaczany obiekt
     */
    public final native void DetachChild(ControlObject obj);

    /**[native] Podlacza dany obiekt jako dziecko do obj.
     * @param obj - obiekt, do ktorego zostanie podlaczony dany obiekt
     */
    public final native void AttachAsChildTo(ControlObject obj);

    /**[native] Podlacza dany obiekt jako dziecko do obj. Zachowuje pozycje obiektu w swiecie.
     * @param obj - obiekt, do ktorego zostanie podlaczony dany obiekt
     */
    public final native void AttachAsChildToAndKeepWorldPos(ControlObject obj);

    /**[native] Odlacza obiekt od rodzica
     */
    public final native void DetachFromParent();
    
    /**[native] Odlacza obiekt od rodzica zachowujac jego pozycje w swiecie
     */
    public final native void DetachFromParentAndKeepWorldPos();
    
    /**[native] Kasuje wszystkie obiekty dzieci dla obiektu.
     */
    public final native void DeleteAllChildren();
    
    /**[native] Pobiera rodzica obiektu
     */
    public final native ControlObject GetParent();
    
    /** Zwraca nazwe obiektu rodzica
     */
    public final String GetParentName()
    {
        ControlObject cObj = GetParent();
        return (cObj == null) ? "" : ("<"+cObj.GetName()+":"+cObj.GetUniqueID()+">");
    }
    
    /**[native] Pobiera wszystkie dzieci obiektu, jesli obiekt nie ma dzieci zwraca null
     */
    public final native ControlObject[] GetChildren();
    
    /** Metoda okresla czy obiekt jest podpiety do jakiegos PortalObjecta.
     * @return true gdy obiekt jest podpiety do portalobjecta
     */
    public boolean IsInPortalObject()
    {
        ControlObject cParent = this;
        while(cParent != null)
        {
            if( (cParent.GetFlags() & EFlags._PORTAL_OBJECT) != 0 ||
                (cParent.GetFlags() & EFlags._PORTAL_INDEPENDENT) != 0 ||
                (cParent.GetFlags() & EFlags._LINKED_TO_PORTAL_OBJECT) != 0)
                return true;
            
            cParent = cParent.GetParent();
        }
        
        return false;
    }

    //////////////////////
    // obsluga dzwiekow //
    //////////////////////
    
    /**[native] Powoduje podlaczenie do obiektu zapetlonego dzwieku
	 * @param sSoundName nazwa dzwieku (nazwa pliku z dzwiekiem bez rozszerzenia)
	 * @return identyfikator dzwieku
	 */
	native int AttachSound(String sSoundName);

    /**[native] Powoduje podlaczenie do obiektu zapetlonego dzwieku
	 * @param sSoundName nazwa dzwieku (nazwa pliku z dzwiekiem bez rozszerzenia)
     * @param fMin odleglosc od emitera dzwieku po ktorej zacznie dzwiek wygasac
     * @param fMax odleglosc od emitera dzwieku po ktorej dzwieku nie slychac
	 * @return identyfikator dzwieku
	 */
	native int AttachSound(String sSoundName, float fMin, float fMax);
    
    /**[native] Powoduje podlaczenie do obiektu dzwieku odgrywanego tylko raz
     * dzwiek jest automatycznie usuwany po odegraniu
     * UWAGA!!! stosowac tylko dla obiektow zmieniajacych pozycje,
     * w innych przypadkach stosowac PlaySound !!!
	 * @param sSoundName nazwa dzwieku (nazwa pliku z dzwiekiem bez rozszerzenia)
	 */
	native void AttachOneShotSound(String sSoundName);
	
	/**[native] Powoduje podlaczenie do obiektu dzwieku odgrywanego tylko raz
     * dzwiek jest automatycznie usuwany po odegraniu
     * UWAGA!!! stosowac tylko dla obiektow zmieniajacych pozycje,
     * w innych przypadkach stosowac PlaySound !!!
	 * @param sSoundName nazwa dzwieku (nazwa pliku z dzwiekiem bez rozszerzenia)
	 * @param fVolume glosnosc (0 - 1)
	 */
	native void AttachOneShotSound(String sSoundName, float fVolume);

    /**[native] Powoduje podlaczenie do obiektu dzwieku odgrywanego tylko raz
     * dzwiek jest automatycznie usuwany po odegraniu
     * UWAGA!!! stosowac tylko dla obiektow zmieniajacych pozycje,
     * w innych przypadkach stosowac PlaySound !!!
	 * @param sSoundName nazwa dzwieku (nazwa pliku z dzwiekiem bez rozszerzenia)
     * @param fMin odleglosc od emitera dzwieku po ktorej zacznie dzwiek wygasac
     * @param fMax odleglosc od emitera dzwieku po ktorej dzwieku nie slychac
	 */
	native void AttachOneShotSound(String sSoundName, float fMin, float fMax);
    
    /**[native] Powoduje usuniecie dzwieku podlaczonego metoda AttachSound
	 * @param iSoundID identyfikator dzwieku zwrocony przez AttachSound
	 */
	native void DetachSound(int iSoundID);
    
    /**[native] Powoduje usuniecie wszystkich podlaczonych do obiektu dzwiekow
	 */
	native void DetachAllSounds();
    
    /**[native] Ustawia glosnosc dzwieku
	 * @param iSoundID identyfikator dzwieku zwrocony przez AttachSound
	 * @param fVolume glosnosc (0 do 1)
	 */
	native void SetSoundVolume(int iSoundID, float fVolume);
    
    /**[native] Ustawia czestotliwosc dzwieku
	 * @param iSoundID identyfikator dzwieku zwrocony przez AttachSound
	 * @param fFreq mnoznik czestotliwosci w stosunku do oryginalu
	 */
	native void SetSoundFreq(int iSoundID, float fFreq);

    /**[native] Zatrzymuje odgrywanie dzwieku przypietego do obiektu
     * @param iSoundID identyfikator dzwieku zwrocony przez AttachSound
     */
    native void StopSound(int iSoundID);
   
    /**[native] Ponawia odgrywanie dzwieku zatrzymanego poprzez StopSound
     * @param iSoundID identyfikator dzwieku zwrocony przez AttachSound
     */
    native void PlaySound(int iSoundID);
    
    /**[native] Metoda sprawdza czy podany dzwiek aktualnie gra.
     * @param iSoundID identyfikator dzwieku zwrocony przez AttachSound
     * @return true gdy dzwiek gra
     */
    native boolean IsSoundPlaying(int iSoundID);
    
    /////////////////////////////////////////////////
    // meotdy dotyczace obslugi uszkodzen obiektow //
    /////////////////////////////////////////////////
    
    /** [native] Zadaje obrazenia o podanej wartosci. Powoduje wywolanie handlera OnTakeDamage()
     * @param fDamage wartosc obrazen, powinna odjac sie od energii.
     * @param fPhSimDamage wartosc obrazen przekazywana do symulacji fizyki po smierci "klienta", im wieksza wartosc, tym bardziej postac "fruwa"
     * @param Asassin obiekt ktory jest inicjatorem damage'a np. wlasciciel pocisku, ktory trafia.
     * @param vHitLocation punkt w ktorym obiekt otrzymuje obrazenia.
     * @param vSpeed wektor predkosci (w kierunku celu).
     * @param nDamageType rodzaj trafienia, sluzy do przekazywania dodatkowych informacji. Na razie nie ustalone.
     */
    public native void TakeDamage(float fDamage, float fPhSimDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed, int nDamageType);

    /** Handler otrzywany wskutek zawolania TakeDamage() */
    //OnTakeDamage(float fDamage, float fPhSimDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed, int nDamageType);

    /** [native] Zadaje obrazenia o podanej wartosci. Powoduje wywolanie handlera OnTakeDamage().
     * W porownaniu do powyzszego TakeDamage jest rozbudowany o dodatkowego inta (zmienione jest tez
     * znaczenie inta nDamageType - teraz jest to jedna z wartosci z EnumDamageType)
     * @param fDamage wartosc obrazen, powinna odjac sie od energii.
     * @param fPhSimDamage wartosc obrazen przekazywana do symulacji fizyki po smierci "klienta", im wieksza wartosc, tym bardziej postac "fruwa"
     * @param Asassin obiekt ktory jest inicjatorem damage'a np. wlasciciel pocisku, ktory trafia.
     * @param vHitLocation punkt w ktorym obiekt otrzymuje obrazenia.
     * @param vSpeed wektor predkosci (w kierunku celu).
     * @param nMeshElementorBoneID - meshelement, w ktory nastapilo trafienie.
     * @param nDamageType rodzaj trafienia: knife, bullet, explosive itp.
     */
    public native void TakeDamage(float fDamage, float fPhSimDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed, int nMeshElementorBoneID, int nDamageType);
	
    /** Handler otrzywany wskutek zawolania TakeDamage() */
    //OnTakeDamage(float fDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed, int nMeshElementorBoneID, int nDamageType);
	
    //////////////////////////////////////////
    // metody ustawiajace parametry kolizji //
    //////////////////////////////////////////
   
    /** [native] Ustawia typ kolizji przy poruszaniu obiektu
     * @param iType typ kolizji (z ECollFlags)
     */
    public native void SetCollType(int iType);
    
    /** [native] Ustawia typ reakcji na kolizje przy poruszaniu
     * @param iAction akcja po kolizji (z ECollFlags)
     */
    public native void SetCollAction(int iAction);
    
    /** [native] Ustawia typ handlera wolanego w momencie kolizji
     * @param iType typ wolanego handlera (z ECollFlags)
     */
    public native void SetCollHandler(int iHandler);

    /** [native] Ustawia flagi kolizji (poprzednie_flagi | iFlags), nie zastepuje poprzednich
     * @param iFlags flagi kolizji (z ECollFlags)
     */
    public native void SetCollFlags(int iFlags);
    
    /** [native] Kasuje flagi kolizji
     * @param iFlags flagi kolizji (z ECollFlags)
     */
    public native void ClearCollFlags(int iFlags);
    
    /** [native] Podmienia wszystkie flagi kolizji
     * @param iFlags flagi kolizji (z ECollFlags)
     */
    public native void ReplaceCollFlags(int iFlags);
    
    /** [native] Zwraca wszystkie flagi kolizji
     * @return flagi kolizji (z ECollFlags)
     */
    public native int GetCollFlags();
    
    
    //////////////////////////////////////////
    // metody ustawiajace parametry pojazdu //
    //////////////////////////////////////////
   
    /** [native] Ustawia kat skretu kol
     * @param fVal kat w stopniach (ujemny oznacza skret w lewo)
     */
    public native void SetWheelsAngle(float fVal);
    
    /** [native] Ustawia granice wejscia w poslizg
     * @param fVal (im wieksza wartosc tym trudniej wchodzi w poslizg,
     * jest bardziej przyczepny, przykl. wartosc 8)
     */
    public native void SetFrictionWheels(float fVal);
    
    /** [native] Ustawia granice wyjscia z poslizgu
     * @param fVal (powinna to byc wartosc mniejsza niz w FrictionWheels)
     */
    public native void SetFrictionWheels2(float fVal);
    
    /** [native] Ustawia ile razy pojazd jest mniej podatny na poslizgi w poprzek kierunku ruchu
     * @param fVal (przykladowa wartosc 0.5)
     */
    public native void SetFrictionCross(float fVal);
    
    /** [native] Ustawia sztywnosc sprezyny resoru
     * @param fVal (im mniejsza wartosc tym bardziej sztywna sprezyna, przykladowa wartosc 0.5)
     */
    public native void SetSpringer(float fVal);
    
    /** [native] Ustawia tlumienie wahan kola w amortyzatorze
     * @param fVal (im wiekszy tym bardziej tlumi, przykladowa wartosc 0.5)
     */
    public native void SetAbsorber(float fVal);
    
    /** [native] Ustawia dlugosc sprezyny resoru (maksymalne wysuniecie kola)
     * @param fVal dlugosc sprezyny (w cm, np. 50)
     */
    public native void SetMaxSwing(float fVal);
    
    /** [native] Przesuwa bryle sztywna o podany wektor
     * @param vShift wektor przesuniecia
     */
    public native void ApplyFloorShift(Vector vShift);
    
    /** [native] 
     * Tymczasowa procka do dema!!!
     * Autko przy poruszaniu powinno "bujac" sie na wszsytkie strony,
     * tzn podczas:
     *  ruszania do przodu powinno odchylac sie do tylo, 
     *  hamowania powinno pochylac sie w przod,
     *  podczas skrecania w lewo powinno wychylac sie w prawo, natomiast
     *  podczas skrecania w prawo powinno wychylac sie w lewo
     *  itp, itd...
     * Z jakis tajemniczych przyczyn cos sie z tym dzieje nie tak, i mozna
     * to naprawic poprzez ustawienie tej wartosci
     * @param fOffset - warotsc o jaka nalezy zmodyfikowac defaultowe ustawienia
     * jezeli auto przechyla sie ZA SLABO lub W ZLA STRONE to ZWIEKSZ ta wartosc
     * jezeli auto przechyla sie ZA MOCNO to ZMNIEJSZ ta wartosc
     * domyslna wartosc to zero 
     * Powodzenia! :-)
     */
    public native void SetMagicOffset(float fOffset);
    
    /**
     * pstryczek dla samochodzika
     * 1: // w srodek do gory
     * 2: // kon pociagowy za przednie kola 
     * 3: // kon pociagowy za wszystkie kola
     * 4: // lewe przednie kolo do gory
     * 0: // zerowanie predkosci obrotowej
     */
    public native void SetJumpMode(int nMode);
    
    ///////////////////////////////////////
    // obsluga obiektow aktywnych w grze //
    ///////////////////////////////////////
    
    /** Pobiera aktywny trigger na podstawie podanych parametrow. Obiekt, ktory wola ta
     * metode jest brany za aktywatora triggera.
     * @param vStart punkt z ktorego sprawdzamy triggery
     * @param vDir kierunek w jakim sprawdzamy triggery
     * @param fDistance odleglosc na jakiej dziala od podanego punktu startu zaznaczanie triggerow
     * @return zwraca obiekt, ktory jest aktywny
     */
    TriggerObject GetActiveTrigger(Vector vStart, Vector vDir, float fDistance)
    {
        Vector vEnd=new Vector(vDir);
        vEnd.Mul(fDistance);
        vEnd.Add(vStart);

        // liczymy kolizje z drzewami, zeby nie bylo glupiej sytuacji, ze cos za drzewem widac
        // rozwazyc czy jeszcze nie liczyc z terenem
        
        // liczymy kolizje z obiektami
        Collision stCollInfo = TraceObjectsExclude(vStart, vEnd, this);
        
        if(stCollInfo!=null)
        {
            // kolizja jest z jakims obiektem, sprawdzamy czy to nie TriggeredObject
            if(stCollInfo.Obstacle instanceof TriggerObject)
            {
                //CrashLog("Object on celownik = stCollInfo.Obstacle="+stCollInfo.Obstacle.GetName()+"\n");
            
                TriggerObject Obj=(TriggerObject)stCollInfo.Obstacle;
                
                // tylko aktywne obiekty moga byc zaznaczone
                if( Obj.IsActive(this) && Obj.IsActive(this, stCollInfo.ObstacleElement) )
                {
                    if(Obj.CanActivate(stCollInfo.ObstacleElement))
                    {
                        return Obj;
                    }
                }
            }
        }
        return null;
    }

    native public void SetPortal(ControlObject Portal);


    ///////////////////////////////////////
    // obsluga sieci w grze              //
    ///////////////////////////////////////
    
    
    
    
    /** Oblicza odleglosc obiektu od glownego obiektu (kamery?) aktualnie
     * przetwarzanego klienta, 
     * Dziala na podstawie GetCurrentClientPosition()
     * metoda do wykorzystania na serwerze w metodzie GetRelevantLevel()
     * podczas sprawdzania czy obiektu sieciowego jest w poblizu aktualnie 
     * przetwarzanego klienta
     * @return zwraca odleglosc od klienta (<code>float</code>)
     * W przyszlosci najprawdopodobniej nie bedzie to raczej wykorzystywane 
     * bo bedziemy oceniac widocznosc obiektu na podstawie kamery
     * 
     */
    float CalcCurrentClientDistance()
    {
        Vector vPos = GetCurrentClientPosition();
        if ( vPos != null )
            return vPos.Distance( this.GetPosition() );
        
        return -1.0f;
    }
    
    
    /** Probuje przesunac obiekt systemem elipsoid o vMove, zwraca true jesli
     * obiekt moze byc przesuniety bez problemu do konca, w przeciwnym wypadku
     * zwraca w vMove wektor rzeczywistego przesuniecia
     * UWAGA: obiekt _nie jest_ przesuwany tylko jest testowana mozliwosc przesuniecia!!!!!
     */
    public native boolean MoveByEllipsoid(Vector vMove);
    
    /** Probuje przesunac obiekt systemem elipsoid o vMove, zwraca true jesli
     * obiekt moze byc przesuniety bez problemu do konca, w przeciwnym wypadku
     * zwraca w vMove wektor rzeczywistego przesuniecia
     * UWAGA: obiekt _nie jest_ przesuwany tylko jest testowana mozliwosc przesuniecia!!!!!
     * UWAGA: nie sa wolane zadne handlery kolizji, ani nie
     * sa zmieniane jakiekolwiek stany obiektow
     */
    public native boolean MoveByEllipsoidPure(Vector vMove);

    /** Przesuwa obiekt systemem elipsoid o vMove */
    public native void SetWorldPosByEllipsoid(Vector vMove);
    
    /** Przesuwa obiekt systemem elipsoid o vMove
     * UWAGA: przy przesuwaniu nie sa wolane zadne handlery kolizji, ani nie
     * sa zmieniane jakiekolwiek stany obiektow
     */
    public native void SetWorldPosByEllipsoidPure(Vector vMove);

    /** Powoduje, ze obiekt trzymany jest poza gridem. Obiekt taki jest zawsze
     * widoczny i zawsze sie renderuje
     */
    public native void OutOfGridObject();

    /** Ustawia nazwe pliku z tekstura cienia (bloba)
     * UWAGA! przestarzale - uzywac SetBlobMaterial
     */
    public native void SetBlobMapFile(String sFileName);

    /** Ustawia material bloba
     */
    public native void SetBlobMaterial(String sFileName);

    /** Ustawia rozmiar tekstury cienia (bloba)
     */
    public native void SetBlobMapSize(float fSize);

    /** Ustawia rotacje tekstury cienia (bloba)
     */
    public native void SetBlobMapRotation(float fDegrees);

    /** Ustawia zasieg w pionie wyciagania faceow, na ktore pada cien od bloba
     * domyslnie fUp = 50, fDown = 1000
     */
    public native void SetBlobMapYExtents(float fUp, float fDown);

    /** Ustawia stala orientacje bloba w swiecie. Blob nie obraca sie z obiektem. */
    public native void SetBlobFixedOrientation(boolean bFixed);

    /** Pobiera informacje czy blob ma stala orientacje w swiecie. */
    public native boolean GetBlobFixedOrientation();

    //////////////////////////////
    // ISTOTNOSC
    //////////////////////////////
    
    /** Pobiera nieistotnosc obiektu (nieistotnosc = 1 - istotnosc)
     * @return wartosci od 0 do 1: 0 - bardzo istotny, 1 - nieistotny
     */
    public native float GetUnImportance();
    
    /** Pobiera odleglosc obiektu od najblizszej kamery
     */
    public native float GetCameraDistance();

    /** Zwraca true jesli obiekt jest we frustumie jakiejs kamery
     */
    public native boolean IsInFrustum();

    /** Ustawia czynnik nieistotnosci obiektu, im wieksza jego wartosc,
     * tym szybciej obiekt staje sie nieistotny ze wzrostem odleglosci do kamery
     * @param wartosc od 0 do +inf, domyslnie 0.01,
     * 0 powoduje, ze obiekt jest zawsze istotny (nieistotnosc = 0)
     */
    public native void SetUnImportanceFactor(float fFactor);

    /** Pobiera czynnik nieistotnosci obiektu
     */ 
    public native float GetUnImportanceFactor();

    /** Ustawia odleglosc progowa do kamery, ponizej ktorej obiekt jest zawsze istotny
     * @param odleglosc - domyslnie 2000cm = 20m
     */
    public native void SetUnImportanceThreshold(float fFactor);

    /** Pobiera odleglosc progowa
     */ 
    public native float GetUnImportanceThreshold();
    
    /** Ustawia rozmiar obiektu, im wiekszy obiekt, tym bardziej istotny
     * @param wartosc od 0 do +inf, domyslnie 100 (dla MeshObjectow wartosc
     * ustawiana jest wg rozmiarow mesha)
     */
    public native void SetObjectSize(float fSize);
    
    /** Pobiera rozmiar obiektu
     */
    public native float GetObjectSize();


    /** Rejestruje obiekt w systemie okreslajacym iterakcje przy kolizjach
     */
    public final native void RegisterInInteractionMap();

    /** Odrejestrowuje obiekt w systemie okreslajacym iterakcje przy kolizjach
     */
    public final native void UnregisterInInteractionMap();

    /** Ustawia czy obiekt koliduje z obiektem Object, oba obiekty musza byc
     * wczesniej zarejestrowane w systemie interakcji
     * @param Object - drugi obiekt w interakcji
     * @param bCollide - true jesli maja kolidowac
     */
    public final native void SetInteraction(ControlObject Object, boolean bCollide);
    
    
    /**[native] Dodaje obiekt do modulu, w ktorym zostal on utworzony.
     * Obiekt dziala niezaleznie od widocznosci wg grida, tzn. obiekt
     * ZAWSZE uczestniczy we WSZYSTKICH operacjach (symulacja, rendering, kolizje, itp.)
	 */
    public native void ActivateOnModuleGridIndependent();
    
    /**[native] Usuwa obiekt niezalezny od grida z modulu */
    public native void DeactivateOnModuleGridIndependent();
    

    /**Zwraca string z wektorem polozenia, przydatne przy uzyciu konsoli javy*/
    String GetPositionString()
    {
        return GetPosition().toString();
    }
    
    String GetForwardString()
    {
        return GetForwardVector().toString();
    }

    String GetUpString()
    {
        return GetUpVector().toString();
    }

    /////////
    // otrzymywane handlery
    ////////
    
    /** Handler wolany za kazdym updejtem fizyki.
     * @param fAbsoulteTime czas symulacji bezwzgledny od poczatku gry
     * @param fSimInterval interwal czasowy o jaki posuwana jest symulacja
     */
    //void OnPhysicsUpdate(float fAbsoulteTime, float fSimInterval) {}


    /////////////////////////////////////////////////////////////////////////////////////////
    //  Interfejs do ODE - Open Dynamics Engine
    //  Obiekt musi uzywac fizyki typu ODE.
    /////////////////////////////////////////////////////////////////////////////////////////

    /******************
     * ODE: World 
     ******************/
    /** Ustawia przyspieszenie grawitacyjne w swiecie ODE [cm/s^2].*/
    public final native void ODE_SetWorldGravity( Vector vGravity );
   
	/** Pobiera przyspieszenie grawitacyjne ze swiata ODE [cm/s^2].*/
    public final native Vector ODE_GetWorldGravity();

    /** Ustawia Error Reduction Parameter dla calego systemu.
        Ta wartosc ustala jak mocno maja byc korygowane ewentualne bledy polaczen(jointow).
        Zakresem ERP jest <0..1> 
        0 - oznacza brak korekcji i powoduje ze obiekty moga od siebie oddryfowac po kilku krokach symulacji.
        1 - oznacza maksymalna korekcje i jest goraca niepolecany!
        domyslna wartoscia ERP jest 0.2 . 
        Zalecana maksymalna wartoscia jest 0.8 .
        Patrz dokument "ODE vX.XX User Guide".
    */
    public final native void ODE_WorldSetERP( float fERP );
    
	/** Zwraca aktualny Error Reduction Parameter.*/
    public final native float ODE_WorldGetERP();

    /** Ustawia Constraint Force Mixing dla calego systemu.
        Wartosc 0 usztywnia system i uniemozliwia nawet delikatne przekraczanie
        ogranicznikow(kolizja, jointy). W konsekwencji moze to niebezpiecznie rozchwiac system.
        Wartosci > 0 powoduja ze system staje sie bardziej elastyczny dopuszczajac do 
        drobnych przekroczen (wchodzenie bryly w bryle, przekraczanie ogranicznikow) co
        wbrew pozorom daje systemowi wieksza stabilnosc.
        
        Patrz dokument "ODE vX.XX User Guide".
    */
    public final native void ODE_WorldSetCFM( float fCFM );
    
	/** Zwraca aktualny Constraint Force Mixing dla calego systemu.*/
    public final native float ODE_WorldGetCFM();

    /******************
     * ODE: Object
     ******************/
    /** Okresla sposob w jaki ODE ma sterowac CControlObjectem.
    @param iCenterBody cialo sztywne potrzebne dla trybu kCenterBody.
    @see EODEControlMode*/
    public final native void ODE_SetControlMode( int eMode, int iCenterBody );

    /******************
     * ODE: Kolizje
     ******************/
	/** Czy obiekt koliduje z okreslonymi typami?*/
    public final native int ODE_GetCollisionFlags();
    public final native void ODE_SetCollisionFlags( int iFlags );
    public final native void ODE_ClearCollisionFlags( int iFlags );
    
	/** Czy obiekt chce byc informowany o kolizjach z okreslonymi typami?*/
    public final native int ODE_GetCollisionNotifyFlags();
    public final native void ODE_SetCollisionNotifyFlags( int iFlags );
    public final native void ODE_ClearCollisionNotifyFlags( int iFlags );

    /** Handler wywolywany dla obiektu uczestniczacego w kolizji.
    Niezbednym warunkiem otrzymania powiadomienia o kolizji jest
    wczesniejsze ustawienie odpowiedniej flagi za pomoca ODE_SetCollisionNotifyFlags(). 
    Domyslnym zachowaniem systemu ciala sztywnego jest odbicie sie od przeszkody.
    Mozna to zmienic wywolujac wewnatrz Handlera metode ODE_SetCollisionResponse().
    @param iOurGeom   identyfikator naszej bryly z ktora nastapila kolizja.
    @param obstacle przeszkoda na ktora natrafila bryla. Moze byc null w przypadku 
        terenu lub drzewa!
    @param iOtherGeom   identyfikator bryly w obstacle z ktora nastapila kolizja.
    @param eType    typ przeszkody na ktora natrafila bryla. @see EObstacleType
    @param vHitPoint    punkt trafienia we wspolrzednych swiata.
    UWAGA: parametry takie jak predkosc uderzenia mozna wyciagna z ODE
        za pomoca ODE_BodyGetRelPointVel().
    */
    public void OnCollisionODE( int iOurGeom, ControlObject obstacle, int iOtherGeom, int eType, Vector vHitPoint )    {}

    /** [native] Okresla jak ma zareagowac bryla sztywna na aktualnie zgloszona kolizje.
            @see EODECollisionResponse. 
    */
    public final native void ODE_SetCollisionResponse( int eCollisionGeomClass, int eResponse );

    /** [native] Zwraca jak ma zareagowac bryla sztywna na aktualnie zgloszona kolizje.
            @see EODECollisionResponse. 
    */
    public final native int ODE_GetCollisionResponse( int eCollisionGeomClass );

    /** [native] Okresla parametr reakcji na kolizje. Jesli np. reakcja jest odbicie to jest to 
     * wtedy wspolczynnik odbicia. Parametr przewaznie jest z przedzialu [0 - 1]
     */
    public final native void ODE_SetCollisionResponseParam( int eCollisionGeomClass, float fParamValue );

    /** [native] Zwraca parametr reakcji na kolizje. */
    public final native float ODE_GetCollisionResponseParam( int eCollisionGeomClass );

    /** [native] Ustawia domyslne reakcje na kolizje z roznymi rodzajami geometrii. */
    public final native void ODE_SetDefaultCollisionResponse();

    /******************
     * ODE: Body 
     ******************/
    /** Tworzy bryle sztywna (body).
        @param sName - nazwa bryly, musi byc niepowtarzalna!
        @return zwraca identyfikator stworzonej bryly.
    */
    public final native int ODE_BodyCreate( String sName );

    /** Usuwa z systemu bryle o podanym identyfikatorze.*/
    public final native void ODE_BodyDestroy( int iBody );

    /** Na podstawie nazwy znajduje identyfikator bryly, 
        @return zwraca 0 jezeli nie znalazl bryly o tej nazwie.*/
    public final native int ODE_BodyGetID( String sName );

    /** Nadaje cialu dowolna mase w dowolnym ksztalcie.
     * @param fMass - masa ciala
     * @param vCenterOfGravity - srodek ciezkosci w ukladzie lokalnym ciala
     * @param afInertiaTensorMatrix33 - maciez 3x3 momentu bezwladnosci w ukladzie lokalnym ciala
     * */
    public final native void ODE_BodySetMass( int iBody, float fMass );
    public final native void ODE_BodySetMassCenterOfGravity( int iBody, Vector vCenterOfGravity );
    public final native void ODE_BodySetMassInertiaTensor( int iBody, float [] afInetriaTensorMatrix33 );
    public final native float ODE_BodyGetMass( int iBody );
    public final native Vector ODE_BodyGetMassCenterOfGravity( int iBody );
    public final native void ODE_BodyGetMassInertiaTensor( int iBody, float [] afInetriaTensorMatrix33 );

    /** Nadaje cialu mase w ksztalcie kuli.*/
    public final native void ODE_MassSetSphere( int iBody, float fMass, float fRadius );
    
	/** Cylinder zakonczony polkulami.
        @param sName - nazwa bryly, musi byc niepowtarzalna!
        @param fMass masa;
        @param iDirection kierunek w jakim zwrocony jest cylinder. 1=x, 2=y, 3=z;
        @param fRadius  promien
        @param fLength  dlugosc
    */
    public final native void ODE_MassSetCappedCylinder( int iBody, float fMass, int iDirection, float fRadius, float fLength );

    /** Nadaje cialu mase w ksztalcie boxa.*/
    public final native void ODE_MassSetBox( int iBody, float fMass, Vector vBoxSizes );
    
    /** Przesuwa srodek ciezkosci o zadany wektor */
    public final native void ODE_MassTranslate( int iBody, Vector vMassTranslate );

    /** Wlacza i wylacza bryle. Wylaczone nie biora udzialu w symulacji.
        Uwaga: Jezeli wylaczona bryla jest podlaczona do bryly wlaczonej 
        to w nastepnym kroku symulacji zostanie automatycznie wlaczona!
        Nowo utworzone bryly sa domyslnie wlaczone.
    */
    public final native void ODE_BodyEnable( int iBody );
    public final native void ODE_BodyDisable( int iBody );
    public final native int ODE_BodyIsEnabled( int iBody );

	/** Pozycja ciala. */
    public final native void ODE_BodySetPosition( int iBody, Vector vPosition );
    public final native Vector ODE_BodyGetPosition( int iBody );

	/** Ustawia obroty. afMatrix to macierz 9 floatow. */
    public final native void ODE_BodySetRotation( int iBody, float [] afMatrix33 );
    public final native void ODE_BodyGetRotation( int iBody, float [] afMatrix33 );

	/** Alternatywne ustawianie obrotu przez macierz 4 floatow bedaca quaternionem (w, x, y, z).*/
    public final native void ODE_BodySetQuaternion( int iBody, float [] afQuaternion );
    public final native void ODE_BodyGetQuaternion ( int iBody, float [] afQuaternion );

    /** Aktualna szybkosc liniowa bryly */
	public final native void ODE_BodySetLinearVel( int iBody, Vector vVelocity );
    public final native Vector ODE_BodyGetLinearVel( int iBody );

    /** Aktualna szybkosc obrotowa bryly */
	public final native void ODE_BodySetAngularVel( int iBody, Vector vVelocity );
    public final native Vector ODE_BodyGetAngularVel( int iBody );

    /** Pobiera pozycje i obrot z meshElementu i ustawia je w bryle. */
    public final native void ODE_BodySynchronizeWithMeshElement( int iBody, int iElement );

    /** Pobiera pozycje i obrot z bryly i ustawia je meshElementowi. */
    public final native void ODE_MeshElementSynchronizeWithBody( int iBody, int iElement );

    /** Dodawanie roznych sil do bryly. 
     * Kolejno dodane sily sumuja sie i zostaja wyzerowane po wykonaniu kroku symulacji.
     * Jednostka jest N = [kg * cm / s*s]
     * ForceAtPos i ForceAtRelPos nadaja sile w okreslonym punkcie, pozostale dzialaja na srodku masy.    */
    public native void ODE_BodyAddForce( int iBody, Vector vForce );
	public native void ODE_BodyAddTorque( int iBody, Vector vTorque );
    public native void ODE_BodyAddRelForce( int iBody, Vector vRelForce);
    public native void ODE_BodyAddRelTorque( int iBody, Vector vRelTorque );
    public native void ODE_BodyAddForceAtPos( int iBody, Vector vForce, Vector vPosition );
    public native void ODE_BodyAddForceAtRelPos( int iBody, Vector vForce, Vector vPosition );
    public native void ODE_BodyAddRelForceAtPos( int iBody, Vector vForce, Vector vPosition );
    public native void ODE_BodyAddRelForceAtRelPos ( int iBody, Vector vForce, Vector vPosition );

	/** Ustawia wartosci akumulatorow sil dla ciala. */
	public native void ODE_BodySetForce( int iBody, Vector vForce );
	
    /** Ustawia wartosci akumulatorow momentu obrotowego dla ciala. */
    public native void ODE_BodySetTorque( int iBody, Vector vTorque );

	/** Pobiera wartosci z akumulatorow sil dla ciala. */
	public native Vector ODE_BodyGetForce( int iBody );
	
    /** Pobiera wartosci z akumulatorow momentu obrotowego dla ciala. */
    public native Vector ODE_BodyGetTorque( int iBody );

    /** Podaje globalna pozycje dowolonego punktu z lokalnego ukladu bryly.*/
    public final native Vector ODE_BodyGetRelPointPos( int iBody, Vector vLocalPosition );
    
    /** Podaje pozycje w lokalnym ukladzie ciala z globalnych wspolrzednych.*/
    public final native Vector ODE_BodyGetPosRelPoint( int iBody, Vector vGlobalPosition );
    	
	/** Podaje globalna predkosc dowolnego punktu z lokalnego ukladu bryly. */
    public final native Vector ODE_BodyGetRelPointVel( int iBody, Vector vLocalPosition );
    
    /** Podaje globalna predkosc dowolnego punktu w globalnych wspolrzednych. */
    public final native Vector ODE_BodyGetPointVel( int iBody, Vector vGlobalPosition );

	/** Dla podanego wektora we wspolrzednych lokalnych ciala podaje wektor obrocony we wspolrzednych globalnych. */
    public final native Vector ODE_BodyVectorToWorld( int iBody, Vector vLocalVector );
	
    /** Dla podanego wektora we wspolrzednych globalnych podaje wektor obrocony we wspolrzednych lokalnych ciala. */
    public final native Vector ODE_BodyVectorFromWorld( int iBody, Vector vGlobalVector );

	/// finite rotation mode
    /** 0 - infinite - domyslnie, 
      * 1 - finite - pilnowany w danej osi by nie przekraczal granic rozsadku.*/
    public final native void ODE_BodySetFiniteRotationMode( int iBody, int iMode );
    public final native int ODE_BodyGetFiniteRotationMode( int iBody );
	
    /** Ustawia os pilnowania przekroczenia predkosci obrotowej. Gdy jest ustawiona na (0,0,0) (domyslnie)
	 *  to predkosc obrotowa jest pilnowana w kazdym kierunku.
	 */
	public final native void ODE_BodySetFiniteRotationAxis( int iBody, Vector vAxis );
	public final native Vector ODE_BodyGetFiniteRotationAxis( int iBody );

	/** Ustawia oddzialywanie grawitacji na dane cialo.
	 * 0 - grawitacja wylaczona
	 * rozne od 0 - grawitacja wlaczona (domyslnie)
	 */
	public final native void ODE_BodySetGravityMode( int iBody, int iGravityMode );
	public final native int ODE_BodyGetGravityMode( int iBody);
	
    /******************
     * ODE: Geom 
     ******************/
    /** Wlacza i wylacza geometrie. Wylaczone nie biora udzialu w kolizjach. */
    public final native void ODE_GeomEnable( int iGeom );
    public final native void ODE_GeomDisable( int iGeom );
    public final native int ODE_GeomIsEnabled( int iGeom );

    /** Tworzy geometrie w ksztalcie sfery.
        @param sName - nazwa geometrii, musi byc niepowtarzalna!
        @param iSpace - przestrzen do ktorej ma nalezec geometria, ustawienie 0
            oznacza nie podpiecie do zadnej przestrzeni - geometria nie koliduje!!!.
        @return zwraca identyfikator stworzonej geometrii.
    */
    public final native int ODE_GeomCreateSphere( String sName, int iSpace, float fRadius );
    public final native void ODE_GeomSphereSetRadius( int iGeom, float fRadius );
    public final native float ODE_GeomSphereGetRadius( int iGeom );

    /** Zwraca glebokosc podanego punktu w sferze.
     * @param iGeom - geometria sfery 
     * @param vPoint - badany punkt
     * @return zaglebienie punktu - punkt w sferze ma dodatnia glebokosc, poza nia ujemna.
     */
    public final native float ODE_GeomSpherePointDepth( int iGeom, Vector vPoint );
    
    /** Tworzy geometrie w ksztalcie boxa.
        @param sName - nazwa geometrii, musi byc niepowtarzalna!
        @return zwraca identyfikator stworzonej geometrii.
    */
    public final native int ODE_GeomCreateBox( String sName, int iSpace, Vector vBoxSizes );
    public final native void ODE_GeomBoxSetLengths( int iGeom, Vector vLengths );
    public final native Vector ODE_GeomBoxGetLengths( int iGeom );

    /** Zwraca glebokosc podanego punktu w boxie.
     * @param iGeom - geometria boxa 
     * @param vPoint - badany punkt
     * @return zaglebienie punktu - punkt w boxie ma dodatnia glebokosc, poza nim ujemna.
     */
    public final native float ODE_GeomBoxPointDepth( int iGeom, Vector vPoint );

    /** Tworzy geometrie w ksztalcie plaszczyzny.
        @param sName - nazwa geometrii, musi byc niepowtarzalna!
        @param iSpace - przestrzen w jakiej nalezy umiescic geometrie;
        @param fA, fB, fC, fD - wspolczynniki rowania plaszczyzny Ax + By + Cz = D
        @return zwraca identyfikator stworzonej geometrii.
    */
    public final native int ODE_GeomCreatePlane( String sName, int iSpace, float fA, float fB, float fC, float fD );
    public final native void ODE_GeomPlaneSetParams( int iGeom, float fA, float fB, float fC, float fD );
    public final native void ODE_GeomPlaneGetParams( int iGeom, float fA, float fB, float fC, float fD );

    /** Zwraca glebokosc podanego punktu w plaszczyznie.
     * @param iGeom - geometria plaszczyzny 
     * @param vPoint - badany punkt
     * @return zaglebienie punktu - punkt pod/za plaszczyzna ma dodatnia glebokosc, nad/przed nia ujemna.
     */
    public final native float ODE_GeomPlanePointDepth( int iGeom, Vector vPoint );

    /** Tworzy geometrie w ksztalcie cylindra zakonczonego polkulami.
        jest ustawiony wzdluz osi Y.
        @param sName - nazwa geometrii, musi byc niepowtarzalna!
        @param iSpace - przestrzen w jakiej nalezy umiescic geometrie;
        @param fRadius - promien walca i jednoczesnie promien polkul
        @param fLength - dlugosc walca
        @return zwraca identyfikator stworzonej geometrii.
    */
    public native int ODE_GeomCreateCappedCylinder( String sName, int iSpace, float fRadius, float fLength );
    public native void ODE_GeomCappedCylinderSetParams( int iGeom, float fRadius, float fLength );
    public native void ODE_GeomCappedCylinderGetParams( int iGeom, float fRadius, float fLength );

    /** Zwraca glebokosc podanego punktu w CCylindrze - walcu z dwoma polkulami.
     * @param iGeom - geometria plaszczyzny 
     * @param vPoint - badany punkt
     * @return zaglebienie punktu - punkt w bryle to dodatnia glebokosc, poza nia ujemna
     */
    public final native float ODE_GeomCappedCylinderPointDepth( int iGeom, Vector vPoint );

    /** Tworzy geometrie promienia. Ma ona troszke inna interpretacje i nie reprezentuje fizycznego obiektu 3D,
     * jak to mam miejsce dla pozostalych geometrii. Jest to nieskonczenie cienka linia startujaca z punktu
     * pozycji geometrii o dlugosci fLength i ulozona na poczatku wzdluz osi Y ukladu globalnego.
        @param sName - nazwa geometrii, musi byc niepowtarzalna!
        @param iSpace - przestrzen w jakiej nalezy umiescic geometrie;
        @param fLength - dlugosc 
        @return zwraca identyfikator stworzonej geometrii.
    */
    public native int ODE_GeomCreateRay( String sName, int iSpace, float fLength );
    public native void ODE_GeomRaySetLength( int iGeom, float fLength );
    public native float ODE_GeomRayGetLength( int iGeom );
    public native void ODE_GeomRaySetParams( int iGeom, Vector vPosition, Vector vDirection );
    public native void ODE_GeomRayGetParams( int iGeom, Vector vPosition, Vector vDirection );

    /** Tworzy geometrie Transform mogaca zawierac w sobie inna geometrie z uwzglednieniem jej pozycji i rotacji.
        @param sName - nazwa geometrii, musi byc niepowtarzalna!
        @param iSpace - przestrzen w jakiej nalezy umiescic geometrie;
        @return zwraca identyfikator stworzonej geometrii.
    */
    public native int ODE_GeomCreateTransform( String sName, int iSpace );
    public native void ODE_GeomTransformSetGeom( int iTransformGeom, int iGeom );
    public native int ODE_GeomTransformGetGeom( int iTransformGeom );
   
    /** Ustawia tryb czyszczenia geometrii Transform. 
     * 0 - po zniszczeniu geometrii Transform geometria bedaca w jej posiadaniu zostaje (domyslnie)
     * 1 - wraz z geometria Transform niszczona jest geometria wewnetrzna
     */
    public native void ODE_GeomTransformSetCleanup( int iGeom, int iMode );
    public native int ODE_GeomTransformGetCleanup( int iGeom );
    
    /** Ustawia tryb "informowania" geometrii Transform. 
     * 0 - podczas kolizji zwracana jest geometria wewnetrzna
     * 1 - podczas kolizji zwracana jest geometria Transform
     */
    public native void ODE_GeomTransformSetInfo( int iGeom, int iMode );
    public native int ODE_GeomTransformGetInfo( int iGeom );
    
    /** Usuwa z systemu geometrie o podanym identyfikatorze.*/
    public final native void ODE_GeomDestroy( int iGeom );

    /** Na podstawie nazwy znajduje identyfikator geometrii, 
        @return zwraca -1 jezeli nie znalazl geoemtrii o tej nazwie.*/
    public final native int ODE_GeomGetID( String sName );

    /** Podaje boxa AABB otaczajacego geometrie badz przestrzen. */
    public final native void ODE_GeomGetAABB( int iGeom, Vector vMinAABB, Vector vMaxAABB );

    /** Podczepia geometrie pod bryle. Od tej chwili zmiana pozycji lub
    rotacji bryly powoduje odpowiednia zmiane w geometrii i vice versa.
    */
    public final native void ODE_GeomSetBody( int iGeom, int iBody );

    /** Na podstawie nazwy znajduje identyfikator bryly, 
        @return zwraca bryle do ktorej podpieta jest geometria.*/
    public final native int ODE_GeomGetBody( int iGeom );

    /** Pozycja geometrii */
    public native void ODE_GeomSetPosition( int iGeom, Vector vPosition );
    public native Vector ODE_GeomGetPosition( int iGeom );

	/** Obrot geometrii */
    public native void ODE_GeomSetRotation( int iGeom, float [] afMatrix33);
    public native void ODE_GeomGetRotation( int iGeom, float [] afMatrix33);

    /** Podaje czy dana geometria jest przestrzenia */
    public final native int ODE_GeomIsSpace( int iGeom );

    /** POdaje klase geometrii @see EODEGeomClass */
    public final native int ODE_GeomGetClass( int iGeom );

    /** Do jakiej kategorii kolizyjnej nalezy geometria.
     * @param iBits - 32 bitowa mapa bitowa kategorii
     * */
    public native void ODE_GeomSetCategoryBits( int iGeom, int iBits );
    public native int ODE_GeomGetCategoryBits( int iGeom );

    /** Z obiektami jakich kategorii chce kolidowac geometria.
     * @param iBits - 32 bitowa mapa bitowa kategorii
     */
    public native void ODE_GeomSetCollideBits( int iGeom, int iBits );
    public native int ODE_GeomGetCollideBits( int iGeom );

    /** Synchronizuje rozmiar geometrii z meshem .
     */
    public native void ODE_GeomSizesAdjustWithMesh( int iGeom, int bWithMass );
    
    /******************
     * ODE: Space 
     ******************/
    /** przestrzenie kolizyjne - umozliwiaja okreslenie grup obiektow ktore ze soba nie koliduja.
        W systemie jest zdefiniowana przestrzen kolizji w ktorej powinny znalezc sie wszystkie
        geometrie/przestrzenie zdefiniowane przez uzytkownika.
    */
    public final native int ODE_SpaceGetGlobal();

    /** Tworzy nowa przestrzen. Geometrie nalezace do tej samej przestrzeni nie koliduja ze soba.
     * @param iSpace - (jezeli != 0) przestrzen rodzic, pod ktorego podpinana jest tworzona przestrzen.
     * @return nowa utworzona przestrzen
     */
    public final native int ODE_SimpleSpaceCreate( String sName, int iSpace );

    /** Ustawia tryb czyszczenia przestrzeni.
     * @param iMode:
     * 1 - po zniszczeniu przestrzeni niszczone sa wszystkie geometrie wchodzace w jej sklad (domyslnie)
     * 0 - po zniszczeniu przestrzeni wszystkie geometrie wchodzace w jej sklad zostaja
     */
    public final native void ODE_SpaceSetCleanup( int iSpace, int iMode );
    public final native int ODE_SpaceGetCleanup( int iSpace );

    /** Tworzy nowa przestrzen. Geometrie nalezace do tej samej przestrzeni nie koliduja ze soba.
     * Do pewnego stopnia przestrzenie i geometrie sa wymienne, ale nie wolno podpinac przestrzeni
     * do bryly sztywnej! Jezeli chcesz by geometrie z danej przestrzeni byly podpiete do bryly 
     * to musisz podpiac je pojedynczo.
     * @param iSpace - (jezeli != 0) przestrzen rodzic, pod ktorego podpinana jest tworzona przestrzen.
     * @return nowa utworzona przestrzen
    */
    public final native int ODE_HashSpaceCreate( String sName, int iSpace );

    /** Ustawia parametry przestrzeni hashujacej. 
     * Najmniejszy i najwiekszy rozmiar komorki to odpowiednio 2^iMinLevel i 2^iMaxLevel
     */
    public final native void ODE_HashSpaceSetLevels( int iSpace, int iMinLevel, int iMaxLevel );
    
    /** Usuwa przestrzen wraz z wszystkimi geometriami w niej zawartymi.
    */
    public final native void ODE_SpaceDestroy( int iSpace );

    /** Dodaj do przestrzeni nowa geometrie/podprzestrzen .
     * Jedna geometria nie moze nalezec do wiecej niz jednej przestrzeni na raz.
     * Jezeli dodawana geometria jest juz w jakiejs przestrzeni nic sie nie dzieje.
	*/
    public final native void ODE_SpaceAdd( int iSpace, int iGeom );
    
	/** Usuwa  z przestrzeni geometrie/podprzestrzen.
	 * Jezli geometria iGeom nie nalezala do przestrzeni iSpace nic sie nie dzieje.
	*/
    public final native void ODE_SpaceRemove( int iSpace, int iGeom );

	/** Sprawdza czy podana geometria iGeom znajduje sie w przestrzeni iSpace.
	*/
    public final native int ODE_SpaceQuery( int iSpace, int iGeom );

    /** Na podstawie nazwy znajduje identyfikator przestrzeni, 
        @return zwraca -1 jezeli nie znalazl przestrzeni o tej nazwie.
	*/
    public final native int ODE_SpaceGetID( String sName );

    /** Zwraca liczbe geometrii zawartych w przestrzeni iSpace.
	*/
    public final native int ODE_SpaceGetNumGeoms( int iSpace );
    
    /** Zwraca z przestrzeni geometrie o podanym indeksie.
     * @param iIndex - numer kolejny geometrii (od 0 do ODE_SpaceGetNumGeoms() - 1)
     */
    public final native int ODE_SpaceGetGeom( int iSpace, int iIndex );

    /******************
     * ODE: Joint 
     ******************/
    /** Joint reprezentuje polaczenie pomiedzy dwiema brylami.
     *  Jointy moga byc laczone w grupy dla szybszego usuwania. */
    /** Zwraca ilosc jointow podlaczonych do bryly.*/
    public native int ODE_BodyGetNumJoints( int iBody );

    /** Zwraca identyfikator jointa.
     * @param iIndex - numer jointa od 0 do n-1 (n = ODE_BodyGetNumJoints()) */
    public native int ODE_BodyGetJoint( int iBody, int iIndex );

    /** Zwraca ID jointa o nazwie sName.*/
    public final native int ODE_JointGetID( String sName );

    /** Niszczy jointa, odlacza go z bryl do ktorych jest przyczepiony, usuwa ze swiata.
        Uwaga: jezeli usuwany joint nalezy do grupy funkcja nie dziala, najpierw nalezy
        taka grupe wyzerowac lub zniszczyc. */
    public final native void ODE_JointDestroy( int iJoint );

    /** polacz Podanym Jointem 2 ciala. Jezeli Joint byl polaczony wczesniej
     * z jakims cialem to najpierw zostanie od niego odlaczony. */
    public final native void ODE_JointAttach( int iJoint, int iBody1, int iBody2 );
    
	/** Zwraca typ jointa @see EODEJointTypes */
    public final native int ODE_JointGetType( int iJoint );
    
	/** Zwraca pierwsza lub druga bryle ktora spina joint.
        @param iIndex 0-pierwsza bryla, 1-druga bryla
        @return zwraca id bryly, jezeli zwroci 0 oznacza ze joint ta strona jest podlaczony 
        do statycznego swiata. */
    public final native int ODE_JointGetBody( int iJoint, int iIndex );
    
	/** Zwraca true jezeli dwa ciala sa poloaczone jointem.*/
    public final native int ODE_AreConnected( int iBody1, int iBody2 );

    /** Zwraca true jezeli dwa ciala sa polaczone jakims jointem nie typu iJointTypeExclude. 
	 * @param iJointTypeExclude - @see EODEJointTypes
	 */
    public final native int ODE_AreConnectedExcluding( int iBody1, int iBody2, int iJointTypeExclude );

    // Joint: BALL
    /** Tworzy polaczenie kulkowe. */
    public final native int ODE_JointCreateBall( String sName, int iGroup );
    public final native void ODE_JointSetBallAnchor( int iJoint, Vector vAnchor );
    public final native Vector ODE_JointGetBallAnchor( int iJoint );
    
    // Joint: HINGE
	/** Tworzy polaczenie zawiasowe.*/
    public final native int ODE_JointCreateHinge( String sName, int iGroup );
    public final native void ODE_JointSetHingeAnchor( int iJoint, Vector vAnchor );
    public final native Vector ODE_JointGetHingeAnchor( int iJoint );
    public final native void ODE_JointSetHingeAxis( int iJoint, Vector vAxis );
    public final native Vector ODE_JointGetHingeAxis( int iJoint );
    public final native float ODE_JointGetHingeAngle( int iJoint );
    public final native float ODE_JointGetHingeAngleRate( int iJoint );
    public final native void ODE_JointSetHingeParam( int iJoint, int iParameter, float fValue );
    public final native float ODE_JointGetHingeParam( int iJoint, int iParameter );
    
    // Joint: SLIDER
	/** Tworzy polaczenie suwakowe.*/
    public native int ODE_JointCreateSlider( String sName, int iGroup );
    public final native void ODE_JointSetSliderAxis( int iJoint, Vector vAxis );
    public final native Vector ODE_JointGetSliderAxis( int iJoint );
    public final native float ODE_JointGetSliderPosition( int iJoint );
    public final native float ODE_JointGetSliderPositionRate( int iJoint );
    public final native void ODE_JointSetSliderParam( int iJoint, int iParameter, float fValue );
    public final native float ODE_JointGetSliderParam( int iJoint, int iParameter );
    
    // Joint: UNIVERSAL
	/** Tworzy polaczenie 2-osiowe. */
    public native int ODE_JointCreateUniversal( String sName, int iGroup );
    public final native void ODE_JointSetUniversalAnchor( int iJoint, Vector vAnchor );
    public final native Vector ODE_JointGetUniversalAnchor( int iJoint );
    public final native void ODE_JointSetUniversalAxis1( int iJoint, Vector vAxis1 );
    public final native Vector ODE_JointGetUniversalAxis1( int iJoint );
    public final native void ODE_JointSetUniversalAxis2( int iJoint, Vector vAxis2 );
    public final native Vector ODE_JointGetUniversalAxis2( int iJoint );

    // Joint: HINGE2
	/** Tworzy wachacz dla kola samochodu. */
    public native int ODE_JointCreateHinge2( String sName, int iGroup );
    public final native void ODE_JointSetHinge2Anchor( int iJoint, Vector vAnchor );
    public final native Vector ODE_JointGetHinge2Anchor( int iJoint );
    public final native void ODE_JointSetHinge2Axis1( int iJoint, Vector vAxis );
    public final native Vector ODE_JointGetHinge2Axis1( int iJoint );
    public final native void ODE_JointSetHinge2Axis2( int iJoint, Vector vAxis );
    public final native Vector ODE_JointGetHinge2Axis2( int iJoint );
    public final native float ODE_JointGetHinge2Angle1( int iJoint );
    public final native float ODE_JointGetHinge2Angle1Rate( int iJoint );
    public final native float ODE_JointGetHinge2Angle2Rate( int iJoint );
    public final native void ODE_JointSetHinge2Param( int iJoint, int iParameter, float fValue );
    public final native float ODE_JointGetHinge2Param( int iJoint, int iParameter );

    // Joint: FIXED
    /** Tworzy polaczenie sztywne */
    public native int ODE_JointCreateFixed( String sName, int iGroup );
	public final native void ODE_JointSetFixed( int iJoint );

    // Joint: AMOTOR
	/** Tworzy naped kontaktu */
    public native int ODE_JointCreateAMotor( String sName, int iGroup );
	public final native void ODE_JointSetAMotorParam( int iJoint, int iParameter, float fValue );
    public final native float ODE_JointGetAMotorParam( int iJoint, int iParameter );

    /** Tryb napedu:
     * @see EODEAMotorMode
     * _AMOTOR_USER - osie motoru i katy jointa sa kontrolowane przez uzytkownika
     * _AMOTOR_EULER - system liczy automatycznie katy Eulera. Liczona jest tez os obrotu.
     * Ten tryb wymaga odpowiedniego ustawienia osi. 
     */
	public final native void ODE_JointSetAMotorMode( int iJoint, int iMode );
    public final native int ODE_JointGetAMotorMode( int iJoint );

    /** Ustawia/Pobiera liczbe osi kontrolowanych przez Angular Motor. */
	public final native void ODE_JointSetAMotorNumAxes( int iJoint, int nNum );
	public final native int ODE_JointGetAMotorNumAxes( int iJoint );
	
	/** Ustawia os AMotora:
	 * @param iAxisNum - numer osi
	 * @param iRel - 0 - os jest zakotwiczona do swiata globalnego
	 *		         1 - os jest zakotwiczona do ciala 1
	 *               2 - os jest zakotwiczona do ciala 2
	 * @param vAxis - wektor osi
	 */
	public final native void ODE_JointSetAMotorAxis( int iJoint, int iAxisNum, int iRel, Vector vAxis );
	public final native Vector ODE_JointGetAMotorAxis( int iJoint, int iAxisNum );
	public final native int ODE_JointGetAMotorAxisRel( int iJoint, int iAxisNum );
	
	/** Ustawia kat dla podanej osi. */
    public final native void ODE_JointSetAMotorAngle( int iJoint, int iAxisNum, float fAngle );
    public final native float ODE_JointGetAMotorAngle( int iJoint, int iAxisNum );
    public final native float ODE_JointGetAMotorAngleRate( int iJoint, int iAxisNum );

    /******************
     * ODE: Joint group
     ******************/
    /** Zwraca ID grupy jointow o nazwie sName.*/
    public final native int ODE_JointGroupGetID( String sName );

    /** Dla unikalnej nazwy tworzy grupe jointow i zwraca jej identyfikator */
	public native int ODE_JointGroupCreate( String sName );
    
	/** Zwraca identyfikator grupy jointow dla nazwy */
	public native int ODE_JointGroupGetId( String sName );
	
	/** Zniszcz grupe i wszystkie nalezace do niej jointy.*/
    public native void ODE_JointGroupDestroy( int iGroup );
    
	/** Zniszcz wszystkie jointy nalezace do grupy i zostaw grupe.*/
    public native void ODE_JointGroupEmpty( int iGroup );
	
    /** Zwraca grupe jointow kolizyjnych(kontaktowych) ktora jest czyszczona 
    przed rozpoczeciem testow kolizji.*/
    public native int ODE_JointGroupGetCollision();

    /******************
     * ODE: inne
     ******************/
	/** Renderowanie geometrii - w tej chwili obslugiwany jest box, sphere i CappedCylinder */
	public final native void ODE_RenderGeom( int iGeom );

    /** Ustawia tryb automatycznego updateowania fizyki ODE (z poziomu levelu) */
	public final native void ODE_SetAutoUpdate( int iAutoUpdate );
    
    /** Zwraca czy jest ustawiony automatyczny tryb updateowania fizyki ODE (z poziomu levelu) */
	public final native int ODE_GetAutoUpdate();
    
    /** Wykonuje krok symulacji */
    public final native void ODE_WorldUpdate();

    /** Ustawia krok symulacji */
    public final native void ODE_SetStepSize( float fStep );

    /** Zwraca krok symulacji */
    public final native float ODE_GetStepSize();
    
    /** Rysuje punkt o ustalonym kolorze i alphie */
    public final native void ODE_RenderPoint( Vector vPoint, Vector vColor, float fAlpha );
    
    /** Rysuje linie o ustalonym kolorze i alphie */
    public final native void ODE_RenderLine( Vector vBegin, Vector vEnd, Vector vColor, float fAlpha );

    public final native void ODEGWIgnorePositionChange(boolean bIgnore);
    public final native void ODE_DisableODEGWUntilInputImpulse();
    public final native void ODE_EnableODEGWAfterInputImpulse();
    public final native void SetODEGWPos(Vector vPos);
    public final native void SetODEGWLinVel(Vector vLinVel);
    public final native void SetODEGWForce(Vector vForce);
    public final native void SetODEGWFDir1(Vector vFDir1);
    public final native Vector GetODEGWPos();
    public final native Vector GetODEGWLinVel();
    public final native Vector GetODEGWForce();
    public final native Vector GetODEGWFDir1();
    public final native Vector GetJumpSpeed();
    public final native void SetODEGWVelocityAccumulator(Vector vVelAccu);
    public final native Vector GetODEGWVelocityAccumulator();
    
    public final native void SetODEGWStateFParam1(float fParam1);
    public final native float GetODEGWStateFParam1();
    public final native void SetODEGWStateFParam2(float fParam2);
    public final native float GetODEGWStateFParam2();
    public final native void SetODEGWStateFParam3(float fParam3);
    public final native float GetODEGWStateFParam3();
    public final native void SetODEGWStateFParam4(float fParam4);
    public final native float GetODEGWStateFParam4();
    public final native void SetODEGWStateFParam5(float fParam5);
    public final native float GetODEGWStateFParam5();

    public final native void SetODEGWStateBParam1(boolean bParam1);
    public final native boolean GetODEGWStateBParam1();
    public final native void SetODEGWStateBParam2(boolean bParam2);
    public final native boolean GetODEGWStateBParam2();
    public final native void SetODEGWDoneUpdates(int iODEDoneUpdatesCount);
    public final native int GetODEGWDoneUpdates();
    public final native void EnableODEPhysics(int iEnable);
    
    // ustawienie parametru float do celow testowych
    public final native void SetTestParam(float fTestValue);
    
    /////////////////////////////////////////////////////////////////////////////
    // Obsluga akcji act_Appear
    protected boolean bDisappearGridIndependent = false;
    public void Disappear(ControlObject cDestinationObj)
    {
        if (cDestinationObj != null && 
            cDestinationObj.IsActiveOnModule())
        {
            FromUpForwardPos(cDestinationObj.GetUpVectorVolatile(),
                             cDestinationObj.GetForwardVectorVolatile(),
                             cDestinationObj.GetPositionVectorVolatile());
        }
        bDisappearGridIndependent = IsGridIndependent();
        DeactivateOnModuleAlways();
    }

    public void Appear(ControlObject cDestinationObj)
    {
        if (cDestinationObj != null && 
            cDestinationObj.IsActiveOnModule())
        {
            FromUpForwardPos(cDestinationObj.GetUpVectorVolatile(),
                             cDestinationObj.GetForwardVectorVolatile(),
                             cDestinationObj.GetPositionVectorVolatile());
        }
        if (bDisappearGridIndependent)
            ActivateOnModuleGridIndependent();
        else
            ActivateOnModule();
    }
    /////////////////////////////////////////////////////////////////////////////////////////

    //////////////////////////
    // MAPA TYPU TERENU
    //////////////////////////

    /** Ustawia typ terenu na pozycji obiektu
     * @param wartosc do ustawienia
     */
    public void SetTerrainType(int value)
    {
       Vector pos = GetPositionVectorVolatile();
       SetTerrainType(pos.fX, pos.fZ, value);
    }

    /** Pobiera typ terenu na pozycji obiektu
     * @return typ terenu lub -1 gdy pozycja poza mapa
     */
    public int GetTerrainType()
    {
       Vector pos = GetPositionVectorVolatile();
       return GetTerrainType(pos.fX, pos.fZ);
    }

    /** Sprawdza czy na pozycji obiektu jest dany typ terenu
     * Przy sprawdzaniu wykonywane jest filtrowanie wartosci 
     * (percentage closer filtering)
     * @param wartosc do sprawdzenia
     * @return -1 gdy pozycja poza mapa, 0 gdy nie ma danej wartosci, 1 gdy jest
     */
    public int GetTerrainTypeSmooth(int value)
    {
       Vector pos = GetPositionVectorVolatile();
       return GetTerrainTypeSmooth(pos.fX, pos.fZ, value);
    }	
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
	public native void LoadLocalMatrix(FileChunk cFC);
	public native void SaveLocalMatrix(FileChunk cFC);
	public native void LoadWorldMatrix(FileChunk cFC);
	public native void SaveWorldMatrix(FileChunk cFC);	
	public native void LoadPathFinder(FileChunk cFC);
	public native void SavePathFinder(FileChunk cFC);	
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[ControlObject.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
		
        if (cFC.GetID() == ESGChunks._CHK_CONTROL_OBJECT)
        {						
			super.SGLoadChunk(cFC);
			
			ControlObject cParent = (ControlObject)cFC.LoadObjectRef();
			ControlObject cCurrentParent = GetParent();						

			//Vector vLocal = cFC.LoadVector();		
												
			m_vVolPosition = cFC.LoadVector();
			m_vVolSpeed = cFC.LoadVector();
			m_vVolLocalSpeed = cFC.LoadVector();
			m_vVolForward = cFC.LoadVector();
			m_vVolRight = cFC.LoadVector();
			m_vVolUp = cFC.LoadVector();
			
			if (IsUpdatingPathMap())
				UpdatePathMap(false);			
			
			// musi by przed oczytaniem macierzy lokalnej
			if (cCurrentParent != null)
				DetachFromParent();				
			
			if (cParent != null)
				AttachAsChildTo(cParent);									
						
			LoadLocalMatrix(cFC);
			
			// odswiezenie hierarchii po ustaleniu pozycji
			if (cParent != null)
				DetachFromParentAndKeepWorldPos();				

			if (cParent != null)
				AttachAsChildToAndKeepWorldPos(cParent);												
																					
			if (GetParent() != null)
				FileChunk.Log("[ControlObject.SGLoadChunk] Parent Name = <"+GetParent().GetName()+"> ClassName = <"+GetParent().getClass().getName()+">\n");			
						
			//SetPosition(vLocal);
			
			LoadPathFinder(cFC);
						
            bDisappearGridIndependent = cFC.LoadBool(); 
			
			SetFlags(cFC.LoadInt());
		}
		
		cFC.delete();
		
		FileChunk.Log("[ControlObject.SGLoadChunk] - END\n");
	}
	
    /** Metoda zapisujaca dane do Save gracza
     */		
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[ControlObject.SGSaveChunk]\n");
		
		FileChunk cFC = NewChunk(ESGChunks._CHK_CONTROL_OBJECT, cFCParent);
		
		super.SGSaveChunk(cFC);
		        
		cFC.SaveObjectRef(GetParent());
				
		if (GetParent() != null)
			FileChunk.Log("[ControlObject.SGSaveChunk] Parent Name = <"+GetParent().GetName()+"> ClassName = <"+GetParent().getClass().getName()+">\n");
		
		//cFC.SaveVector(GetLocalPosition());		
		
		cFC.SaveVector(m_vVolPosition);
		cFC.SaveVector(m_vVolSpeed);
		cFC.SaveVector(m_vVolLocalSpeed);
		cFC.SaveVector(m_vVolForward);
		cFC.SaveVector(m_vVolRight);
		cFC.SaveVector(m_vVolUp);
		
		SaveLocalMatrix(cFC);
		SavePathFinder(cFC);
				
        cFC.SaveBool(bDisappearGridIndependent);
		
		cFC.SaveInt(GetFlags());			
		
		ControlObject[] aObjs = GetChildren();				
		
		if (aObjs != null)
			for(int i=0; i < aObjs.length; i++) 
				aObjs[i].bSG = true;								
		        
		cFC.delete();			
	}	
}


