import java.lang.reflect.*;

/** Klasa obsluguje debugowanie Javy
 */
public class Debug
{
    /** Metoda pomocnicza do CompleteFieldMember
     */
    private static void GetClassFieldsNames(Class cClass, String sMethodToComplete, java.util.Vector aNames)
    {
        // dokladamy membery z klasy bierzacej
        java.lang.reflect.Field [] aFields = cClass.getDeclaredFields();
        int nMethodLen = sMethodToComplete.length();
        for(int i=0; i < aFields.length; i++)
        {
            //GameObject.LogD("Memeber <"+aFields[i].getName()+"> completing <"+sMethodToComplete+">\n");
            if( sMethodToComplete.regionMatches(true, 0, aFields[i].getName(), 0, nMethodLen) )
            {
                aNames.addElement(new String(aFields[i].getName()));
            }
        }
        
        Class cSuperClass = cClass.getSuperclass();
        if(cClass != Object.class)
        {
            GetClassFieldsNames(cSuperClass, sMethodToComplete, aNames);
        }
    }
    
    public static boolean CheckClassMember(String sClassName, String sMemberName)
    {
        try {
            Class cClass = Class.forName(sClassName);
            return CheckClassMember(cClass, sMemberName);
        }        
        catch (ClassNotFoundException e)
        {
            GameObject.LogD("No such class <"+sClassName+">\n");
            return false;
        } 
    }
    
    /** Sprawdza, czy podany member jest polem klasy
     */
    public static boolean CheckClassMember(Class cClass, String sMemberName)
    {
        //GameObject.LogD(cClass.getName() + " " + sMemberName + "\n");
        try {
            java.lang.reflect.Field field = cClass.getDeclaredField(sMemberName);
        }
        catch (NoSuchFieldException e)
        {
            Class cSuperClass = cClass.getSuperclass();
            if(cClass != Object.class)
                return CheckClassMember(cSuperClass, sMemberName);
            else
                return false;
        }
        catch (SecurityException e)
        {
            GameObject.LogD("Security exception\n");
            return false;
        }
        return true;
    }
    
    public static String[] CompleteFieldMember(String sClassName, String sMethodToComplete)
    {
        try {
            Class cClass = Class.forName(sClassName);
            return CompleteFieldMember(cClass, sMethodToComplete);
        }
        catch (ClassNotFoundException e)
        {
            GameObject.LogD("No such class <"+sClassName+">\n");
            return null;
        }
    }
    
    /** Metoda zwraca nazwy zmiennych pasujacych 
     * @param sClassName nazwa klasy w ktorej jest memeber     
     * @param sMethodToComplete poczatek nazwy memebera, ktora ma byc dokonczona
     * @return zwraca tablice stringow z nazwami, ktore pasuja
     */
    public static String[] CompleteFieldMember(Class cClass, String sMethodToComplete)
    {
        java.util.Vector aNames = new java.util.Vector(0);
        
        // szukamy w hierarchii
        GetClassFieldsNames(cClass, sMethodToComplete, aNames);
        
        // zwracamy tablice ze stringami
        String[] aRet = new String[aNames.size()];
        aNames.copyInto(aRet);
        return aRet;
    }

    ///////////////////////////////////////////////////////
    // METHODS
    ///////////////////////////////////////////////////////
    
    
    public static String[] CompleteMethodMember(String sClassName, String sMethodToComplete)
    {
        try {
            Class cClass = Class.forName(sClassName);
            return CompleteMethodMember(cClass, sMethodToComplete);
        }
        catch (ClassNotFoundException e)
        {
            GameObject.LogD("No such class <"+sClassName+">\n");
            return null;
        }
    }
    
    public static String[] CompleteMethodMember(Class cClass, String sMethodToComplete)
    {
        java.util.Vector aNames = new java.util.Vector(0);
        
        // szukamy w hierarchii
        GetClassMethodsNames(cClass, sMethodToComplete, aNames);
        
        // zwracamy tablice ze stringami
        String[] aRet = new String[aNames.size()];
        aNames.copyInto(aRet);
        return aRet;
    }

    private static void GetClassMethodsNames(Class cClass, String sMethodToComplete, java.util.Vector aNames)
    {
        // dokladamy metody z klasy bierzacej
        Method [] aMethods = cClass.getDeclaredMethods();
        int nMethodLen = sMethodToComplete.length();
        for(int i=0; i < aMethods.length; i++)
        {
            //GameObject.LogD("Memeber <"+aFields[i].getName()+"> completing <"+sMethodToComplete+">\n");
            if( sMethodToComplete.regionMatches(true, 0, aMethods[i].getName(), 0, nMethodLen) )
            {
                aNames.addElement(new String(aMethods[i].getName()));
            }
        }
        
        Class cSuperClass = cClass.getSuperclass();
        if(cClass != Object.class)
        {
            GetClassMethodsNames(cSuperClass, sMethodToComplete, aNames);
        }
    }

    public static String[] GetMethodParameters(Class cClass, String sName, int iNumParams)
    {
        Method [] aMethods = cClass.getDeclaredMethods();
        
        for (int i = 0; i < aMethods.length; i++)
        {
            if (aMethods[i].getName().equals(sName))
            {
                Class[] aParams = aMethods[i].getParameterTypes();
                if (aParams.length != iNumParams) continue;
                
                Class cRetVal = aMethods[i].getReturnType();

                String[] aRet = new String[aParams.length + 1];
                aRet[aParams.length] = cRetVal.toString();
                
                for (int j = 0; j < aParams.length; ++j)
                {
                    aRet[j] = aParams[j].toString();
                }
                
                return aRet;
            }
        }
        
        Class cSuperClass = cClass.getSuperclass();
        if (cClass != Object.class)
        {
            return GetMethodParameters(cSuperClass, sName, iNumParams);
        }
        
        return null;
    }
}
