
/** Flagi ustalajace parametry kolizji.
 * Uwaga: Definicje musza byc zgodne z definicjami w Collision.h
 */

public class ECollFlags
{
    // typ kolizji
    final static int OBJCOL_TYPE_ELLIPSOID       = 0x00000000; // default 
    final static int OBJCOL_TYPE_PERFACE         = 0x00000001;
    final static int OBJCOL_TYPE_PERFACEOBJECT   = 0x00000002;

    // jak kontynuowa ruch po kolizji
    final static int OBJCOL_ACTION_STOP          = 0x00000000;
    final static int OBJCOL_ACTION_SLIDE         = 0x00000010; // default 
    final static int OBJCOL_ACTION_BOUNCE        = 0x00000020;
    final static int OBJCOL_ACTION_GRAVITYSTEP   = 0x00000030; // don't use (engine's internal usage only)

    // jaki handler woa przy kolizji
    final static int OBJCOL_HANDLER_NONE         = 0x00000000; // default 
    final static int OBJCOL_HANDLER_CALL         = 0x00000100;
    final static int OBJCOL_HANDLER_OBJECT       = 0x00000200;
    final static int OBJCOL_HANDLER_FULLINFO     = 0x00000300;
    final static int OBJCOL_HANDLER_FULLINFOMOVE = 0x00000400;

    // FLAGI
    
    // z czym obiekt nie koliduje (domyslnie koliduje ze wszystkim)
    final static int OBJCOL_FLAG_NOTERRAIN   = 0x00001000;
    final static int OBJCOL_FLAG_NOOBJECTS   = 0x00002000;
    final static int OBJCOL_FLAG_NOTREES     = 0x00004000;

    // czy obiekt ma sie ustawiac wzgledem normalnej do terenu
    final static int OBJCOL_FLAG_ALIGNTOGROUND  = 0x00008000;

    // czy ma by wolany handler zmiany nachylenia terenu
    // dziala tylko w fizyce WALK
    final static int OBJCOL_FLAG_SLOPEHANDLER = 0x00010000;

	// gdy ustawiona jest ta flaga, obiekt nie blokuje ruchu, ale 
	// wolany jest handler wystapienia kolizji
    final static int OBJCOL_FLAG_ONHITHANDLER = 0x00020000;

    // normalna zwracana przez handlery jest normaln na elipsoidzie
    final static int OBJCOL_FLAG_ELLNORMAL    = 0x00040000;
    
    // obiekt jest automatycznie przylaczany do obiektu, na ktorym stoi
    final static int OBJCOL_FLAG_AUTOATTACH   = 0x00080000;
    
    // powoduje, ze obiekt zeslizguje sie (na gravitystep)
    final static int OBJCOL_FLAG_SLIPDOWN     = 0x00100000;
    
    // obiekt z ta flaga blokuje ruch tylko wg mapy interakcji
    final static int OBJCOL_FLAG_INTERACTIONONLY = 0x00200000;
    
    // badane sa kolizje na drodze obiektu od pozycji w poprzedniej
    // ramce do pozycji aktualnej
    // przydatne dla obiektow podlaczonych pod poruszajace sie obiekty
    final static int OBJCOL_FLAG_CHECKFROMLASTPOS = 0x00400000;

    // przy wlaczonej fladze OBJCOL_FLAG_AUTOATTACH wylacza detachowanie
    // obiektu, kiedy nie jest wykryta zadna kolizja (np. kiedy obiekt leci)
    final static int OBJCOL_FLAG_DONTDETACHINAIR = 0x00800000;
    
    // blokuje ruch obiektu, jezeli typ terenu na nowej pozycji jest rowny 0
    final static int OBJCOL_FLAG_TESTTERRAINTYPE = 0x01000000;
    
    // wlacza automatyczne podlaczanie obiektu do portal objectow
    // UWAGA: nie wiadomo jak wplywa na obiekty w sieci (not tested)
    final static int OBJCOL_FLAG_PORTALATTACH = 0x02000000;
    
    // do obiektow z ta flaga nic sie nie bedzie autoattachowac
    final static int OBJCOL_FLAG_NONAUTOATTACHABLE = 0x04000000;
        
    // do obiektow z ta flaga particle nie bedza sie autoattachowac
    final static int OBJCOL_FLAG_PARTICLENONAA = 0x08000000;
        
    static String GetFlagName(int nFlag)
    {
        String sRet = "__UNKNOWN FLAG____";
        switch (nFlag)
        {
            case OBJCOL_HANDLER_CALL:
                sRet = "OBJCOL_HANDLER_CALL";
                break;
            case OBJCOL_HANDLER_OBJECT:
                sRet = "OBJCOL_HANDLER_OBJECT";
                break; 
            case OBJCOL_HANDLER_FULLINFO:
                sRet = "OBJCOL_HANDLER_FULLINFO";
                break;
            case OBJCOL_HANDLER_FULLINFOMOVE:
                sRet = "OBJCOL_HANDLER_FULLINFOMOVE";
                break;
            case OBJCOL_FLAG_NOTERRAIN:
                sRet = "OBJCOL_FLAG_NOTERRAIN";
                break; 
            case OBJCOL_FLAG_NOOBJECTS:
                sRet = "OBJCOL_FLAG_NOOBJECTS";
                break; 
            case OBJCOL_FLAG_NOTREES:
                sRet = "OBJCOL_FLAG_NOTREES";
                break; 
            case OBJCOL_FLAG_AUTOATTACH:
                sRet = "OBJCOL_FLAG_AUTOATTACH";
                break; 
            case OBJCOL_FLAG_ALIGNTOGROUND:
                sRet = "OBJCOL_FLAG_ALIGNTOGROUND";
                break; 
            case OBJCOL_FLAG_SLOPEHANDLER:
                sRet = "OBJCOL_FLAG_SLOPEHANDLER";
                break; 
            case OBJCOL_FLAG_INTERACTIONONLY:
                sRet = "OBJCOL_FLAG_INTERACTIONONLY";
                break; 
            case OBJCOL_FLAG_ONHITHANDLER:
                sRet = "OBJCOL_FLAG_ONHITHANDLER";
                break; 
            case OBJCOL_FLAG_ELLNORMAL:
                sRet = "OBJCOL_FLAG_ELLNORMAL";
                break; 
            case OBJCOL_TYPE_ELLIPSOID:
                sRet = "OBJCOL_TYPE_ELLIPSOID or OBJCOL_ACTION_STOP or OBJCOL_HANDLER_NONE";
                break;
            case OBJCOL_TYPE_PERFACE:
                sRet = "OBJCOL_TYPE_PERFACE";
                break;
            case OBJCOL_TYPE_PERFACEOBJECT:
                sRet = "OBJCOL_TYPE_PERFACEOBJECT";
                break;
            case OBJCOL_ACTION_SLIDE:
                sRet = "OBJCOL_ACTION_SLIDE";
                break;
            case OBJCOL_ACTION_BOUNCE:
                sRet = "OBJCOL_ACTION_BOUNCE";
                break;
            case OBJCOL_ACTION_GRAVITYSTEP:
                sRet = "OBJCOL_ACTION_GRAVITYSTEP";
                break;
        }
        return sRet;
    }
    
}
