class FreeCamera extends Camera
{
    private static Module _cObjectNameModule = null;
    
    public static Module GetObjectNameModule()
    {
        if (_cObjectNameModule == null)
        {
            _cObjectNameModule = (Module)Game.cSingleton.CreateObject("Module");
        }
        return _cObjectNameModule;
    }
    
    static FreeCamera cCam = null;
    public MeshObject target = null;

    // stany nacisniecia klawiszy
    int iFwd, iSide, iRotZ, iFOV;

    // wartosci dotyczace ruchu
    float fRotH, fRotV;
    float fRotZ;
    
    // predkosc kamery
    float fSpeed = 500;

    float fFOV = 60;

    float m_fMouseSensitivity = 2.0f;
    
    // predkosc ruchu
    float fAddSpeedX = 0.0f;
    float fAddSpeedZ = 0.0f;
    float fAddSpeedDelta = 100.0f;

    float fAddRotV = 0.0f;
    float fAddRotH = 0.0f;
    float fAddRotDelta = 0.05f;
    
    boolean bShiftL = false;
    boolean bShiftR = false;

    boolean bControlL = false;
    boolean bControlR = false;
    
    public static FreeCamera GetCamera(GameObject cObj)
    {
        if(cCam==null)
        {
            cCam = (FreeCamera)cObj.CreateObject("FreeCamera");
        }
        return cCam;
    }

    public void OnCreate()
    {
		if (GetModule() != null)
		{
			Vector vPos = GetPosition();
			vPos.fY = GetModule().GetWaterLevel() + 10.f;
			SetPosition(vPos);
		}
    }
    
    void OnActivate()
    {
        //CrashLog("on activate z kamery\n");
        // poczatkowe ustawienie widoku
        //FromForwardUpPos(new Vector(0.5f,0.0f,0.5f), new Vector(0,1.0f,0), new Vector(0, 3000.0f, 0));

        RegisterKeyRequest();
        RegisterMouseAxisRequest();

        fLastTime = 0;

        fFOV = 60;
    }

    void OnDeactivate()
    {
        UnregisterKeyRequest();
        UnregisterMouseAxisRequest();
    }

    float fLastTime = 0f;

    void SetPos(float fX, float fY, float fZ)
    {
        Vector v = new Vector(fX, fY, fZ);
        SetPosition(v);
    }
    
    void AlignY()
    {
        Vector vFwd = GetForwardVector();
        vFwd.fY = 0;
        FromForwardUpPos(vFwd, Vector.Up, GetPosition());
    }
    
    void OnPostUpdate()
    {
        float fTime = GetAbsoluteTime();
        float fDeltaTime = fTime - fLastTime;
        fLastTime = fTime;

        if (fLastTime == 0)
            return;

        Vector vFwd = GetForwardVector();
        Vector vRight = GetRightVector();
        Vector vPos = GetPosition();

        if (iFwd != 0)
            vPos.Add(vFwd.Mul2(iFwd*fSpeed*fDeltaTime));
        if (iSide != 0)
            vPos.Add(vRight.Mul2(iSide*fSpeed*fDeltaTime));

        // dodajemy predkosc ruchu
        vPos.Add(vFwd.Mul2(fAddSpeedZ*fDeltaTime));
        vPos.Add(vRight.Mul2(fAddSpeedX*fDeltaTime));

        if (target != null)
        {
            //compute fwd from pos to target
            vFwd = vPos.Sub2(target.GetPosition());
        }

        FromForwardUpPos(vFwd, Vector.Up, vPos);

        if (target == null)
        {
            fRotZ += fDeltaTime * iRotZ * 20;
            Rotate(fRotZ, 0, 0, 1);

            if (fRotH != 0)
            {
                Rotate(fRotH / 10f, 0, 1, 0);
                fRotH = 0f;
            }
            if (fRotV != 0)
            {
                Rotate(fRotV / 10f, 1, 0,0);
                fRotV = 0f;
            }
            
            // dodajemy predkosc obrotowa
            if(fAddRotH!=0) Rotate(fAddRotH, 0, 1, 0);
            if(fAddRotV!=0) Rotate(fAddRotV, 1, 0,0);
        }
        
        fFOV += iFOV * fDeltaTime * 20;
        if(fFOV < 0.1f) fFOV = 0.1f;
        if(fFOV > 175.0f) fFOV = 175.0f;
        SetFOV(fFOV);

    }

    void OnInputKey(int nKeyCode, boolean bKeyState, char nKeyAscii)
    {
        if(nKeyCode==EKey._LSHIFT) bShiftL = bKeyState;
        if(nKeyCode==EKey._RSHIFT) bShiftR = bKeyState;
        if(nKeyCode==EKey._LCONTROL) bControlL = bKeyState;
        if(nKeyCode==EKey._RCONTROL) bControlR = bKeyState;
        
        if (nKeyCode == EKey._N)   
        {
            Game.bCanShowObjectNames = true;
            //nazwa obiektu 
            ShowObjectsNames(bKeyState, false);
        }
        
        if (nKeyCode == EKey._B)
        {
            Game.bCanShowObjectNames = true;
            //nazwa obiektu ze wspolrzednymi
            ShowObjectsNames(bKeyState, true);
        }
        
        if (bKeyState)
        {
            if(bShiftL || bShiftR)
            {
                switch(nKeyCode)
                {
                case EKey._W:
                case EKey._UP: fAddSpeedZ-=fAddSpeedDelta; break;
                case EKey._S:
                case EKey._DOWN: fAddSpeedZ+=fAddSpeedDelta; break;
                case EKey._A:
                case EKey._LEFT : fAddSpeedX-=fAddSpeedDelta; break;
                case EKey._D:
                case EKey._RIGHT : fAddSpeedX+=fAddSpeedDelta; break;
                case EKey._DECIMAL: fAddSpeedX = 0.0f; fAddSpeedZ = 0.0f; break;
                }
            }
            else
            {
                if(bControlL || bControlR)
                {
                    switch(nKeyCode)
                    {
                    case EKey._UP: fAddRotV+=fAddRotDelta; break;
                    case EKey._DOWN: fAddRotV-=fAddRotDelta; break;
                    case EKey._LEFT : fAddRotH+=fAddRotDelta; break;
                    case EKey._RIGHT : fAddRotH-=fAddRotDelta; break;
                    case EKey._DECIMAL: fAddRotH = .0f; fAddRotV = .0f; break;
                    }
                }
                else
                {
                    switch(nKeyCode)
                    {
                    case EKey._UP: iFwd = -1; break;
                    case EKey._DOWN: iFwd = 1; break;
                    case EKey._LEFT : iSide = -1; break;
                    case EKey._RIGHT : iSide = 1; break;
                    }
                }
            }
            
            
            switch(nKeyCode)
            {
            case EKey._NUMPAD1 : iRotZ = 1; break;
            case EKey._NUMPAD3 : iRotZ = -1; break;                                     
            case EKey._MINUS : iFOV = -1; break;
            case EKey._EQUALS  : iFOV = 1; break;
            case EKey._COLON : fSpeed *= 0.5; break;
            case EKey._COMMA : fSpeed *= 2; break;
            }
        }
        else
        {
            if(bShiftL || bShiftR)
            {
            }
            else
            {
                switch(nKeyCode)
                {
                case EKey._UP:
                case EKey._DOWN: iFwd = 0; break;
                case EKey._NUMPAD1 :
                case EKey._NUMPAD3 : iRotZ = 0; break;
                case EKey._LEFT :
                case EKey._RIGHT : iSide = 0; break;
                case EKey._MINUS :
                case EKey._EQUALS  : iFOV = 0; break;
                }
            }
        }
    }

        
    /**Okrag w ktorym bedziemy szukac obiektow do pokazania ich nazwy*/
    float fRadiusObjectNames = 1000f;
    
    /**Pokazuje/ukrywa sprite'y z nazwami widocznych obiektow
     * @param bShow jesli true to pokazac, wpp ukryc
     * @param bPosition jesli true to pokazuje tez pozycje obiektu w swiecie*/
    void ShowObjectsNames(boolean bShow, boolean bPosition)
    {
        if (bShow)
            GetObjectNameModule().Activate();
        else
            GetObjectNameModule().Deactivate();
        
        Module mod = GetModule();
        if (mod == null)
            mod = Game.GetSShotModule();
        if (mod != null)
        {    
            GameObject obj[] = mod.GetObjectsInRadiusFromClass(GetPosition(), fRadiusObjectNames, "MeshObject");
            if (obj != null)
                for (int i = 0; i < obj.length; i++)
                {
                    ((MeshObject)obj[i]).ShowObjectName(bShow, bPosition, this);
                }
        }
    }
    
    void OnInputMouseAxis(int nAxis, int nDelta)
    {
        switch (nAxis)
        {
        case EMouse._AXIS_X :
            fRotH += nDelta * m_fMouseSensitivity;
            break;
        case EMouse._AXIS_Y :
            fRotV += nDelta * m_fMouseSensitivity;
            break;
        case EMouse._AXIS_Z:
            {
                Vector CameraPos=GetPosition();
                Vector Forward=GetForwardVector();
                Vector Right=GetRightVector();

                // zmieniamy pozycje kamery
                Forward.Mul(nDelta);
                CameraPos.Add( Forward );

                // ustawiamy odpowiednio camere
                FromForwardUpPos(GetForwardVector(), Vector.Up, CameraPos);
            }
            break;
        }
    }
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[FreeCamera.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunks._CHK_FREE_CAMERA)
        {
			super.SGLoadChunk(cFC);
			
			// MeshObject target;
			target = (MeshObject)cFC.LoadObjectRef();							
			
			// stany nacisniecia klawiszy
			iFwd = cFC.LoadInt();
			iSide = cFC.LoadInt();
			iRotZ = cFC.LoadInt();
			iFOV = cFC.LoadInt();
				
			// wartosci dotyczace ruchu
			fRotH = cFC.LoadFloat();
			fRotV = cFC.LoadFloat();
			fRotZ = cFC.LoadFloat();
    
			// predkosc kamery
			fSpeed = cFC.LoadFloat();
		
			fFOV = cFC.LoadFloat();
		
			m_fMouseSensitivity = cFC.LoadFloat();
    
			// predkosc ruchu
			fAddSpeedX = cFC.LoadFloat();
			fAddSpeedZ = cFC.LoadFloat();
			fAddSpeedDelta = cFC.LoadFloat();

			fAddRotV = cFC.LoadFloat();
			fAddRotH = cFC.LoadFloat();
			fAddRotDelta = cFC.LoadFloat();
    
			bShiftL = cFC.LoadBool();
			bShiftR = cFC.LoadBool();

			bControlL = cFC.LoadBool();
			bControlR = cFC.LoadBool();			
        }
		
		cFC.delete();
	}
			
    /** Metoda zapisujaca dane do Save gracza
     */		
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[FreeCamera.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunks._CHK_FREE_CAMERA, cFCParent);

		super.SGSaveChunk(cFC);
		
		// MeshObject target;
		cFC.SaveObjectRef(target);			
		
		// stany nacisniecia klawiszy
		cFC.SaveInt( iFwd );
		cFC.SaveInt( iSide );
		cFC.SaveInt( iRotZ );
		cFC.SaveInt( iFOV );
			
		// wartosci dotyczace ruchu
		cFC.SaveFloat( fRotH );
		cFC.SaveFloat( fRotV );
		cFC.SaveFloat( fRotZ );
    
		// predkosc kamery
		cFC.SaveFloat( fSpeed );
		
		cFC.SaveFloat( fFOV );
		
		cFC.SaveFloat( m_fMouseSensitivity );
    
		// predkosc ruchu
		cFC.SaveFloat( fAddSpeedX );
		cFC.SaveFloat( fAddSpeedZ );
		cFC.SaveFloat( fAddSpeedDelta );

		cFC.SaveFloat( fAddRotV );
		cFC.SaveFloat( fAddRotH );
		cFC.SaveFloat( fAddRotDelta );
    
		cFC.SaveBool( bShiftL );
		cFC.SaveBool( bShiftR );

		cFC.SaveBool( bControlL );
		cFC.SaveBool( bControlR );
							
		cFC.delete();
	}
}