class Cursor extends Sprite
{
	Sprite cursorTex = null;
    int iTexNum = 0;
    Sprite toolTip = null;
    String sToolTipTex = null;
    float fToolTipMargin = 2f;
    Vector vCursorSize = new Vector();    
    Module module = null;
    
    void OnCreate()
    {
        ProcessCapturedMouseMove(true);
    }
    
    /**Ustawia kolor tekstu w tooltipie
     * @param fR wartosc czerwonej skladowej koloru (0..1)
     * @param fG wartosc zielonej skladowej koloru (0..1)
     * @param fB wartosc niebieskiej skladowej koloru (0..1)*/
    void SetToolTipTextColor(float fR, float fG, float fB)
    {
        if (toolTip != null)
        {
            toolTip.SetTextColor(fR, fG, fB, 1);        
        }
        else
        {
            //LogLine("Cursor.SetToolTipTextColor : musisz najpierw utworzyc sprite tooltipa!!!");
            CrashLog("Cursor.SetToolTipTextColor : musisz najpierw utworzyc sprite tooltipa!!!");
        }
           
    }
    /**Tworzy okno dla tooltipa, ustawia wszystkie parametry
     * @param mod modul w ktorym jest kursor
     * @param sTexName nazwa pliku z tekstura tla dla tooltipa, moze byc null, wtedy nie ma tla
     * @param rTexPos pozycja w teksturze
     * @param sFontID identyfikator fonta, ktory ma byc uzyty, jesli null to wybrany zostanie "MS Sans Serif" 12
     * @param fR wartosc czerwonej skladowej koloru tekstu (0..1)
     * @param fG wartosc zielonej skladowej koloru tekstu (0..1)
     * @param fB wartosc niebieskiej skladowej koloru tekstu (0..1) 
     * @param fA wartosc alfy (przezroczystosci tekstu) (0 - niewidoczny..1 - nieprzezroczysty)
     * @see Fonts*/
    void InitToolTip(Module mod, String sTexName, Rect rTexPos, String sFontID, float fR, float fG, float fB, float fA)
	{
        module = mod;
        if (toolTip == null)
        {
    		toolTip = (Sprite)mod.CreateObject("Sprite");
	    	if (sTexName != null)
            {
                if (sTexName.length() > 0)
                {
                    iTexNum = cursorTex.AddTexture(sTexName, true, true);
	    	        toolTip.SetTexture(iTexNum, rTexPos);
                }
            }
		    toolTip.SetSize(new Vector(0,0,0));
		    toolTip.SetPos(new Vector(0,0,0));
		    toolTip.ProcessMouse(false);
		    cursorTex.AddChild(toolTip);
            toolTip.MoveTop();
        }
        else
        {
            iTexNum = cursorTex.AddTexture(sTexName, true, true);
		    toolTip.SetTexture(iTexNum, rTexPos);
        }
        if (cursorTex != null)
            vCursorSize = cursorTex.GetSize();
        toolTip.SetTextPosFlags(ESpriteText.HORIZ_MIDDLE|ESpriteText.VERT_MIDDLE);
        if (sFontID == null) toolTip.SetFont(Fonts.FONT_MSSANSSERIF_12);
        else toolTip.SetFont(sFontID);
        toolTip.SetTextColor(fR, fG, fB, fA);
    }
    
	void SetCursorTex(Module mod, String textureName)
	{
        module = mod;
		cursorTex = (Sprite)mod.CreateObject("Sprite");
		iTexNum = cursorTex.AddTexture(textureName, true, true);
		cursorTex.SetTexture(iTexNum, new Rect(0, 0, 1, 1));
		cursorTex.SetSize(new Vector(32,32,0));
		cursorTex.SetPos(new Vector(0,0,0));
		cursorTex.ProcessMouse(false);
		cursorTex.Show();
		mod.AddSprite(cursorTex);
        cursorTex.MoveTop();
        vCursorSize = cursorTex.GetSize();
        
		ProcessMouse(true);
		SetPos(new Vector(0,0,0));
		SetSize(new Vector(GetScreenWidth(), GetScreenHeight(), 0));
		mod.AddSprite(this);
	}
    
    /**Nalezy wolac ta funkcje przy zmianie rozdzielczosci ekranu*/
    void OnChangeResolution()
    {
        SetSize(new Vector(GetScreenWidth(), GetScreenHeight(), 0));
    }
    
    void OnDestroy()
    {
        if (cursorTex != null)
        {
            if ( module != null )
                module.RemoveSprite( cursorTex );
            
            cursorTex.ReleaseTexture(iTexNum);
            //cursorTex.delete();
            cursorTex=null;
        }
        
        if (toolTip != null)
        {
            if ( module != null )
                module.RemoveSprite( toolTip );
            //toolTip.delete();
            toolTip = null;
        }
        
        if ( module != null )
             module.RemoveSprite( this );
    }

	void OnMouseMove(float fMouseX, float fMouseY, int iButtonsFlags)
	{
		cursorTex.SetPos(new Vector(fMouseX, fMouseY, 0.0f));
	}
	
	public void Hide()
	{
		cursorTex.Hide();	
	}
	
	public void Show()
	{
		cursorTex.Show();	
	}
    
    public void MoveTop()
    {
        super.MoveTop();
        if (toolTip != null)
           toolTip.MoveTop();
        if (cursorTex != null)
           cursorTex.MoveTop();
    }
    
    /**Wyswietla tooltip. 
     * @param sText wyswietlany tekst
     * @see Window#SetToolTip(String sText)*/
    void DisplayToolTip(String sText)
    {
        float fScreenWidth = 0f;
        float fScreenHeight = 0f;
        if (sText.length() > 0)
        {
            Vector vCursorPos = cursorTex.GetPos();
            Vector vToolTipPos = new Vector(vCursorPos.fX + vCursorSize.fX, vCursorPos.fY + vCursorSize.fY,0);
            fScreenWidth = GetScreenWidth();
            fScreenHeight = GetScreenHeight();
            Vector vTextSize = toolTip.GetTextExtent(sText);
            vTextSize.fX += 2f * fToolTipMargin;
            vTextSize.fY += 2f * fToolTipMargin;
            toolTip.SetSize(vTextSize);
            if ((vToolTipPos.fY + vTextSize.fY) > fScreenHeight)
            {
                vToolTipPos.fY -= vCursorSize.fY;
            }
            if ((vToolTipPos.fX + vTextSize.fX) > fScreenWidth)
            {
                vToolTipPos.fX = fScreenWidth - vTextSize.fX;
            }
            toolTip.SetPos(vToolTipPos);
            
            toolTip.DisplayText(sText);
            toolTip.Show();        
                
        }
        else
            toolTip.Hide();
    }
    
    public void SetPos(Vector vPos)
    {
        SetCursorPos(vPos);
        if (cursorTex != null)
            cursorTex.SetPos(vPos);
    }
    
    void Destroy()
    {
        if (module != null)
        {
            if (toolTip != null)
            {
                module.RemoveSprite(toolTip);
                toolTip.delete();
                toolTip = null;
            }
            if (cursorTex != null)
            {
                module.RemoveSprite(cursorTex);
                cursorTex.delete();
                cursorTex = null;
            }
        }
    }
	
	Vector GetPointerPos()
	{
		if (cursorTex != null)
			return cursorTex.GetPos();
		else
			return null; 
	}
}