/**Klasa reprezentujaca lisboxa z pojedyncza selekcja, rozmiar elementow ustalany jest na podstawie
 * pojemnosci i wielkosci listy. Jesli chcemy miec elementy o zadanym rozmiarze, to trzeba pokombinowac.
 */
public class ListBox extends Button 
{
    /**Numer tekstury tla*/
    int nBkgTex = 0;
    /**Wektor elementow listboxa*/
    java.util.Vector vItems = new java.util.Vector();
    /**Indeks elementu, ktory jest na poczatku listy*/
    int nFirstVisible = 0;
    /**Pojemnosc listy (Liczba widocznych elementow)*/
    int nCapacity = 0;
    /**Wielkosc elementow listy*/
    Vector vElemSize = new Vector();
    /**Rozmiar pola listy (bez scrollbara)*/
    Vector vClientSize = new Vector();
    /**Przycisk strzalki w gore*/
    Button btnUpArrow = null;
    /**Przycisk strzalki w dol*/
    Button btnDnArrow = null;
	/**Pasek przewijania*/
	Button btnScrollBar = null;
    /**Ilosc elementow na liscie*/
    int nItemCount = 0;
    /**Ustala, czy lista ma automatycznie pokazywac i chowac scrollbar zaleznie od ilosci elementow*/
    boolean bAutoShowScroll = false;
    /**Jesli zmieni sie ten vektor to Itemy beda przesuniety o podany vektor*/
    Vector vItemsFirstPos = new Vector();
    
    boolean bScrollVisible = false;
    
    /**Ustala, czy scrollbar ma sie pokazywac i chowac automatycznie zaleznie od ilosci elementow na liscie
     */
    void SetAutoShowScroll(boolean bAuto)
    {
        bAutoShowScroll = bAuto;
    }
    
    /**Zwraca ilosc elementow w liscie.*/
    int GetItemCount()
    {
        return nItemCount;
    }
    
    /**Ustala ilosc widocznych elementow. Na jej podstawie i 
     * na podstawie rozmiaru okna wyliczany jest rozmiar elementu*/
    void SetCapacity(int nNewCapacity)
    {
        nCapacity = nNewCapacity;
        vElemSize.fY = vClientSize.fY / (float) nNewCapacity;
        vElemSize.fX = vClientSize.fX; 
        for (int i = 0; i < nItemCount; i++) 
            ((Window) vItems.elementAt(i)).SetSize(vElemSize);
    }
    
    /**Inicjuje listboxa*/
    void Init(UIElement data, Module module)
    {
		vItemsFirstPos = new Vector(0, 3, 0);
		
        SetModule(module);
        nID = data.nID;
        nBtnType = data.nFlags & EButtonFlags.BTN_TYPE_MASK;
        if ((data.nFlags & EButtonFlags.BTN_STATE_MASK) != 0)
        {
            nBtnState = data.nFlags & EButtonFlags.BTN_STATE_MASK;
        }
        if (data.sUncheckedTex.length()>0) 
        {
            nUncheckedTex = AddTexture(data.sUncheckedTex, true, true);
            SetTexture(nUncheckedTex, data.rUncheckedTex);
        }
        ProcessMouse(true);
        SetRelativePos(data.vPos);
        CreateScrollBar();
        SetSize(data.vSize);
        UpdateList();
        Show();
    }
    
    /**Ta metode nalezy przeciazyc w klasach potomnych i ustalac wlasne tekstury i rozmiary
     * scrollbara.
     */
    void CreateScrollBar()
    {
        btnUpArrow = (Button) CreateObject("Button");
        btnUpArrow.Init(EUI.ScrollUpArrow, module);
        AddChild(btnUpArrow);
        AddChildWnd(btnUpArrow, wndRoot);
                
        btnDnArrow = (Button) CreateObject("Button");
        btnDnArrow.Init(EUI.ScrollDnArrow, module);
        AddChild(btnDnArrow);
        AddChildWnd(btnDnArrow, wndRoot);
		
		btnScrollBar = (Button)CreateObject("Button");
		btnScrollBar.Init( new UIElement(0,0,
										 new Vector(0.0f,0.0f,0.0f),
										 new Vector(32.0f,32.0f,0.0f),
										 "Data/Menu/checkbox.png",
										 "Data/Menu/checkbox.png",
										 "Data/Menu/checkbox.png",
										 "Data/Menu/checkbox.png",
										 EButtonFlags.BTN_TYPE_PRESSBUTTON,
										 ""),
						   module);
		AddChild(btnScrollBar);
        AddChildWnd(btnScrollBar, wndRoot);
    }
	
    void OnRBUp(float fMouseX, float fMouseY){}
    void OnRBDown(float fMouseX, float fMouseY){}
    //void OnLBUp(float fMouseX, float fMouseY){}
    //void OnLBDown(float fMouseX, float fMouseY){}
    
    /**Dodaje do listboxa element.
     * @return indeks dodanego elementu*/
    int AddItem(Button btnItem)
    {
        if (btnItem == null)
			return -1;
        vItems.addElement(btnItem);
        AddChildWnd(btnItem, wndRoot);
        AddChild((Sprite)btnItem);
        btnItem.SetSize(vElemSize);
        nItemCount = vItems.size();
        UpdateList();
        return vItems.indexOf((Object) btnItem);            
    }
    
    /**Wstawia dany element na podana pozycje
     * @param btnItem wstawiany element
     * @param nIndex pozycja do wstawienia
     * @return pozycja na ktora byl wstawiony*/
    int InsertItem(Button btnItem, int nIndex)
    {
        if ( (btnItem == null) || (nIndex < 0))
			return -1;
        vItems.insertElementAt(btnItem, nIndex);
        AddChildWnd(btnItem, wndRoot);
        AddChild((Sprite)btnItem);
        btnItem.SetSize(vElemSize);
        nItemCount = vItems.size();
        UpdateList();
        return vItems.indexOf(btnItem);  
    }
    
    /**Usuwa element o indeksie <code>nIndex</code>*/
    void RemoveItem(int nIndex)
    {
        if ( (nIndex < 0) || (nIndex >= nItemCount) )
			return;
        Window wnd = (Window)vItems.elementAt(nIndex);
        wnd.Hide();
        RemoveSprite(wnd);
        RemoveChildWnd(wnd);
        wnd.Destroy();
        vItems.removeElementAt(nIndex);
        wnd.CallOnFrameEnd("delete");
        int nLastFirstVisible = nItemCount - nCapacity;
        if ( (nIndex == nFirstVisible) || (nFirstVisible > nLastFirstVisible) )
        {
            if (nFirstVisible > 0) nFirstVisible--;
        }
        nItemCount = vItems.size();
        UpdateList();
    }
    
    /**Zaznacza wybrany element, kasuje zaznaczenie pozostalych.
     * @param nIndex indeks parametru do zaznaczenia, jesli -1, nic
     * nie zaznaczone*/
    void SetSelection(int nIndex)
    {
        int i;
        String sClassName = "";
        for(i = 0; i < nItemCount; i++)
        {
            ((Button)vItems.elementAt(i)).SetCheck(false);
        }
        if ((nIndex > -1) && (nIndex < nItemCount))
        {
            ((Button)vItems.elementAt(nIndex)).SetCheck(true);
        }
    }
    
    /**Zwraca indeks zaznaczonego elementu, -1 jesli nic nie zaznaczone.*/
    int GetSelection()
    {
        int i;
        boolean bCheck = false;
        for(i = 0; i < nItemCount; i++)
        {
            bCheck = ((Button)vItems.elementAt(i)).GetCheck();
            if (bCheck) return i;
        }
        return -1;
    }
    /**Zwraca indeks szukanego elementu.
     * @param btnToFind szukany element
     * @return indeks pierwszego wystapienia <code>btnToFind</code> lub
     * -1 jesli nie znajdzie*/
    int FindItem(Button btnToFind)
    {
        return (vItems.indexOf(btnToFind));
    }
    
    void OnButton(Button btnSender)
    {
        if (btnSender.nBtnType == EButtonFlags.BTN_TYPE_LISTBOX_ITEM) 
        {
            //TODO jesli konieczny multiselect to zmienic
            int nIndex = FindItem(btnSender);
            //Log("indeks: " + nIndex +"\n");
            SetSelection(nIndex);
            if (wndParent != null)
            {
                wndParent.CallOnButton(btnSender);
                return;
            }
        }
        switch(btnSender.nID)
        {
            case EUI.ID_SCROLL_DN:
                OnScrollDown();
                break;
            case EUI.ID_SCROLL_UP:
                OnScrollUp();
                break;
        }
    }
    
    void OnScrollDown()
    {
        //ostatni element ktory moze byc na szczycie listy
        int nLastFirstVisible = nItemCount - nCapacity;
        if (nFirstVisible < nLastFirstVisible) 
        {
            nFirstVisible++;
            UpdateList();
        }
    }
    
    void OnScrollUp()
    {
        if (nFirstVisible > 0) 
        {
            nFirstVisible--;
            UpdateList();
        }
    }
    /**Ustala rozmiar listboxa, pozycjonuje scrollbara i wylicza rozmiar
     * elementow biorac pod uwage ustalona pojemnosc listy.*/
    public void SetSize(Vector v)
    {
        super.SetSize(v);
		
        Vector arrowSize = btnUpArrow.GetSize();
        Vector pos = GetPos();
        Vector size = GetSize();
        btnUpArrow.SetRelativePos(new Vector(size.fX - arrowSize.fX, 0, 0));
        btnDnArrow.SetRelativePos(new Vector(size.fX - arrowSize.fX, size.fY - arrowSize.fY ,0));
		if (btnScrollBar != null)
			btnScrollBar.SetRelativePos(new Vector(size.fX - arrowSize.fX,
												   arrowSize.fY,
												   0f));
        vClientSize.fX = v.fX - arrowSize.fX;
        vClientSize.fY = v.fY;
        SetCapacity(nCapacity);
    }
    /**Wywoluje {@link Sprite#SetSize(Vector v)}*/
    void _SetSize(Vector v)
    {
        super.SetSize(v);
    }
    /**Uklada elementy listy w oknie i wyswietla te ktore powinny byc widoczne*/
    void UpdateList()
    {
        int i, j;
        //najpierw chowamy niewidoczne
        Window wndTmp;
        for (i = 0; i < nFirstVisible; i++)
        {
            wndTmp = ((Window)vItems.elementAt(i));
            wndTmp.Hide();
            wndTmp.ProcessMouse(false);
        }
        for (i = nFirstVisible + nCapacity; i < nItemCount;i++)
        {
            wndTmp = ((Window)vItems.elementAt(i));
            wndTmp.Hide();
            wndTmp.ProcessMouse(false);
        }
        if (bAutoShowScroll)
        {
		    if (nItemCount <= nCapacity)
				ShowScroll(false);
		    else
				ShowScroll(true);
        }
        //potem pozycjonujemy widoczne
        ChangeScrollBarPos();
        Vector pos = new Vector(vItemsFirstPos);
        if (nItemCount == 0)
			return;
        for (i = nFirstVisible, j = 0; i < nFirstVisible + nCapacity; i++, j++)
        {
            if (i == nItemCount)
				break;
            wndTmp = ((Window)vItems.elementAt(i));
            wndTmp.SetRelativePos(pos);
            pos.fY = pos.fY + vElemSize.fY;
            if (IsVisible())
				wndTmp.Show();
            else
				wndTmp.Hide();
            wndTmp.ProcessMouse(true);
            wndTmp.MoveTop();
        }
    }
    
    /**Zmienia pozycje ScrollBara*/
    void ChangeScrollBarPos()
    {
        if ( btnScrollBar != null )
		{
            if (nCapacity < nItemCount)
            {
			    Vector size = GetSize();
			    Vector arrowSize = btnUpArrow.GetSize();
			    Vector vScrollSize = btnScrollBar.GetSize();
			    btnScrollBar.SetRelativePos(new Vector(size.fX - arrowSize.fX, arrowSize.fY + ((nFirstVisible) * ((size.fY - (2.0f * arrowSize.fY + vScrollSize.fY))/(nItemCount - nCapacity))),0.0f));   	
                if (bScrollVisible)
                    btnScrollBar.Show();
            }
            else
                btnScrollBar.Hide();
		}
    }
    
    /**Umieszcza element o podanym indeksie w widocznej czesci listy*/
    void EnsureVisible(int nPos)
    {
        if ((nPos < 0)||(nPos >= nItemCount)) return;
        int nLastFirstVisible = nItemCount - nCapacity;
        if (nPos < nFirstVisible) 
        {
            nFirstVisible = nPos;
            UpdateList();
        }
        if (nPos >= nFirstVisible + nCapacity)
        {
            if (nPos > nLastFirstVisible) nFirstVisible = nLastFirstVisible;
            else nFirstVisible = nPos;
            UpdateList();
        }
    }
    
    /**Pokazuje lub ukrywa scrollbar w listboxie*/
    void ShowScroll(boolean bShow)
    {
        //TODO dostosowac do nowego scrolla jak powstanie
        if (bShow)
        {
            bScrollVisible = true;
			if (btnUpArrow != null)
			{
				//LogLine("btnUpArrow.ProcessMouse(true);"+btnUpArrow);
				btnUpArrow.ProcessMouse(true);
				//LogLine("btnUpArrow.Show();");
				btnUpArrow.Show();
			}
			if (btnDnArrow != null)
			{
				//LogLine("btnDnArrow.ProcessMouse(true);");
				btnDnArrow.ProcessMouse(true);
				//LogLine("btnDnArrow.Show();");
				btnDnArrow.Show();
			}
			if ( btnScrollBar != null )
			{
				//LogLine("btnScrollBar.Show();"+btnScrollBar);
				btnScrollBar.Show();
				//LogLine("btnScrollBar.ProcessMouse(true);");
				btnScrollBar.ProcessMouse(true);
			}
        }
        else
        {
            bScrollVisible = false;
			if (btnUpArrow != null)
			{
				//LogLine("btnUpArrow.ProcessMouse(false);");
				btnUpArrow.ProcessMouse(false);
				//LogLine("btnUpArrow.Hide();");
				btnUpArrow.Hide();
			}
			if (btnDnArrow != null)
			{
				//LogLine("btnDnArrow.ProcessMouse(false);");
				btnDnArrow.ProcessMouse(false);
				//LogLine(" btnDnArrow.Hide();");
				btnDnArrow.Hide();
			}
			if ( btnScrollBar != null )
			{
				//LogLine("btnScrollBar.Hide();" + btnScrollBar);
				btnScrollBar.Hide();
				//LogLine("btnScrollBar.ProcessMouse(false);");
				btnScrollBar.ProcessMouse(false);
			}
        }
    }
    /**Usuwa wszystkie elementy listy*/
    void Clear()
    {
        while (nItemCount > 0) RemoveItem(0);
    }
    
    /**Zwraca referencje na przycisk bedacy elementem listy o indeksie nIndex.
     */
    Button GetItem(int nIndex)
    {
        if ((nIndex < 0) || (nIndex >= vItems.size()))
            return null;
        else
            return (Button)vItems.elementAt(nIndex);
    }

    /**Metoda sprawdza czy dany przycisk jest na liscie, sprawdza id przycisku*/
    boolean IsInList(int nBtnId)
    {
        for( int index = 0 ; index < vItems.size() ; index++)
        {
            //LogD("Sprawdzamy element o numerze = " + index + " \n");
            if ( nBtnId == ((Button)vItems.elementAt(index)).nID ) return true;
        }
        return false;
        //return vItems.indexOf(btn_find) > -1 ;
    }
    
    /**Metoda sprawdza czy dany przycisk jest na liscie, sprawdza referencje*/
    boolean IsInList(Button btn_find)
    {
        for( int index = 0 ; index < vItems.size() ; index++)
        {
            if ( btn_find == (Button)vItems.elementAt(index) ) 
                return true;
        }
        return false;
    }
    
        
    public void DumpHeader(ScriptWriter cWriter)
    {
        cWriter.Write("ControlObject(\"" + "UIListBox\"" + ", \"ID_" + String.valueOf(nID) + "\", \"Specific\")");
    }
    
    public void DumpData(ScriptWriter cWriter)
    {
        super.DumpData(cWriter);
        
        
    }
    
    public void Dump(ScriptWriter cWriter)
    {
        cWriter.NewLine();
        DumpHeader(cWriter);        
        cWriter.Write("{");
        cWriter.IncIndent();
        DumpData(cWriter);
        //dzieci nie zapisujemy
        cWriter.DecIndent();
        cWriter.Write("}");
    }
}
