class ProgressBar extends Window 
{
    /**Tekstura tla*/
    int nBkgTex;
    /**Tekstura progressa*/
    int nProgressTex;
    /**Fragment tekstury do wykorzystania*/
    Rect rBkgTex, rProgressTex;
    /**Wektor elementow progressa, jesli progress ciagly to 
     * zawiera tylko jeden element*/
    java.util.Vector vProgress = new java.util.Vector();
    /**Rodzaj progressu, mozliwosci: ciagly lub dyskretny {@link EButtonFlags}*/
    int nProgressType;
    /**Ilosc pozycji progressa, 0 to pierwsza*/
    int nRange = 10;
    /**Odstep pomiedzy kropami dyskretnego progressa*/
    int nSeparator = 1;
    /**Szerokosc kropy dyskretnego progressa*/
    float fDiscreteWidth;
    /**Aktualna pozycja progressa*/
    int nCurPos = 0;
    /**Ustala dane progress bara. 
     * @param data dane potrzebne przy tworzeniu progressu, {@link UIElement}  
     */
    void Init(UIElement data, Module module)
    {
        SetModule(module);
        nProgressType = data.nFlags & EButtonFlags.BTN_TYPE_MASK;
		if (data.sUncheckedTex.length() > 0)
		{
	        nBkgTex  = AddTexture(data.sUncheckedTex, true, true);
		    rBkgTex = data.rUncheckedTex;
	        SetTexture(nBkgTex, rBkgTex);
		}
        nProgressTex = AddTexture(data.sCheckedTex, true, true);
        rProgressTex = data.rCheckedTex;
        SetSize(data.vSize);
        SetRelativePos(data.vPos);
        SetRange(nRange);
        ProcessMouse(false);
        Show();
    }
    /**Ustawia zakres progressa od 0 do nNewRange. Na jego podstawie wyliczana jest szerokosc
     * kropy progressa dyskretnego
     */
    void SetRange(int nNewRange)
    {
        if (vProgress.size() != nNewRange)
        {
            if (nProgressType == EButtonFlags.WND_TYPE_PROGRESS_BAR)
            {//progress ciagly
                if (vProgress.size() == 0)
                {
                    Window progress = (Window)module.CreateObject("Window");
                    AddChildWnd(progress, wndRoot);
                    AddChild(progress);
                    progress.module = module;
                    progress.SetTexture(nProgressTex, rProgressTex);
                    progress.SetRelativePos(new Vector());
                    progress.Show();
                    progress.ProcessMouse(false);
                    vProgress.addElement((Object)progress);
                }
            }
            else
            {
                int i, nPrgCount = vProgress.size();
                Window progress;
                Vector size = GetSize();
                //wyliczam szerokosc kropy
                fDiscreteWidth = (size.fX - (nNewRange - 1)*nSeparator)/ nNewRange;//((float)size.fX * (float)nNewRange - ((float)nNewRange - 1f)*(float)nSeparator)/(float)nNewRange;
                Vector pos = new Vector();
                for (i = 0; i < nPrgCount; i++)
                {
                    ((Window)vProgress.elementAt(i)).Hide();
                }
                
                for (i = nPrgCount; i < nNewRange; i++)
                {
                    progress = (Window)module.CreateObject("Window");
                    AddChildWnd(progress, wndRoot);
                    AddChild(progress);
                    progress.SetModule(module);
                    progress.SetTexture(nProgressTex, rProgressTex);
                    progress.Hide();
                    progress.ProcessMouse(false);
                    vProgress.addElement((Object)progress);
                    
                }
                
                for (i = 0; i < nNewRange; i++)
                {
                    progress = (Window)vProgress.elementAt(i);
                    progress.SetSize(new Vector(fDiscreteWidth, size.fY, 0));
                    progress.SetRelativePos(pos);
                    pos.fX = pos.fX + fDiscreteWidth + nSeparator;
                }

            }
        }
        nRange = nNewRange;
        UpdateProgress();
    }
    
    /**Ustala pozycje na progresie, jesli pozycja nie wychodzi poza zakres, to ustawiana na
     * koniec, a jesli mniejsza od 0 to ustawiana na 0.
     * @param nNewPos nowa pozycja na progresie
     */
    void SetProgressPos(int nNewPos)
    {
        if (nNewPos == nCurPos) return;
        if (nNewPos > nRange) nNewPos = nRange;
        if (nNewPos < 0) nNewPos = 0;
        nCurPos = nNewPos;
        UpdateProgress();
    }
    
    void UpdateProgress()
    {
        Window progress = null;
        if (nProgressType == EButtonFlags.WND_TYPE_PROGRESS_BAR)
        {
            Vector size = GetSize();
            float fTexUnitWidth = (rProgressTex.fRight - rProgressTex.fLeft)/(float)nRange;
            float fUnitWidth = size.fX / (float)nRange;
            progress = (Window)vProgress.elementAt(0);
            if (progress != null)
            {
                Rect rect = new Rect(rProgressTex.fLeft, 
                                             rProgressTex.fTop,
                                             rProgressTex.fLeft + (float)nCurPos * fTexUnitWidth,
                                             rProgressTex.fBottom);
                progress.SetTexture(nProgressTex, rect);
                float P1024 = 1.0f/1024.0f; 
                float P256 = 1.0f/256.0f;
                progress.SetSize(new Vector(fUnitWidth * (float)nCurPos, size.fY, 0));
                progress.Show();
            }
        }
        else
        {
            for (int i = 0; i < nRange; i++)
            {
                progress = (Window)vProgress.elementAt(i);
                if (progress != null)
                {
                    if (i < nCurPos) progress.Show();
                    else progress.Hide();
                }
            }
        }
    }
}
