public class Sprite extends GameObject
{
    /**Tryby odpalania filmu.
     */ 
    static final int _MOVIE_MODE_FITTOSPRITE   = 0; // rozciagniecie filmu na caly sprite
    static final int _MOVIE_MODE_CENTER        = 1; // wycentrowane na srodku sprita, bez skalowania
    
	/**[native]
	 * Czy sprite ma w danej chwili ustawiona teksture.
	 */    
    public native boolean HasTexture();
	
	/**[native]
	 * Czy sprite jest widoczny. Sprite moze nie miec tekstury i byc
	 * widoczny (w takim sensie ze jego dzieci posiadajace teksture sa widoczne).
	 * Jezeli nie jest widoczny to nie jest renderowany i jego dzieci nie sa
	 * renderowane.
	 * 
	 * Uwaga: zwraca jedynie stan flagi widocznosci dla spritea, jego dzieci mimo, ze ich
	 * nie widac maja flage ustawiona na true (przy ukrywaniu flaga sie nie propaguje w 
	 * drzewie, jedynie odcina sie jego galaz przy renderowaniu)
	 */    
    public native boolean IsVisible();
	
    /**[native] Zwraca czy sprite jest rzeczywiscie widoczny uwzgledniajac wplyw 
     * widocznosci rodzicow*/
    public native boolean IsActuallyVisible();
    
	/**[native]
	 * Ukrycie sprite'a oraz automatycznie wszystkich jego dzieci. Sprite nie musi
	 * miec tekstury (wtedy nie sa pokazywane jego dzieci majace teksture).
	 */
    public native void Hide();
	
    /**[native]
     * Ustala, czy ukryty sprite ma dostawac OnFrame, domyslnie true*/
    public native void CallOnFrameWhileHidden(boolean bCall);
    
	/**[native]
	 * Pokazanie sprite'a oraz automatycznie wszystkich jego dzieci. Sprite
	 * nie musi miec tekstury.
	 */
    public native void Show();
	
	/**[native]
	 * Pobranie pozycji ekranowej sprite'a.
	 * @return pozycja ekranowa sprite'a
	 */
    public native Vector GetPos();
	
	/**[native]
	 * ustawienie rotacji 
	 * @param Angle kat obrotu w osi Z (stopnie)
	 */
	public native void SetRotation( float Angle );
	
	/**[native]
	 * pobranie rotacji (stopnie)
	 */
	public native float GetRotation();
	
	/**[native]
	 * Ustalenie pozycji ekranowej sprite'a.
	 * @param v wektor pozycji, wspolrzedne fX i fY sa wspolrzednymi, fZ jest pomijana
	 */
    public native void SetPos(Vector v);
	
	/**[native]
	 * Ustalenie pozycji sprite'a wzgldem jego rodzica (jesli go posiada, jezeli
	 * nie posiada rodzica, nic sie nie dzieje).
	 * @param v wektor pozycji, wspolrzedne fX i fY sa wspolrzednymi, fZ jest pomijana
	 */
    public native void SetRelativePos(Vector v);

	/**[native]
	 * Pobranie wielkosci sprite'a.
	 * @return wielkosc sprite'a
	 */
	public native Vector GetSize();				
	
    public String GetSizeString()
    {
        return GetSize().toString();
    }
    
    public String GetPosString()
    {
        return GetPos().toString();
    }
    
	/**[native]
	 * Ustalenie wielkosci sprite'a.
	 * @param wielkosc sprite'a
	 */
    public native void SetSize(Vector v);
	
	/**[native]
	 * Dodaje sprite'a dziecko dla tego sprite'a. Jezeli parametr byl
	 * wczesniej dzieckiem innego sprite'a, sprite-parametr zmienia rodzica
	 * @param sprite dziecko
	 */
    public native void AddChild(Sprite sprite);
    
    /**[native]
     * Metoda odlacza od rodzica obiekt.
     */
    public native void DetachFromParent();
    
    /**[native]
     * Ustawia podana texture sprite'owi i zwraca numerek identyfikacjyjny
     * textury uzywany do operowania pozniej na niej.<BR>Defaultowo tekstura ma
     * ustawione Clampowanie. Jesli ma byc ono wylaczone trzeba uzyc alternatywnej
     * metody {@link #AddTexture(String textName, boolean bClamp, boolean bFilter)}
     * zamiast tej. Defaultowo filtrowanie textury jest wylaczone.
     * Teksture nalezy pozniej usunac w javie metoda ReleaseTexture - sprite sam
     * nie usuwa zadnych tekstur !!!
     * @param textName nazwa pliku z textura na dysku
     * @return numerek id textury do wykonywania operacji na niej
     */
    public native int AddTexture(String textName);
    
    /**[native]
     * Ustawia podana texture sprite'owi i zwraca numerek identyfikacjyjny
     * textury uzywany do operowania pozniej na niej.
     * Teksture nalezy pozniej usunac w javie metoda ReleaseTexture - sprite sam
     * nie usuwa zadnych tekstur !!!
     * @param textName nazwa pliku z textura na dysku
     * @param bClamp gdy true to textura NIE BEDZIE sie zawijala przy ustawianiu
     * wspolrzednych na krancach bitmapy
     * @param bFilter gdy true to textura jest filtrowana tzn. rozmyje sie i nie bedzie ostra
     * @return numerek id textury do wykonywania operacji na niej
     */
    public native int AddTexture(String textName, boolean bClamp, boolean bFilter);

    /**[native]
     * Jak poprzednia, ale dodatkowo mozna ustawic texutr quality (stale
     * z klasy ETextureQuality 
     */
    public native int AddTexture(String textName, boolean bClamp, boolean bFilter, int iQuality);
    
    /**[native]
     * Ustawienie tekstury tla dla sprite'a. Jezeli sprite mial wczesniej teksture
     * tla, zostanie ona usunieta.
     * @param sBackFileName nazwa textrury z dysku, ktora ma byc w tle wyswietlona
     */
    public native boolean SetBackground( String sBackFileName );
    
    /**[native]
     * Usuwa biezaco ustawiona teksture tla.
     */
    public native void ReleaseBackground();
    
    /**[native]
     * Ustawienie przezroczystosci tekstury tla sprite'a.
     */    
    public native void SetBackgroundAlpha(float fAlpha);
    
    /**[native]
     * Pobranie przezroczystosci tla sprite'a.
     * return wartosc alfy tla (0 - 1), jesli nie ma tekstury tla, zwraca -1
     */
    public native float GetBackgroundAlpha();
    
    /**[native]
     * Pobranie rozmiaru tla wczytanego. Gdy nie ma tla to zwraca 0.
     * @return szerkosc tla
     */
    public native int GetBackgroundWidth();
    
    /**[native]
     * Pobranie rozmiaru tla wczytanego. Gdy nie ma tla to zwraca 0.
     * @return wysokosc tla
     */
    public native int GetBackgroundHeight();

    /**[native]
     * Ustalenie tekstury dla sprite'a (tekstura musi byc wczesniej
     * zaladowana AddTexture()!!! ).
     * @param texNum numerek tekstury otrzymany z AddTexture()
     * @param coords koordynaty tekstury z zakresu <0, 1>
     * @return czy udalo sie ustawienie tekstury, jesli false to cos jest nie tak z texNum
     */
    public native boolean SetTexture(int texNum, Rect coords);

	/**[native]
     * 	Ustalenie tekstury sprite'a (tekstura musi byc wczesniej
     * zaladowana AddTexture()!!! ).
     * @param coords koordynaty tekstury z zakresu <0, 1>
     */
    public native void SetTextureCoords(Rect coords);
	
	/**[native]
	 * Ustawia texture typu FRAME dla sprite'a i mapowanie 
	 * @param texNum numerek textury z AddTexture()
	 * @param texRect koordynaty textury <0, 1>
	 * @param cornerWidth szerokosc naroznika w pixelach
	 * @param cornerHeight wysokosc naroznika w pixelach
	 */
	public native boolean SetFrameTexture( int texNum, Rect texRect, int cornerWidth, int cornerHeight );
	
	/**[native]
	 * Ustala mapowanie textury
	 * @param texRect koordynaty textury <0, 1>
	 * @param cornerWidth szerokosc naroznika w pixelach
	 * @param cornerHeight wysokosc naroznika w pixelach
	 */
	public native void SetFrameTextureCoords( Rect texRect, int cornerWidth, int cornerHeight );

	/**[native]
     * Ustalenie braku tekstury dla tego sprite'a. Od Hide() rozni sie tym,
     * ze ukrywa tylko tego sprite'a a nie wszystkie sprite'y w hierarchii.
     */
	public native void SetNoTexture();
	
	/**[native]
     * Zwolnienie tekstury zaladowanej AddTexture(). Po wywolaniu tej
     * metody nalezy ustawic brak tekstury metoda SetNoTexture() lub nowa
     * teksture metoda SetTexture
     * @param texNum numerek tekstury do usuniecia
     */
    public native void ReleaseTexture(int texNum);
	
    /**[native] Zwraca szerokosc podanej tekstury
     * @param iTexture id tekstury otrzymany z metody AddTexture
     * @return rozmiar w pixelach tekstury
     */
    public native float GetTextureWidth(int iTexture);
    
    /**[native] Zwraca wysokosc podanej tekstury
     * @param iTexture id tekstury otrzymany z metody AddTexture
     * @return rozmiar w pixelach tekstury
     */
    public native float GetTextureHeight(int iTexture);
    
	/**[native]
     * Czy sprite ma przetwarzac komunikaty od klawiatury.
     * @param czy przetwarzac komunikaty
     */
    public native void ProcessKb(boolean bProcess);
	
	/**[native]
     * Czy sprite ma przetwarzac komunikaty od myszy - jezeli rodzic ma false
     * to dzieci nie beda przetwarzac niezaleznie jak maja ustawione.
     * @param czy przetwarzac komunikaty
     */
    public native void ProcessMouse(boolean bProcess);

	/**[native]
     * Czy sprite przetwarza komunikaty od joya., dla sprite'ow to tylko przyciski
     * @return czy przetwarza komunikaty
     */
    public native void ProcessJoy(boolean bProcess);
	
	/**[native]
     * Czy sprite przetwarza komunikaty od klawiatury.
     * @return czy przetwarza komunikaty
     */
    public native boolean ProcessingKb();
	
	/**[native]
     * Czy sprite przetwarza komunikaty od myszy.
     * @return czy przetwarza komunikaty
     */
    public native boolean ProcessingMouse();

	/**[native]
     * Czy sprite przetwarza komunikaty od joya.
     * @return czy przetwarza komunikaty
     */
    public native boolean ProcessingJoy();
	
	/**[native]
     * Przesuwa sprite'a ponad inne sprite'y w danym poziomie hierarchii.
     * Nastepnie jezeli ten sprite jest dodawany jako dziecko do innego sprite'a,
     * doda sie w hierarchii jego dzieci na szczyt.
     */
    public native void MoveTop();
    
    /**[native]
     * Ustawia font do pisania w spritie
     * @param sFontName ID fonta ( pierwszy paramert z GameObject::LoadFont(..) )
     * @return =true gdy font zostal odnaleziony i ustawiony
     */
    public native boolean SetFont(String sFontID );
    
    public native String GetFont();
    
    /**[native]
     * Ustawia skale fonta
     * @param fScale skala (moze byc ujemna)
     */
    public native void SetFontScale(float fScale);
    
    /**[native]
     * Pobiera skale fonta
     * @param fScale skala (moze byc ujemna)
     */
    public native float GetFontScale();
    
    
    /**[native]
     * Ustala przesuniecie w pionie linii tekstu, w efekcie tekst multiline moze byc bardziej zbity lub
     * bardziej rozstrzelony
     */
    public native void SetFontCharactersShiftV(float fShift);
    
    /**[native]
     * Wyswietla tekst w spricie z fontem wczesniej ustawionym poprzez SetFont
     * oraz kolorem ustawionym przez SetTextColor. Gdy nie bylo ustawienia
     * wczesniej to pisze fontem "MS Sans Serif" 12. Gdy nie bylo wczesniej
     * zawolania SetTextColor to ustawiony jest kolor bialy.
     * @param sText tekst do wyswietlenia
     */
    public native void DisplayText(String sText);
    
    /**[native]
     * Oblicza dlugosc i szerokosc tekstu, nie wyswietlajac go
     * @param sText tekst, ktorego rozmiar chcemy znac
     * */
    public native Vector GetTextExtent(String sText);

    /**[native]
     * Podaje dlugosc i szerokosc tekstu, ktory jest obecnie wyswietlany*/
    public native Vector GetTextExtent();
    
    /**[native]
     * Modyfikuje mapowanie liter w foncie (by dopasowac ewentualne
     * przesuniecia - np. zbyt szerokie odstepy w literach)
     * */
	public native void ChangeFontCharMapping(String sFontID, char aChar, boolean bFromLeft, int iMove);
	
    /**[native]
     * Ustawienie koloru textu. Pamietac aby ustawic parametr a na 1 jesli
     * nie chce sie miec przezroczystych napisow.
     */
    public native void SetTextColor( float r, float g, float b, float a );
    
    public void SetTextColorV( Vector color )
    { SetTextColor( color.fX, color.fY, color.fZ, 1 ); }

    /**[native]
     * Ustawienie przezroczystosci sprite'a.
     */
    public native void SetAlpha( float fAlpha );

    /**[native]
     * Pobranie przezroczystosci sprite'a.
     */
    public native float GetAlpha();
    
    /**[native] Ustawianie pozycji tekstu wzgledem sprite'a. Powoduje
     * przelaczenie flagi pozycjonowania tekstu na
     * ESpriteText.NONE {@link ESpriteText}.
     * @param fX pozycja X
     * @param fY pozycja Y
     */
    public native void SetTextPos(float fX, float fY);
    
    /**[native] Pobranie pozycji tekstu wzgledem sprite'a.*/
    public native Vector GetTextPos();
    
    /**[native]
     * Ustawianie centrowania tekstu.
     * @param iFlags znajduja sie w {@link ESpriteText}
     */
    public native void SetTextPosFlags(int iFlags); 
    
    /**[native]
     * Dla sprite'a, ktory ma ustawiony font, wielkosc zwraca tablice kolejnych linii
     * tekstu, wynik zalezny jest tez od wielkosci sprite'a, i od pozycji tekstu
     * @param sText tekst, dla ktorego chcemy poznac rozklad linii.
     */
    public native String [] GetTextLines(String sText);
    
    /**[native]
     * Dla sprite'a, ktory ma ustawiony font, wielkosc zwraca indeksy znakow tekstu,
     * ktore sa pierwszymi slowami w liniach (takie multiline bez wyswietlania)
     * @param sText tekst, dla ktorego chcemy policzyc indeksy slow.
     * @param fLineSize maksymalna wielkosc okienka.
     */
    public native int [] GetTextBreakings(String sText, float fLineSize); 
	
	int aTmpTab[] = new int[512];
	public int [] GetTextBreakingsJava(String sText, float fLineSize)
	{
		CrashLog("!!!!!!!!!!!!!! Sprite.GetTextBreakingsJava() TEJ METODY JUZ NIE MA UZYJ GetTextLines(String sText)\n"); 
		return null;
	}
	
    /**[native] Zwraca pozycje myszy na ekranie.
     * @return pozycja myszy (x - w fX, y - w fZ)
     * @see Vector*/
    public native Vector GetMousePos();
    
    /**[native]
     * Ustawia tego sprite'a jako jedynego, ktory przechwytuje mysz. Komunikaty
     * od myszy odbieraja takze dzieci tego sprite'a. Jezeli
     * jakis inny sprite przechwytuje w tej chwili mysz, ta metoda nic nie zrobi
     * (nalezy wpierw zwolnic mysz dla tamtego sprite'a). 
     */
    public native void CaptureMouse();
    
    /**[native]
     * Pozwala wszystkim sprite'om otrzymywac komunikaty od myszy
     * (po CaptureMouse() na jakims sprite).
     */
    public native void ClearCaptureMouse();
    
    /**[native]
     * Zwraca czy dany sprite moze przetwarzac komunikaty od myszy, jesli capture ma 
     * sprite, ktory nie jest tym sprite'm ani jego rodzicem to zwracamy false wpp true. 
     * (jesli mysz nie jest capturowana to true)
     */
    public native boolean CanProcessCapturedMouse();
    
    /**[native]
     * Ustala, czy maja dochodzic komunikaty o ruchach myszy mimo ze inny sprite ma ustawiony
     * Capture.
     */
    public native void ProcessCapturedMouseMove(boolean bProcess);
    
    /**[native]
     * Powoduje odegranie filmu AVI z podanej sciezki
     * @param sName nazwa pliku AVI na dysku
     * @param nMode tryb odpalenia filmu podany na poczatku pliku Sprite.java
     */
    public native void PlayAVI(String sName, int nMode);
    
    /**[native]
     * Zatrzymuje aktualnie odgrywany avi
     */
    public native void StopAVI();

    /**[native] Ustawienie trybu blendowania sprite'a.
     * Tryb blendowania zadaje sie tak jak w OpenGL. Wprowadzone do szczegolnego
     * mieszania na sprite'ach. Jesli nie wiesz o co chodzi nie uzywaj tej metody.
     * Jesli chcesz sie dowiedziec spytaj Yoghurt lub Kuby.
     * @param eMode kombinacja flag z enumu EBlendingModes
     */
    public native void SetBlendingMode(int eMode);
    
    /**Wyswietla tekst i zmienia rozmiar sprite'a, tak, zeby sie zgadzal z wielkoscia wpisanego tekstu*/
    void DisplayTextFitSize(String sText)
    {
        DisplayTextFitSizeWithMargin(sText, new Vector());
    }
    
    /**Wyswietla tekst i zmienia  rozmiar sprite'a, tak, zeby sie zgadzal z wielkoscia wpisanego 
     * tekstu plus zadany margines (vMargin.fX pionowy, vMargin.fY poziomy), margines
     * dziala dla tekstu wysrodkowanego*/
    void DisplayTextFitSizeWithMargin(String sText, Vector vMargin)
    {
		DisplayText(sText);
        Vector vTextSize = GetTextExtent(sText);
        vTextSize.fX += 2f * vMargin.fX;
        vTextSize.fY += 2f * vMargin.fY;
        SetSize(vTextSize);
        DisplayText(sText);
    }
	
	public native final void MaskSetPosition( float x, float y );
	public native final void MaskSetRotation( float rad );
	public native final void MaskSetScale( float x, float y );
	
	/** dodaje texture maski do sprite'a
	 * jesli tekstura jest 32bitowa kanal alfa brany jest jako maska
	 * jesli tekstura jest 8bitowa to staje sie ona maska
	 * jesli tekstura jest 24bitowa wszystkie wartosci maski ustawione sa na 255 (dodatkowo generowany jest warning w crashlog'u)
	 * @param sTextureName - nazwa textury do zaladowania 
	 */
	public native final int AddMask( String sTextureName );
	
	/** ustawia bierzaca maske sprita 
	 * @param texNum - textura maski
	 * @param coords - mapowanie maski
	 */
	public native final boolean SetMask( int texNum, Rect coords ); 

    /**Funkcje podmiany materialow dla odpowiednio: sprita, tla sprita, tekstu sprita
     */
    public native final void SetSpriteMaterial(String sMaterialFile);
    public native final void SetBackgroundMaterial(String sMaterialFile);
    public native final void SetTextMaterial(String sMaterialFile);
    
	////////////////////////////////////////////////////////////////////
    // standardowe handlery od myszy
    ////////////////////////////////////////////////////////////////////
    /*
    void OnMouseMove(float fMouseX, float fMouseY, int iButtonsFlags)
    {
    }

    void OnMouseEnter(float fMouseX, float fMouseY)
    {
    }
    
    void OnMouseLeave(float fMouseX, float fMouseY)
    {
    }

    void OnLBDown(float fMouseX, float fMouseY)
    {
    }

    void OnLBUp(float fMouseX, float fMouseY)
    {
    }

    void OnRBDown(float fMouseX, float fMouseY)
    {
    }

    void OnRBUp(float fMouseX, float fMouseY)
    {
    }

    void OnLBDownOut(float fMouseX, float fMouseY)
    {
    }

    void OnLBUpOut(float fMouseX, float fMouseY)
    {
    }

    void OnRBDownOut(float fMouseX, float fMouseY)
    {
    }

    void OnRBUpOut(float fMouseX, float fMouseY)
    {
    }

    void OnKeyPressed(int nKey, char nAsci)
    {
    }

    void OnKeyReleased(int nKey, char nAsci)
    {
    }
    
    void OnMovieFinished()
    {
    }
	
    void OnInputJoyButton(int iJoyNum, int iButtonNum, boolean bButtonState)
    {
    }
	
    void OnInputJoyAxis(int iJoyNum, int iAxis, float fAxisValue)
    {
    }
    */

    /** Dziala tak samo jak DisplayText(String sText), jednak
     * dodatkowo tekst jest lamany i skladany w osobne linie
     * @param sText tekst do wyswietlenia, 
     * @param fMaxLineWidth maksymalna dlugosc linii, 
     */
    public int DisplayTextJustify(String sText, float fMaxLineWidth)
    {
        //TODO: odremowac to i uzyc w GetTextJustify sTextToJustify a nie sText
        //bo niestety do tej metody nie moze zostac przekazywany _identyfikator_
        //stringa, ale juz sam gotowy _string_
        //Na razie ja tego nie robie, bo ja "za piec dwunasta" przed
        //buildem dema i nie chce zeby powalily sie inne napisy, by MK
        //String sTextToJustify = Text.Get( sText );
        
        String sTextToDisplay = GetTextJustify( sText, fMaxLineWidth );
        SetTextPosFlags( ESpriteText.MULTILINE );         
        DisplayText( sTextToDisplay );
        
        return m_nLineCnt;
    }
    
    /**
     * Lamie tekst tak, aby w jedna linia nie byla dluzsza niz fMaxLineWidth
     * @param sText tekst do obrobienia, 
     * @param fMaxLineWidth maksymalna dlugosc linii,
     * @return obrobiony (pociety na linie) tekst
     * */
    public String GetTextJustify(String sText, float fMaxLineWidth)
    {
        float fLine		= 0f; // szerokosc biezacej kompletowanej linii
        String sResult  = ""; // rezultat
        int nLineCnt    = 0;  // liczba linii
            
        java.util.StringTokenizer cTokenizer = new java.util.StringTokenizer(sText, " ", true);
		
        while ( cTokenizer.hasMoreTokens() )
        {
            String sNextToken = cTokenizer.nextToken();
            float fWordWidth = GetTextExtent(sNextToken).fX;
                        
			//CrashLog("token <" + sNextToken + "> width:" + fWordWidth + ", line width:" + fLine + "\n");
            if ( (fLine + fWordWidth > fMaxLineWidth) && (fLine > 0f) )
            {
                sResult = sResult + "\n";
                fLine = 0f;
            }
                
            // ten warunek zapewni nam, ze spacje na poczatku nowej linii beda omijane
            if ( (fLine > 0f) || (!sNextToken.equals(" ")) )
                sResult += sNextToken;
            if ( fLine == 0f )
                nLineCnt++;
            fLine += fWordWidth;
        }
        /*
        LogLine("fMaxLineWidth "+String.valueOf(fMaxLineWidth)+
                " nLineCnt "+String.valueOf(nLineCnt)+
                " end result:\n"+sResult);        
        */
        
        // ilosc lini przechowajmy w naszym hamskim staticu
        m_nLineCnt = nLineCnt;
        return sResult;
    }
    // ten static bedzie ustawiany w procce GetTextJustify i bedzie tam
    // ilosc lini jakie zajmuje obrobiony tekst
    static int m_nLineCnt = 1;
    
    void ShowObjectName()
    {
        if (GetSize().fY >= 600)
        {
            SetFont(Fonts.GetDefaultFont());
            SetTextPosFlags(ESpriteText.HORIZ_RIGHT);
            SetTexture(AddTexture("er_t_white.png"), new Rect(0,0,1,1));
            SetAlpha(0.4f);
            DisplayText(GetName() + " " + GetPos().toString() + " " + GetSize().toString());
        }
        
    }
}
