 class Window extends Sprite
{
    /**Menu kontekstowe dla danego okna*/
    ContextMenu wndContextMenu = null;
    /**Aktywne okno modalne - klikniecia myszy obsluguja tylko
     * okna ktore jako wndRoot maja ten wskaznik, jesli null to 
     * wszystkie okna przetwarzaja klikniecia.*/
    static Window wndCurrentModal = null;
    /**Wskaznik na okno rodzica (nadrzedne).*/
    Window wndParent = null;
    /**Wskaznik na szczyt danej hierarchii okien, np dla przycisku
     * dialog w ktorym sie znajduje, standardowo przy tworzeniu
     * ustawiany na <code>this</code>*/
    Window wndRoot = null;
    /**modul-rodzic okna*/
	Module module = null; 
    /**Wektor okien potomnych.*/
    java.util.Vector vChildWindows = new java.util.Vector();
    
    /**Stan przetwarzania myszy przy chowaniu okna*/
    boolean bProcessingMouse = false;
    /**Stan przetwarzania klawiatury przy chowaniu okna*/
    boolean bProcessingKb = false;
    /**Stan przetwarzania joysticka przy chowaniu okna*/
	boolean bProcessingJoy = false;
	
	/**Tekst tooltipa*/
	String sToolTipText = null;
    /**Czas po jakim pojawia sie tooltip*/
	float fToolTipTime = 0.5f;
	/**Czy tooltip aktywny*/
	boolean bToolTipEnabled = false;
    /**Widocznoc tooltipa.*/
    boolean bToolTipVisible = false;
    float fMouseMoveTime = 0.0f;
	
	
	// pjp
	/** Wspolrzedne logiczne - wyznaczaja srodek okna*/
	float m_lxpos = 0.0f;
	float m_lypos = 0.0f;
	/** Wielkosc okna (logiczna)*/
	float m_lwidth = 1.0f;
	float m_lheight = 1.0f;	
	
	boolean m_blpos = false;	
	
	/**
	 * Pozycjonowanie okna wzgledem rodzica (jesli null to wzgledem ekranu)
	 * @param
	 *  - lw, lh - logiczne rozmiary okna (0..1)
	 *  - lx, ly - logiczne polozenie okna w stosunku do okna rodzica	 
	 *  - bpos - jesli true, to (lx, ly) okresla lewy gorny rog okna,
	 *           jesli false, to (lx, ly) okresla srodek okna
	 */
	void SetSizeAndPos(float lw, float lh, float lx, float ly, boolean bpos)
	{
		m_lxpos = lx;
		m_lypos = ly;
		m_lwidth = lw;
		m_lheight = lh;
		m_blpos = bpos;
		
		int width, height;
		
		if (wndParent == null)
		{
			// Pozycjonowanie wzgledem ekranu
			width = GameObject.GetScreenWidth();
			height = GameObject.GetScreenHeight();
		}
		else
		{
			// Pozycjonowanie wzgledem rodzica
			Vector v = wndParent.GetSize();
			width = (int)v.fX;
			height = (int)v.fY;						
		}
		
		
		if (width <= 0 || height <= 0)
			return;
		
		int w = (int)(lw * (float)width);
		int h = (int)(lh * (float)height);				
		
		int px, py;
		
		if (bpos)
		{
			px = (int)(lx * (float)width) - w / 2;
			py = (int)(ly * (float)height) - h / 2;
		}
		else
		{
			px = (int)(lx * (float)width);
			py = (int)(ly * (float)height);
		}							
		
		if (wndParent == null)
			this.SetPos(new Vector(px, py, 0));
		else
			this.SetRelativePos(new Vector(px, py, 0));
		this.SetSize(new Vector(w, h, 0));				
	}
	
	/** Ponowne przeskalowanie oraz pozycjonowanie okien (przydatne po zmianie rozdzielczosci)*/
	void RefreshSetSizeAndPos()
	{
		SetSizeAndPos(m_lwidth, m_lheight, m_lxpos, m_lypos, m_blpos);
		
		Window wnd = null;
		
		int cnt = vChildWindows.size();
		for(int i = 0; i < cnt; i++)
		{
			wnd = (Window)vChildWindows.elementAt(i);
			if (wnd != null)
				wnd.RefreshSetSizeAndPos();			
		}		        
	}
	
	/** Zwraca fizyczne wspolrzedne srodka okna	
	 * @return nowy wektor zawierajacy wspolrzedne srodka okna
	 */
	Vector GetCenterPos()
	{
		// Pobranie wspolrzednych sprajta
		Vector p = ((Sprite)this).GetPos();
		Vector s = ((Sprite)this).GetSize();				
		
		return (new Vector( p.fX + s.fX / 2.0f, p.fY + s.fY / 2.0f, p.fZ + s.fZ / 2.0f));				
	}
		
	/** Zwraca logiczne wspolrzedne podanego punktu wzgledem okna
	 * @param hitpoint - fizyczne wspolrzedne punktu	
	 * @return wektor zawierajacy logiczne wspolrzedne punktu hitpoint w oknie
	 */
	Vector GetLogicalPos(Vector hitpoint)
	{
		Vector p = ((Sprite)this).GetPos();
		Vector s = ((Sprite)this).GetSize();
		
		float x = (hitpoint.fX - p.fX);
		if (s.fX != 0.0f)
			x /= s.fX;
		else
			x = 0.0f;
		
		float y = (hitpoint.fY - p.fY);
		if (s.fY != 0.0f)
			y /= s.fY;
		else
			y = 0.0f;
		
		return (new Vector(x, y, 0.0f));
	}
	
	/** Sprawdzenie, czy punkt nalezy do okna*/
	boolean HitTest(Vector hitpoint)
	{
		// Sprawdzenie, czy punkt nalezy do okna
		Vector p = ((Sprite)this).GetPos();
		Vector s = ((Sprite)this).GetSize();
		
		if (p == null || s == null)
			return false;
		
		return (
				(hitpoint.fX >= p.fX) &&
				(hitpoint.fX < p.fX + s.fX) &&
				(hitpoint.fY >= p.fY) &&
				(hitpoint.fY < p.fY + s.fY)
				);
	}
	
	/**
	 * Ustawienie alfy dla okna oraz okien potomnych
	 */
	void SetAlphaForAll(float _alpha)
	{
		SetAlpha(_alpha);
		
		Window wnd = null;
		
		int cnt = vChildWindows.size();
		for(int i = 0; i < cnt; i++)
		{
			wnd = (Window)vChildWindows.elementAt(i);
			if (wnd != null)
				wnd.SetAlphaForAll(_alpha);			
		}		        
	}
	
	
    void OnCreate()
    {
        SetFont(Fonts.GetDefaultFont());
        wndRoot = this;
    }
        
    /**Dodaje okno potomne do wektora okien potomnych. Jesli 
     * okno ktore dodajemy mialo jakiegos rodzica to jest on zmieniany.
     * @param wndNew okno dodawane jako potomek
     * @param wndRt okno na szczycie danej hierarchii*/
    void AddChildWnd(Window wndNew, Window wndRt)
    {
        if (wndNew.wndParent != null)//to okno mialo juz rodzica
        {
            wndNew.wndParent.RemoveChildWnd(wndNew);
        }
        vChildWindows.addElement((Object)wndNew);
        wndNew.SetRoot(wndRt);
        wndNew.wndParent = this;
    }
    /**Ustawia korzen hierarchii okien dla danego okna i wszystkich
     * jego potomkow. Wazne przy modalnosci okien.*/
    void SetRoot(Window wndRt)
    {
        wndRoot = wndRt;
        int nChildCount = vChildWindows.size();
        Window child;
        for (int i = 0; i < nChildCount; i++)
        {
            child = (Window)vChildWindows.elementAt(i);
            if (child != null) child.SetRoot(wndRt);
        }
    }
    /**Usuwa okno potomne z wektora okien potomnych.
     * @param wnd usuwane okno
     */
    void RemoveChildWnd(Window wnd)
    {
        vChildWindows.removeElement((Object)wnd);
    }
    
	void SetModule(Module windowMod)
	{
		module = windowMod;
	}
    
    /**[native] Wywolywana gdy nacisnieto przycisk.
     * Wola metode OnButton(...)
     * @param btnSender wcisniety przycisk 
     */
    public native void CallOnButton(Button btnSender);
    
    /**[native] Wywolywana gdy byl doubleclick na przycisku.
     * Wola metode OnButtonDoubleClick(...)
     * @param btnSender wcisniety przycisk 
     */
    public native void CallOnButtonDoubleClick(Button btnSender);
    /**[native] Wywolywana gdy kliknieto przycisk prawym klawiszem myszy.
     * Przez klikniecie rozumiemy wcisniecie i puszczenie klawisza
     * nad oknem przycisku. Wola metode OnButtonRB(...).
     * @param btnSender klikniety przycisk
     */
    public native void CallOnButtonRB(Button btnSender);    
    
    /**[native] Wywolywana gdy przycisk zmienia stan na podswietlony
     * (highlighted). Wola metode OnButtonHighlight(...).
     * @param btnSender podswietlany przycisk
     */
    public native void CallOnButtonHighlight(Button btnSender);    
    
    void OnButtonHighlight(Button btnSender)
    {
        if (btnSender == null) return;
        if (wndParent != null) wndParent.CallOnButtonHighlight(btnSender);
    }

    void OnButtonRB(Button btnSender)
    {
        if (btnSender == null) return;
        if (wndParent != null) wndParent.CallOnButtonRB(btnSender);
    }
    
    void OnButton(Button btnSender) 
    {
        //obslugiwac przyciski w dziedziczonych
        if (btnSender == null) return;
        if (wndParent != null) wndParent.CallOnButton(btnSender);
    }
    void OnButtonDoubleClick(Button btnSender) 
    {
        //obslugiwac przyciski w dziedziczonych
        if (btnSender == null) return;
        if (wndParent != null) wndParent.CallOnButtonDoubleClick(btnSender);
    }
    
    /**Ustawia menu kontekstowe dla danego okna (to okno dostanie komunikat o wybraniu
     * opcji z tego menu, jesli menu juz istnialo to jest kasowane, a nowe menu
     * staje sie modalne.
     */
    void SetContextMenu(ContextMenu menu)
    {
        if (wndContextMenu != null) 
        {
            DestroyMenu();
        }
        if (module == null)
        {
            LogLine("Musisz ustawic modul, zanim zawolasz SetContextMenu dla " +this);
            return;
        }
        wndContextMenu = menu;
        if (menu != null)
        {
            AddChildWnd(menu, wndRoot);
            menu.SetModal();
            menu.MoveTop();
            Cursor cursor = module.GetCursor();
            if (cursor != null) cursor.MoveTop();
            //menu.CaptureMouse();
        }
    }
    
    /**Kasuje menu kontekstowe, jesli istnieje dla danego okna*/
    void DestroyMenu()
    {
        if (wndContextMenu == null) return;
        wndContextMenu.Destroy();
        RemoveChildWnd(wndContextMenu);
        this.module.RemoveSprite(wndContextMenu);
        wndContextMenu.delete();
        wndContextMenu = null;
        //wndContextMenu.ClearCaptureMouse();
        ClearModal();
    }
    
    void OnRBUp(float fMouseX, float fMouseY)
	{
 
    }
    
    ContextMenu CreateContextMenu(float fMouseX, float fMouseY)
    {
        return null;   
    }
    
    /**Sprawdza, czy okno ma przetwarzac komunikaty o klawiszach myszy.
     * Nie moze, jesli aktywne jest inne okno modalne.*/
    boolean CanProcessMouseBtns()
    {
        if (Window.wndCurrentModal == null) //zadne okno nie jest modalne
        {
            return true;
        }
        else
        {
            if (Window.wndCurrentModal == wndRoot) return true;
            else return false;
        }
    }
    
    /**Sprawdza, czy okno ma przetwarzac komunikaty o ruchach myszy.
     * Nie moze, jesli aktywne jest inne okno modalne.*/
    boolean CanProcessMouseMove()
    {
        if (Window.wndCurrentModal == null) //zadne okno nie jest modalne
        {
            return true;
        }
        else
        {
            if (Window.wndCurrentModal == wndRoot) return true;
            else return false;
        }
    }
    /**Dane okno stanie sie modalnym, tylko ono dostanie komunikaty o
     * kliknieciach myszy.
     * @return poprzednio aktywne okno*/
    Window SetModal()
    {
        Cursor cursor = null;
        DisableCallMethod("DisplayToolTip");
        if (module != null)
            cursor = module.GetCursor();
        if (cursor != null)
        {
            cursor.DisplayToolTip("");
            bToolTipVisible = false;
        }
        Window wndPrev = Window.wndCurrentModal;
        SetRoot(this);
        Window.wndCurrentModal = this;
        
        return wndPrev;        
    }
    /**Ustawia wndCurrentModal na null, tzn zadne okno nie jest modalne*/
    void ClearModal()
    {
        DisableCallMethod("DisplayToolTip");
        Cursor cursor = null;
        if (module != null)
            cursor = module.GetCursor();
        if (cursor != null)
        {
            cursor.DisplayToolTip("");
            bToolTipVisible = false;
        }
        wndCurrentModal = null;
    }
    
   
    void OnKeyReleased(int nKey, char nAsci)
	{
        /*switch (nKey)
        {
            case EKey._ESCAPE:
                DestroyMenu();
                break;
        }*/
	}
    /**Jesli kursor jest w obrebie okna to zwraca true, chyba ze inne okno jest modalne.*/
    boolean IsCursorInWindow()
    {
        if (!CanProcessMouseMove()) return false;
        Vector vPos = GetPos();
        Vector vSize = GetSize();
        Vector vMousePos = GetMousePos();
        if (((vMousePos.fX >= vPos.fX)&&(vMousePos.fX <= vPos.fX + vSize.fX))
            &&((vMousePos.fZ >= vPos.fY)&&(vMousePos.fZ <= vPos.fY + vSize.fY)))
        {
            return true;
        }
        else 
        {
            return false;
        }
                                 
    }
    /**Ukrywa okno i odcina komunikaty myszy, klawiatury i joysticka*/
    public void Hide()
    {
        DisableCallMethod("DisplayToolTip");
        Cursor cursor = null;
        if (module != null)
            cursor = module.GetCursor();
        if ((cursor != null) && bToolTipVisible)
        {
            cursor.DisplayToolTip("");
            bToolTipVisible = false;
        }
        super.ProcessMouse(false);
        super.ProcessKb(false);
		super.ProcessJoy(false);
        super.Hide();				
    }
    /**Pokazuje okno i przywraca stan przetwarzania myszy i 
     * klawiatury sprzed ukrycia*/
    public void Show()
    {
        super.ProcessMouse(bProcessingMouse);
        super.ProcessKb(bProcessingKb);
		super.ProcessJoy(bProcessingJoy);
        super.Show();				
    }
    
    public void ProcessMouse(boolean bProcess)
    {
        super.ProcessMouse(bProcess);
        bProcessingMouse = bProcess;
    }
    public void ProcessKb(boolean bProcess)
    {
        super.ProcessKb(bProcess);
        bProcessingKb = bProcess;
    }
	
	public void ProcessJoy(boolean bProcess)
	{
		super.ProcessJoy(bProcess);
		bProcessingJoy = bProcess;
	}
	
	/**Ustala tekst tooltipa, ktory ma sie pokazac przy kursorze
	 * gdy ten znajduje sie przez jakis czas nad oknem. Aby zadzialalo
	 * w module nalezy zaimplementowac funkcje GetCursor(). Ponadto po
	 * utworzeniu kursora trzeba dla niego wywolac {@link Cursor#InitToolTip(Module mod, String sTexName, Rect rTexPos, String sFontID, float fR, float fG, float fB, float fA) Cursor.InitToolTip}
	 * Jesli chcemy miec tooltipy w klasie potomnej, to jesli obslugujemy OnMouseMove i OnMouseLeave
	 * musimy w tych funkcjach wywolac ich odpowiedniki z Window, w Button jest to zrobione.
	 * @param sText tekst, ktory ma byc wyswietlany, jesli pusty string lub 
	 * null, to tooltipy sa wylaczane dla danego okna*/
	void SetToolTip(String sText)
	{
        if (sText == null) bToolTipEnabled = false;
		else bToolTipEnabled = (sText.length() > 0);
        sToolTipText = sText;
        if (bToolTipVisible)
            DisplayToolTip();
	}
    
    void OnMouseMove(float fMouseX, float fMouseY, int iButtonsFlags)
    {
        if (bToolTipEnabled)
        {
            Cursor cursor = null;
            if (module != null)
                cursor = module.GetCursor();
            if ((cursor != null) && bToolTipVisible)
            {
                cursor.DisplayToolTip("");
                bToolTipVisible = false;
            }
            DisableCallMethod("DisplayToolTip");
            CallMethodIn("DisplayToolTip", fToolTipTime);
        }
    }
    
    void OnMouseLeave(float fMouseX,float fMouseY)
    {
        if (bToolTipEnabled)
        {
            DisableCallMethod("DisplayToolTip");  
            Cursor cursor = null;
            if (module!=null) 
                cursor = module.GetCursor();
            if ((cursor != null) && bToolTipVisible)
            {
                cursor.DisplayToolTip("");
                bToolTipVisible = false;
            }
        }
    }
                      
    void DisplayToolTip()
    {
        Cursor cursor = null;
        if (module != null)
            cursor = module.GetCursor();
        if (cursor != null)
        {
            if (sToolTipText != null)
            {
                if (sToolTipText.length() > 0)
                {
                    if (IsVisible())
                    {
                        cursor.DisplayToolTip(sToolTipText);
                        bToolTipVisible = true;
                    }
                    else
                        bToolTipVisible = false;
                }
                else
                    bToolTipVisible = false;
            }
            else
                bToolTipVisible = false;
        }
    }
    
    void DeleteToolTip()
    {
        DisableCallMethod("DisplayToolTip");
        Cursor cursor = null;
        if (module != null)
            cursor = module.GetCursor();
        if ( (cursor != null) && (bToolTipVisible) )
        {
            cursor.DisplayToolTip("");
            bToolTipVisible = false;
        }
    }
    
    /** Kasuje wszystkie okna potomne, najlepiej wolac przez 
     * CallMethodIn("DestroyChildren", 0.0f), powinno sie przeciazyc i 
     * kasowac rzeczy utworzone w oknie, potem nalezy wykonac delete() na tym oknie*/
    void Destroy()
    {
		ReleaseTextures();
		
        Window wnd = null;
        while(vChildWindows.size() > 0)
        {
            wnd = (Window)vChildWindows.elementAt(0);
            vChildWindows.removeElementAt(0);
            wnd.Destroy();
            wnd.delete();
            wnd = null;
        }
        DeleteToolTip();
        if (module != null)
            module.RemoveSprite(this);
        else
            GetModule().RemoveSprite(this);//LogLine("BLAD Brak modulu w oknie " + this + " nie mozna detachowac sprite'a, parent " +wndParent); 
    }
    
    public void SetPos(Vector vPos)
    {
        super.SetPos(vPos);
        DisableCallMethod("DisplayToolTip");
        Cursor cursor = null;
        if (module != null)
            cursor = module.GetCursor();
        if ((cursor != null)&&(bToolTipVisible))
        {
            cursor.DisplayToolTip("");
            bToolTipVisible = false;
        }
    }
    
	// Zwalnianie tekstur
	public void ReleaseTextures()
	{
	}
	
	// Przywracanie tekstur
	public void RestoreTextures()
	{
	}
    
    public void Dump(ScriptWriter cWriter)
    {
        cWriter.NewLine();
        DumpHeader(cWriter);        
        cWriter.Write("{");
        cWriter.IncIndent();
        DumpData(cWriter);
        for (int i = 0; i < vChildWindows.size(); i++)
        {
            ((Window)vChildWindows.elementAt(i)).Dump(cWriter);
        }
        cWriter.DecIndent();
        cWriter.Write("}");
    }

    UIElement m_cElementData = null;
    
    public void DumpHeader(ScriptWriter cWriter)
    {
        cWriter.Write("Window(\"" + "UIWindow" /*getClass().getName()*/ + "\"" + ", \"ID_" + "\", \"Specific\")");
    }
    
    public void DumpData(ScriptWriter cWriter)
    {
        Vector vSize = GetSize();
        cWriter.Write("LogicSize(" +    vSize.fX/(float)GetScreenWidth() + "," +
                                        vSize.fY/(float)GetScreenHeight() + ")");
        Vector vPos = GetPos();
        cWriter.Write("LogicPos(" +     vPos.fX/(float)GetScreenWidth() + "," +
                                        vPos.fY/(float)GetScreenHeight() + ")");
        cWriter.Write("Mouse(1)");
        cWriter.Write("Keyboard(1)");
        cWriter.Write("CanFocus(1)");
    }
    
}