public class UISlider extends UIControlObject
{
    
    float m_fRangeMin = 0f;
    float m_fRangeMax = 100f;
    float m_fRange = 100f;
    float m_fPos = 0f;
    UIButton m_btnThumb = null;
    UIWindow m_wndBkg = null;
    
    public void OnInit()
    {
        super.OnInit();
        m_btnThumb = (UIButton)GetElem("ID_THUMB");
        m_btnThumb.OnSizeAndPos();
        m_wndBkg = (UIWindow)GetElem("ID_BKG");
        SetRange(0,100);
    }
    
    Vector vPrevSize = null;
    
    public void SetSize(Vector vSize)
    {
        if (!m_bInited)
            return;
        if (vPrevSize != null)
        {
            if (vPrevSize.IsEqual(vSize))
                return;
        }
        vPrevSize = new Vector(vSize);
        super.SetSize(vSize);
        
        if (m_wndBkg != null)
        {
            m_wndBkg.SetSize(vSize);
            m_wndBkg.SetRelativePos(new Vector());
            m_wndBkg.UpdateLogicSizeAndPos(vSize, m_wndBkg.GetPos());
        }
    }
    
    void SetRange(float fRangeMin, float fRangeMax)
    {
        m_fRangeMin = fRangeMin;
        m_fRangeMax = fRangeMax;
        m_fRange = m_fRangeMax - m_fRangeMin;
        //SetSliderPos(fRangeMin);
    }
    
    /**Ustawia pozycje z zakresu m_fRangeMin, m_fRangeMax*/
    void SetSliderPos(float fPos)
    {
        if (fPos < m_fRangeMin)
            fPos = m_fRangeMin;
        if (fPos > m_fRangeMax)
            fPos = m_fRangeMax;
        m_fPos = fPos;
        Vector vSize = GetSize();
        Vector vThumbPos = m_btnThumb.GetPos();
        vThumbPos.fX = GetPos().fX + (fPos - m_fRangeMin) * (vSize.fX/m_fRange);
        Vector vThumbSize = m_btnThumb.GetSize();
        
         vThumbPos.fX -= vThumbSize.fX * 0.5f;
        
        if(vThumbPos.fX < GetPos().fX)
            vThumbPos.fX = GetPos().fX;
        else if(vThumbPos.fX > GetPos().fX + vSize.fX - vThumbSize.fX)
            vThumbPos.fX = GetPos().fX + vSize.fX - vThumbSize.fX;

        m_btnThumb.SetPos(vThumbPos);
        m_btnThumb.UpdateLogicSizeAndPos(vThumbSize, m_btnThumb.GetPos());
    }
    
    float GetSliderPos()
    {
        return m_fPos;    
    }

    public void OnLBDown(float fX, float fY)
    {
        Command(this, EUIConst.CM_SLIDER_CHANGEPOS);
        SetSliderPos(PosXScreenToSlider(fX));
        SetState("LBDownState");
    }
    
    /**Zamienia pozycje ekranowa na pozycje na sliderze (przeliczona na range)*/
    float PosXScreenToSlider(float fX)
    {
        Vector vSize = GetSize();
        Vector vPos = GetPos();
        float fPos = fX - vPos.fX;
        if (fPos < 0)
            fPos = 0;
        if (fPos > vPos.fX + vSize.fX)
            fPos = vPos.fX + vSize.fX;
        return m_fRangeMin + fPos*(m_fRange/vSize.fX);
    }
    
    public class LBDownState extends State
    {
        public void OnEnter()
        {
            CaptureMouse();    
        }
        
        public void OnMouseMove(float fX, float fY, int nButtons)
        {
            Command(UISlider.this, EUIConst.CM_SLIDER_CHANGEPOS);
            SetSliderPos(PosXScreenToSlider(fX));
        }
    
        public void OnMouseMoveOut(float fX, float fY, int nButtons)
        {
            Command(UISlider.this, EUIConst.CM_SLIDER_CHANGEPOS);
            SetSliderPos(PosXScreenToSlider(fX));
        }
        
        void OnLBUp(float fX, float fY)
        {
            Command(UISlider.this, EUIConst.CM_SLIDER_CHANGEPOS);
            SetState("IdleState");    
        }
    
        void OnLBUpOut(float fX, float fY)
        {
            SetState("IdleState");
        }
        
        void OnMouseLeave(float fX, float fY)
        {
            //SetState("IdleState");
        }
        
    }
    
    public class IdleState extends State
    {
        public void OnEnter()
        {
            ClearCaptureMouse();
        }
    }
}
