abstract public class Action extends GameObject
{
    /** Opoznienie wykonania akcji */
    public float fDelay = 0.0f;//#;Delay of condition evaluation and possible action execution
    
    /** Okresla czy opoznienie wykonania akcji wplywa na kolejne
     * akcje */
    public boolean bDelayWaits = false;//#;When set to true delay will stop any actions from next on action list
    
    /** Warunek wykonania akcji */
    public Condition cCondition = null;//#;If condition will be true action will be executed
    
    /** Referencja na nastepna akcje na liscie */
    Action cNext = null;
    
    // modul do jakiego jest podlaczony action
    Module cModule = null;
    
    public void OnCreate()
    {
        cModule = GetModule();
        bSG = true;
    }
    
    /** Sprawdzenie warunku wykonania akcji.
     * DO POKRYCIA, aby dolozyc specyficzne warunki wywolania akcji */
    boolean CheckCondition()
    {
        return cCondition==null || (cCondition!=null && cCondition.Evaluate());
    }
    
    /** Wykonanie akcji! 
     * Metoda wykonuje akcje rekurencyjnie po polu cNext
     * DO POKRYCIA gdy trzeba zablokowac natychmiastowe wykonanie
     * nastepnej akcji */
    void Invoke()
    {
        InvokeInternal();
        InvokeNext();
    }
    
    /** Metoda sprawdza warunek i wykonuje akcje z uwzglednieniem opoznienia
     */
    void InvokeInternal()
    {
        // sprawdzamy warunek wykonania
        if( CheckCondition() )
        {
            // wykonanie akcji z uwzglednieniem przesuniecia
            if(fDelay == 0.0f)
                InvokeAction();
            else
                CallMethodIn("InvokeAction", fDelay);
        }
    }
    
    /** Metoda bezposrednio wykonujaca akcje.
     */
    abstract void InvokeAction();
    
    /** Wykonanie akcji nastepnika */
    void InvokeNext()
    {
        if(cNext==null) return;
        
        if(bDelayWaits)
            cNext.CallMethodIn("Invoke", fDelay);
        else
            cNext.Invoke();
    }
    
    
    public void OnDestroy()
    {
        if(cCondition != null)
        {
            cCondition.delete();
            cCondition = null;
        }
    }
    
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    // Metoda wczytujaca zapisane dane w Save gracza
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[Action.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
		
        if (cFC.GetID() == ESGChunks._CHK_ACTION)
        {
			super.SGLoadChunk(cFC);
			
            cNext = (Action)cFC.LoadObjectRef();
		}
		
		cFC.delete();
	}
	
    // Metoda zapisujaca dane do Save gracza
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[Action.SGSaveChunk]\n");
		
		FileChunk cFC = NewChunk(ESGChunks._CHK_ACTION, cFCParent);
		
        super.SGSaveChunk(cFC);

        cFC.SaveObjectRef(cNext);
        
		cFC.delete();
	}	
}
