
/** SGObjectRepository jest statyczna klasa sluzaca serializacji obiektow.
 */
public class SGObjectsRepository
{
	static java.util.Vector m_vObjects = null;
	static int m_nCounter = 0;
	
	public static void Reset()
	{
		m_vObjects = new java.util.Vector();
	}
		
	public static int AddObject(ISaveGame ref)
	{	
		FileChunk.Log("[SGObjectsRepository.AddObject] "+ref+"\n");
		
		if (ref == null) return -1;
			
		boolean bFound = false;
		
		int nRet = 0;
		
        for(int i=0; i < m_vObjects.size(); ++i)
        {
            SGSerializableObject obj = (SGSerializableObject)m_vObjects.elementAt(i);
            if (obj.m_oRef== ref)
			{
				bFound = true;
				nRet = i+1;
                break;
			}
        }
		
		if (bFound == false)
		{
			nRet = m_vObjects.size()+1;
			m_vObjects.add(new SGSerializableObject(nRet,ref)); 
			
		}

		return nRet;
	}
		
	public static ISaveGame GetObject(int nID)
	{
		for (int i=0; i < m_vObjects.size(); i++)
		{
			SGSerializableObject soObject = (SGSerializableObject)m_vObjects.elementAt(i);
			
			if (soObject.m_nID == nID)
				return soObject.m_oRef;
		}
		return null;
	}
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
    
    /** Metoda wczytujaca zapisane dane w Save gracza
     */		
	public static void SGLoadChunk(FileChunk cFC)
	{
		FileChunk.Log("[SGObjectsRepository.SGLoadChunk]\n");
		
        if (cFC.GetID() == ESGChunks._CHK_OBJECTS_REPOSITORY)
        {
			int nSize = cFC.LoadInt();
			
			FileChunk.Log("[SGObjectsRepository.SGLoadChunk] - SIZE = " + nSize + "\n");
			
			m_vObjects = new java.util.Vector(nSize);
				
			for (int i=0; i < nSize; i++)
			{
				SGSerializableObject soObject = new SGSerializableObject(cFC.LoadInt(),cFC.LoadString());
				m_vObjects.addElement(soObject);
			}
			
			for (int i=0; i < nSize; i++)
			{
				SGSerializableObject soObject = (SGSerializableObject)m_vObjects.elementAt(i);
				if (soObject == null)
					GameObject.CrashLog("[SGObjectsRepository] soObject == null \n");
				else if (soObject.m_oRef == null)
					GameObject.CrashLog("[SGObjectsRepository] soObject.m_oRef == null \n");
				soObject.m_oRef.SGLoadChunk(cFC);
			}
        }

		FileChunk.Log("[SGObjectsRepository.SGLoadChunk] - END\n");
	}
	
    /** Metoda zapisujaca dane do Save gracza
     */		
	public static void SGSaveChunk(FileChunk cFCParent, boolean bTrialSave)
	{
		FileChunk.Log("[SGObjectsRepository.SGSaveChunk]\n");
		
		if (bTrialSave)
			cFCParent.SetActive(false);
		
        FileChunk cFC = GameObject.NewChunk(ESGChunks._CHK_OBJECTS_REPOSITORY, cFCParent);
				
		int nSize = 0, nTurn = 0, nPos = 0;
		
		if (bTrialSave)
		{
			for (; nSize != m_vObjects.size();nTurn++)
			{
				FileChunk.Log("[SGObjectsRepository.SGSaveChunk] - SIZE = " + nSize + "\n");
				FileChunk.Log("-----------   BEGIN REPOSITORY TURN  <"+nTurn+"> --------------------------\n");
				
				nSize = m_vObjects.size();
				
				cFC.SetActive(false);
				for (int i=nPos; i < nSize; i++)
				{
					SGSerializableObject soObject = (SGSerializableObject)m_vObjects.elementAt(i);
					soObject.m_oRef.SGSaveChunk(cFC);
				}		
				cFC.SetActive(true);
				nPos = nSize;
				
				FileChunk.Log("-----------   END REPOSITORY TURN  <"+nTurn+"> --------------------------\n");
			}
			
			cFCParent.SetActive(true);
						
			return;
		}
		
		nSize = m_vObjects.size();
		
		cFC.SaveInt(nSize);
			
		for (int i=0; i < nSize; i++)
		{
			SGSerializableObject soObject = (SGSerializableObject)m_vObjects.elementAt(i);	
			cFC.SaveInt(soObject.m_nID);
			cFC.SaveString(soObject.m_oRef.getClass().getName());
			FileChunk.Log("[SGObjectsRepository.SGSaveChunk] ID = <"+soObject.m_nID+"> ClassName = <"+soObject.m_oRef.getClass().getName()+">\n");
		}
		
		FileChunk.Log("-----------------------------------------------------------\n");
		
		for (int i=0; i < nSize; i++)
		{
			SGSerializableObject soObject = (SGSerializableObject)m_vObjects.elementAt(i);
			soObject.m_oRef.SGSaveChunk(cFC);
		}		
		        
		cFC.delete(); 		
		
		FileChunk.Log("[SGObjectsRepository.SGSaveChunk] - END\n");
	}	
	
	public static void SGUpdate()
	{
		int nSize = m_vObjects.size();
	
		for (int i=0; i < nSize; i++)
		{
			SGSerializableObject soObject = (SGSerializableObject)m_vObjects.elementAt(i);
			soObject.m_oRef.SGUpdate();
		}			
	}
}

/** SerializableObject jest klasa reprezentujaca wewnetrzny obiekt ObjectRepository
 */
class SGSerializableObject
{
	int	m_nID = -1;
	ISaveGame m_oRef = null;
	
	SGSerializableObject(int nID, ISaveGame oRef)
	{
		m_nID = nID;
		m_oRef = oRef;
	}	
		
	SGSerializableObject(int nID, String sClassName)
	{
		m_nID = nID;
		m_oRef = (ISaveGame)CreateObject(sClassName);
		
		FileChunk.Log("[SGSerializableObject] Creating ID = <"+m_nID+"> ClassName = <"+sClassName+"> \n");
		
		if (m_oRef == null)
			GameObject.CrashLog("[SGSerializableObject] = ERROR! Created object is null\n");				
	}		

	/** Tworzy obiekt na podstawie jego nazwy. Wymagany jest konstruktor bezparametrowy.
	*/
	Object CreateObject(String sClassName)
	{
		Object object = null;

		FileChunk.Log("[SGSerializableObject.CreateObject] ClassName = <" + sClassName + ">\n");
		
		try
		{
			Class classDefinition = Class.forName(sClassName);
			object = classDefinition.newInstance();
		} catch (InstantiationException e) {
		    GameObject.CrashLog("[SGSerializableObject.CreateObject] = ERROR! " + e + "\n");
		} catch (IllegalAccessException e) {
		    GameObject.CrashLog("[SGSerializableObject.CreateObject] = ERROR! " + e + "\n");
		} catch (ClassNotFoundException e) {
		    GameObject.CrashLog("[SGSerializableObject.CreateObject] = ERROR! " + e + "\n");
		}
        
        if (object == null)
            GameObject.CrashLog("[SGSerializableObject.CreateObject] Object is null !\n");
		
		return object;
	}	
}
