public class Tools
{
    /**Losuje jedna z animacji i ja odgrywa.*/
    static int PlayRandomAnim(MeshObject obj, String sAnims[])
    {
        if (sAnims != null)
		{
			int rnd = GameObject.RandomInt(100);
			if (sAnims.length > 0)
			{
				rnd %= sAnims.length;
				
				if (sAnims[rnd] != null)
					return obj.PlayAnim(sAnims[rnd], 1);
			}
		}	
        return -1;
    }
    
    /**Losuje jeden z dzwiekow i go odgrywa przez playsound.
     * @param tablica stringow z nazwami dzwiekow
     * @return czas trwania dzwieku*/
    static float PlayRandomSound(GameObject obj, String sSounds[])
    {
        if (sSounds != null)
		{
			int rnd = GameObject.RandomInt(100);
			if (sSounds.length > 0)
			{
				rnd %= sSounds.length;
				
				if (sSounds[rnd] != null)
                {
					obj.PlaySound(sSounds[rnd]);
                    return obj.GetWaveDuration(sSounds[rnd]);
                }
			}
		}	
        return -1;
    }
    
    /**Losuje jeden z dzwiekow i go odgrywa przez playwave.
     * @param tablica stringow z nazwami dzwiekow
     * @return id dzwieku*/
    static int PlayRandomWave(GameObject obj, String sSounds[])
    {
        if (sSounds != null)
		{
			int rnd = GameObject.RandomInt(100);
			if (sSounds.length > 0)
			{
				rnd %= sSounds.length;
				
				if (sSounds[rnd] != null)
                {
                    return obj.PlayWave(sSounds[rnd]);
                }
			}
		}	
        return -1;
    }
    
    static float g_fDelta = 10f;
    
    /**Pozwala na "przesuwanie" punktu (wektora) przy pomocy klawiszy w trzech osiach
     * @vPos przesuwany punkt
     * @nKeyCode kod klawisza
     * @fDelta wartosc przesuniec, jesli 0, to brana jest wartosc zapisana w Tools.g_fDelta
     * @return przesuniety punkt*/
    static void MovePointByKeys(Vector vPos, int nKeyCode, float fDelta)
    {
        if (fDelta == 0f)
            fDelta = g_fDelta;
        switch(nKeyCode)
        {
            case EKey._NUMPAD4:
                vPos.fX -= fDelta;
                break;
            case EKey._NUMPAD6:
                vPos.fX += fDelta;
                break;
            case EKey._NUMPAD8:
                vPos.fY += fDelta;
                break;
            case EKey._NUMPAD5:
                vPos.fY -= fDelta;
                break;
            case EKey._NUMPAD7:
                vPos.fZ -= fDelta;
                break;
            case EKey._NUMPAD9:
                vPos.fZ += fDelta;
                break;
            case EKey._NUMPAD1:
                g_fDelta *= 0.1f;
                break;
            case EKey._NUMPAD3:
                g_fDelta *= 10;
                break;
            case EKey._NUMPAD2:
                g_fDelta = 1;
                break;
            case EKey._NUMPADENTER:
                GameObject.CrashLog("Tools.g_fDelta " + g_fDelta + "\n");
                break;
                                 
		}
    }
    
    /**Pozwala na przesuwanie obiektu przy pomocy klawiszy w trzech osiach
     * @cObj przesuwany obiekt     
     * @nKeyCode kod klawisza
     * @fDelta wartosc przesuniec, jesli 0, to brana jest wartosc zapisana w Tools.g_fDelta
     * @return przesuniety punkt*/
    static void MoveObjectByKeys(ControlObject cObj, int nKeyCode, float fDelta)
    {
        Vector vNewPos = cObj.GetPosition();
        if (fDelta == 0)
            fDelta = g_fDelta;
        
        switch(nKeyCode)
        {
            case EKey._NUMPAD4:
                vNewPos.fX -= fDelta;
                break;
            case EKey._NUMPAD6:
                vNewPos.fX += fDelta;
                break;
            case EKey._NUMPAD8:
                vNewPos.fY += fDelta;
                break;
            case EKey._NUMPAD5:
                vNewPos.fY -= fDelta;
                break;
            case EKey._NUMPAD7:
                vNewPos.fZ -= fDelta;
                break;
            case EKey._NUMPAD9:
                vNewPos.fZ += fDelta;
                break;
            case EKey._NUMPAD1:
                g_fDelta *= 0.1f;
                break;
            case EKey._NUMPAD3:
                g_fDelta *= 10;
                break;
            case EKey._NUMPAD2:
                g_fDelta = 1;
                break;
            case EKey._NUMPADENTER:
                GameObject.CrashLog("Tools.g_fDelta " + g_fDelta + "\n");
                break;
                
		}
        cObj.SetPosition(vNewPos);
    }
    
    static Sprite g_cInfoSprite = null;
    static Module g_cInfoSpriteModule = null;
    
    final static int _INFO_POS_TOP = ESpriteText.VERT_TOP|ESpriteText.HORIZ_MIDDLE,
                     _INFO_POS_BOTTOM = ESpriteText.VERT_BOTTOM|ESpriteText.HORIZ_MIDDLE,
                     _INFO_POS_CENTER = ESpriteText.VERT_MIDDLE|ESpriteText.HORIZ_MIDDLE;
    
    /**Wyswietla na ekranie sprite'a z tekstem
     * @param sInfo wyswietlany tekst
     * @param mod modul w ktorym bedziemy wyswietlac
     * @nPosition jedna ze stalych Tools._INFO_POS... okreslajaca pozycje tekstu na ekranie
     */
    static void ShowInfo(String sInfo, Module mod, int nPosition)
    {
        if (mod == null)
            return;
        if (mod.fonts == null)
        {
            CrashLogWarning("Tools.ShowInfo: w module brak fontow, tworze wlasne");
            mod.fonts = new Fonts();
            mod.fonts.Init(mod);
        }
        if (g_cInfoSprite == null)
        {
            g_cInfoSprite = (Sprite)mod.CreateObject("Sprite");
            g_cInfoSprite.SetSize(new Vector(mod.GetScreenWidth(), mod.GetScreenHeight(),0));
            g_cInfoSprite.SetPos(new Vector());
            g_cInfoSprite.SetTextPosFlags(nPosition);
            mod.AddSprite(g_cInfoSprite);
            g_cInfoSpriteModule = mod;
            g_cInfoSprite.Show();
            g_cInfoSprite.MoveTop();
            g_cInfoSprite.SetFont(Fonts.FONT_MSSANSSERIF_12);
        }
        else
        {
            if (mod != g_cInfoSpriteModule)
            {
                //g_cInfoSpriteModule.RemoveSprite(g_cInfoSprite);
                //nie mozemy tak zrobic, bo modul moze juz nie istniec :(
                mod.AddSprite(g_cInfoSprite);
                g_cInfoSpriteModule = mod;
                
            }
        }
        if (sInfo == null)
            sInfo = "";
        
        g_cInfoSprite.SetName("InfoSprite");
        g_cInfoSprite.SetTextPosFlags(nPosition);
        g_cInfoSprite.DisplayText(sInfo);
        g_cInfoSprite.CallMethodIn("MoveTop", 0);
    }
    
    static void ShowInfoTop(String sInfo, Module mod)
    {
        ShowInfo(sInfo, mod, _INFO_POS_TOP);
    }
    
    static void ShowInfoBottom(String sInfo, Module mod)
    {
        ShowInfo(sInfo, mod, _INFO_POS_BOTTOM);
    }
    
    static void ShowInfoCenter(String sInfo, Module mod)
    {
        ShowInfo(sInfo, mod, _INFO_POS_CENTER);
    }
    
    static java.util.Vector m_vHelperMeshes = new java.util.Vector();
    static java.util.Vector m_vHelperMeshesOwners = new java.util.Vector();
    
    final static String _HM_MESH_LIGHTBULB = "data/interface/zarowka.3da",
                        _HM_MESH_EXCLAMATION = "data/interface/wykrzyknik.3da",
                        _HM_MESH_FLAG = "data/interface/choragiewka.3da",
                        _HM_MESH_DOT = "data/interface/dot.3da",
                        _HM_MESH_CAMERA = "data/interface/Kamera.3da",
                        _HM_MESH_CUBE = "data/interface/KameraTarget.3da",
                        _HM_MESH_TUNE = "data/interface/Nutka.3DA";
    
    
    /**Dodaje mesha pomocniczego dla obiektu, mozna nim potem manipulowac.
     * Jesli dodamy wiecej niz jeden mesh do danego obiektu, to musimy sami pamietac jego 
     * referencje, poniewaz funkcja GetHelperMesh(..) zwraca tylko jeden element 
     * @param cObj obiekt dla ktorego tworzymy mesha
     * @param sMeshName plik mesha, mozna tez uzyc stalej Tools._HM_MESH_...*/
    static MeshObject AddHelperMesh(ControlObject cObj, String sMeshName)
    {
        if (m_vHelperMeshesOwners.contains(cObj))
        {
            CrashLogWarning("Tools.AddHelperMesh: " + cObj + " ma juz helper mesha!");
        }
        
        MeshObject cMesh = (MeshObject)cObj.CreateObject("MeshObject");
        cMesh.LoadMesh(sMeshName);
        cMesh.ActivateOnModule();
        cMesh.ClearFlag(EFlags._COLLISION);
        cMesh.SetLighting(true);
        
        m_vHelperMeshes.addElement((Object)cMesh);
        m_vHelperMeshesOwners.addElement((Object)cObj);
        return cMesh;
    }
    
    static MeshObject GetHelperMesh(ControlObject cObj)
    {
        MeshObject cRet = null;
        int nIndex = m_vHelperMeshesOwners.indexOf((Object)cObj);
        if (nIndex != -1)
            cRet = (MeshObject)m_vHelperMeshes.elementAt(nIndex);
        return (cRet);
    }
    
    final static String _HM_TEX_RED = "data/interface/er_t_red.png",
                        _HM_TEX_BLUE = "data/interface/er_t_blue.png",
                        _HM_TEX_GREEN = "data/interface/ER_T_Green.png",
                        _HM_TEX_WHITE = "data/interface/ER_T_White.png",
                        _HM_TEX_YELLOW = "data/interface/ER_T_Yellow.png";
        
    /**Ustawia teksture dla helpermesha
     * @param cObj obiekt ktorego helpermesha zmieniamy
     * @param sTex nazwa tekstury, moze byc stala Tools._HM_TEX_..
     * @param cHelperMesh jesli nie jest null, to nie szukamy go w wektorze tylko od razu 
     * ustawiamy, wazne przy wielu helpermeshach*/
    static void SetHelperMeshTexture(ControlObject cObj, String sTex, MeshObject cHelperMesh)
    {
        MeshObject cMesh = null;
        if (cHelperMesh != null)
            cMesh = cHelperMesh;
        else
            cMesh = GetHelperMesh(cObj);
        if (cMesh != null)
        {
            cMesh.SetSkinElement(sTex, 0);
        }
    }
    
    /**Wypisuje do crashloga tekst w sposob : "[ERROR] sText \n"*/
    static void CrashLogError(String sText)
    {
        sText = "[ERROR] " + sText + "\n";
        GameObject.CrashLog(sText);
    }
    
    /**Wypisuje do crashloga tekst w sposob : "[ERROR] sText \n"*/
    static void CrashLogWarning(String sText)
    {
        sText = "[WARNING] " + sText + "\n";
        GameObject.CrashLog(sText);
    }

    /**Tworzy "wskaznik laserowy", ktorego mozna uzywac do testowania kierunkow, pozycji, 
     * sciezek itd. Wizualnie jest to czerwona linia ciagnaca sie od punktu do punktu.
     * @param cModule modul, w ktorym chcemy uzywac lasera
     * @param sColor jedna ze stalych LaserPointer._BEAM_...
     * @param nSize szerokosc promienia (standardowo 2)
     * @param fRange zasieg promienia
     * @param bSingleBeam jesli true, to kazde odwolanie do funkcji rysujacej promien 
     * (metody Point...(...) klasy LaserPointer) kasuje poprzedni promien, wpp promienie 
     * pozostaja az do wygasniecia (okreslony jest on w definicji efektu LaserBeam w klasie
     * FXLaserBeam)
     * @see LaserPointer 
     */
    static LaserPointer CreateLaserPointer(Module cModule, String sColor, int nSize, float fRange, boolean bSingleBeam)
    {
        LaserPointer cLaser = CreateLaserPointer(cModule);
        cLaser.SetSize(nSize);
        cLaser.SetRange(fRange);
        cLaser.m_bSingleBeam = bSingleBeam;
        cLaser.m_sFXBeamName = sColor;
        return cLaser;
    }
    /**j.w. tyle ze ze standardowymi parametrami klasy*/
     
    static LaserPointer CreateLaserPointer(Module cModule)
    {
        LaserPointer cLaser = (LaserPointer)(cModule.CreateObject("LaserPointer"));
        cModule.AddControlObject(cLaser);
        return cLaser;
    }
    
    /**j.w. tyle ze ze standardowymi parametrami klasy plus kolor
     * @param sColor jedna ze stalych LaserPointer._BEAM_...*/
    static LaserPointer CreateLaserPointer(Module cModule, String sColor)
    {
        LaserPointer cLaser = (LaserPointer)(cModule.CreateObject("LaserPointer"));
        cModule.AddControlObject(cLaser);
        cLaser.m_sFXBeamName = sColor;
        return cLaser;
    }
///////////////////////////////////////////////////////////////////////////////
// MATH TOOLS    
///////////////////////////////////////////////////////////////////////////////
    
    final static float RAD2DEG = 57.295779513082320876798154814105f;
    final static float DEG2RAD = 0.017453292519943295769236907684886f;

    /** Interpolacja wg. krzywej smoothstep
     * @param a minimum
     * @param b maksimum
     * @param x x
     * @return 0 dla x < a; 1 dla x > b; 0..1 dla x w przedziale [a, b]
     */
    static float InterpSmoothstep(float a, float b, float x)
    {
        if (x < a) return 0;
        if (x >= b) return 1;
        x = (x-a)/(b-a);
        return (x*x * (3-2*x));
    }    
    
    /** Interpolacja liniowa
     * @param a minimum
     * @param b maksimum
     * @param x x
     * @return 0 dla x < a; 1 dla x > b; 0..1 dla x w przedziale [a, b]
     */
    static float InterpLinear(float a, float b, float x)
    {
        if (x < a) return 0;
        if (x >= b) return 1;
        return (x-a)/(b-a);
    }
    
    /**Bezpieczny acos - wartosc na wejsciu jest sprawdzana*/
    static float acos(float fCos)
    {
        if (fCos < -1)
            fCos = -1;
        if (fCos > 1)
            fCos = 1;
        return (float)Math.acos(fCos);
    }

    /**
     * 1D Intepolacja spline
     * @param x parametr 0..1
     * @param k0 1. wartosc
     * @param k1 2. wartosc
     * @param k2 3. wartosc
     * @param k3 4. wartosc
     * @return wartosc 2. dla x=0, wartosc 3. dla x=1, dla x=(0..1)
     * wartosci interpolowane wedlug krzywej spline opisanej macierza
     * Hermite'a i wezlami z wartosci 1-4 
     */
   
    static float spline(float fT, float fA, float fB, float fC, float fD)
    {
        float   fa, fb, fc;

        fa = 0.5f*(3*fB + fD - fA -3*fC);
        fb = fA + 2*fC - 0.5f*(5*fB + fD);
        fc = 0.5f*(fC - fA);
        
        return (fT*(fT*(fT*fa + fb) + fc) + fB);
    }

    /** Interpolacja spline 3D. Patrz opis spline 1D.
     */
    static Vector spline(float x, Vector v0, Vector v1, Vector v2, Vector v3)
    {
        return new Vector( 
            spline(x, v0.fX, v1.fX, v2.fX, v3.fX),
            spline(x, v0.fY, v1.fY, v2.fY, v3.fY),
            spline(x, v0.fZ, v1.fZ, v2.fZ, v3.fZ) );
    }
    

    /** Zwraca znak podaj wartosci.
     */
    static int sign(float v) { return (v > .0f) ? 1 : ((v < .0f) ? -1 : 0) ; }
    

    protected static Vector l_vVect = new Vector();
    protected static Vector l_vDir = new Vector();
    /** Wykonanie rzutu podanego punktu na prosta zadana przez dwa punkty
     * @param start pierwszy punkt na prostej
     * @param end drugi punkt na prostej
     * @param point punkt rzutowany na prosta
     * @param result (out) wynik operacji
     */
    static void projectPointOnLine(Vector start, Vector end, Vector point, Vector result)
    {
        // oblicz wektor zadany przez te dwa punkty
        l_vVect.Set(end);
        l_vVect.Sub(start);
        float vectLen = l_vVect.Len();
        // znormalizuj go
        l_vVect.Mul(1.f / vectLen);

        // policz kierunek od poczatku wektora do punktu badanego
        l_vDir.Set(point);
        l_vDir.Sub(start);
        float dirLen = l_vDir.Len();
        // i tez znormalizuj
        l_vDir.Mul(1.f / dirLen);
        
        // policz rzut l_vDir na l_vVect
        l_vVect.Mul(l_vDir.Dot(l_vVect) * dirLen);
        // oblicz punkt koncowy rzutu
        l_vVect.Add(start);
        
        result.Set(l_vVect);
    }

    
    /** Znalezienie najblizszego podanemu punktowi punktu na odcinku
     * @param start pierwszy punkt odcinka
     * @param end drugi punkt odcinka
     * @param point sprawdzany punkt
     * @param result (out) wynik operacji
     */
    static void findNearestPointOnSection(Vector start, Vector end, Vector point, Vector result)
    {
        Vector tmpRes;
        projectPointOnLine(start, end, point, l_vVect);
        float startDist2 = start.Distance2(l_vVect),
            endDist2 = end.Distance2(l_vVect),
            len2 = end.Distance2(start);
        if (startDist2 > len2 ||
            endDist2 > len2)
        {
            if (startDist2 < endDist2)
                tmpRes = start;
            else
                tmpRes = end;
        }
        else
            tmpRes = l_vVect;
        result.Set(tmpRes);
    }
    
    static void ShowMeshelemOrientation(MeshObject cMesh, int nElemID)
    {
        MeshObject cGizmo = (MeshObject)cMesh.CreateObject("MeshObject");
        cGizmo.LoadMesh("data/interface/gizmo.3da");
        cMesh.GetModule().AddControlObject(cGizmo);
        cMesh.AttachChild(cGizmo, nElemID);
        cGizmo.FromUpForwardPos(cMesh.GetElementUpVector(nElemID), cMesh.GetElementForwardVector(nElemID), cMesh.GetElementPos(nElemID));
        cGizmo.SetScale(100,100,100);
        cGizmo.HideElement(cGizmo.GetElementID("AXIS_XY"));
        cGizmo.HideElement(cGizmo.GetElementID("AXIS_YZ"));
        cGizmo.HideElement(cGizmo.GetElementID("AXIS_XZ"));
        cGizmo.HideElement(cGizmo.GetElementID("AXIS_X_Rotate"));
        cGizmo.HideElement(cGizmo.GetElementID("AXIS_Y_Rotate"));
        cGizmo.HideElement(cGizmo.GetElementID("AXIS_Z_Rotate"));
        cGizmo.SetName("GIZMO");
    }
    
    static void ShowMeshOrientation(MeshObject cMesh)
    {
        MeshObject cGizmo = (MeshObject)cMesh.CreateObject("MeshObject");
        cGizmo.LoadMesh("data/interface/gizmo.3da");
        cMesh.GetModule().AddControlObject(cGizmo);
        cMesh.AttachChild(cGizmo);
        cGizmo.FromUpForwardPos(cMesh.GetUpVector(), cMesh.GetForwardVector(), cMesh.GetPosition());
        cGizmo.SetScale(100,100,100);
        cGizmo.HideElement(cGizmo.GetElementID("AXIS_XY"));
        cGizmo.HideElement(cGizmo.GetElementID("AXIS_YZ"));
        cGizmo.HideElement(cGizmo.GetElementID("AXIS_XZ"));
        cGizmo.HideElement(cGizmo.GetElementID("AXIS_X_Rotate"));
        cGizmo.HideElement(cGizmo.GetElementID("AXIS_Y_Rotate"));
        cGizmo.HideElement(cGizmo.GetElementID("AXIS_Z_Rotate"));
        cGizmo.SetName("GIZMO");
    }
}
